###################################################################

# Copyright (c) 2004 Alan W. Mahoney. All rights reserved.
# This program is free software.  You may modify and/or
# distribute it under the same terms as Perl itself.
# This copyright notice must remain attached to the file.
#

# Fnord7's The Kingdom of Loathing (www.kingdomofloathing.com) chat bot

package KoLBot::Chatbot;

use warnings;
use strict;

use POSIX qw( floor );

use Carp;

use KoLBot::Chatbot::Rule;

BEGIN
{
  use Exporter ();
  our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $ID);

  $ID          = '$Id: Chatbot.pm 195 2010-10-11 08:51:59Z alan $';

  @ISA         = qw( Exporter KoLBot );
  @EXPORT      = qw( );
  %EXPORT_TAGS = ( );

  @EXPORT_OK   = qw();
}

# our @EXPORT_OK;
# our ($VERSION,$ID);

##########################################
## element access
##########################################

for my $field (qw(
		  rules
		  sm
		  ltr
		  chat_state
		  db_connect
		   )) {
  no strict "refs";
  *$field = sub {
    my $self = shift;
    $self->{$field} = shift if @_;
    return $self->{$field};
  }
}

##########################################
## Constructor:
##########################################

sub new {
  my $invocant = shift;
  my $class = ref($invocant) || $invocant;
#  my $bot = { url_i => 0,
#	      @_ };

  my $bot = KoLBot::->new(rules => {},
			  @_);

  $bot->{config}->{site} ||= ['http://www.kingdomofloathing.com/'];

  bless $bot, $class;
}

##########################################
## Initialization
##########################################

my @types = qw( emote normal
		modwarning modannounce ban 
		betlost betwon buff clanannounce clanbuff 
		clanleader clanstat
		logon private
		sysmessage kmail offer_cancelled offer logged_off
		mister_a
		wanged
		brick
		toilet_paper
		heart_necklace
		cream_pie candy_heart
		rng_essing pvp_attack);

my @channels = qw( 
		   newbie  normal  clan  trade  games  radio 
		   lounge  villa  haiku
		   dev  pvp  foodcourt  mod  kwe  valhalla
		   veteran  harem  hardcore hobopolis );

sub read_rules {
  my $bot = shift;
  my $file = shift;

  open FILE,'<',$file or die "Error opening $file";

  my $inrule = 0;
  my $action = new KoLBot::Chatbot::Action();
  my $rule = {};
  while (<FILE>) {
    /^\s*#/ and next;
    
    if (/^\s*$/ && $inrule) {
#       push @{$bot->rules},
#         new KoLBot::Chatbot::Rule(%$rule,
# 				  actions => [$action]);
      $bot->add_rules(
         new KoLBot::Chatbot::Rule(%$rule,
				   actions => [$action]));
      $action = new KoLBot::Chatbot::Action();
      $rule = {};
      $inrule = 0;
      next;
    }
    if (/^\s*$/ && !$inrule) {
      next;
    }

    $inrule = 1;

    if(/^im?: (.*)/) {
      ($rule->{imessage} = $1) =~ s/\s+/\\s+/g;
      next;
    }
    /^ic:\s+(.*?)\s*$/  and $rule->{ichannel} = $1, next;
    /^it:\s+(.*?)\s*$/  and $rule->{itype}    = $1, next;
    /^lq:\s+(.*?)\s*$/  and $rule->{lqueue}   = $1, next;
    /^a: (.*)/          and $action->{action} = $1, next;
    /^om?: (.*)/ and push(@{$action->{omessage}},$1), next;
    die "unrecognized input line: $_";

  }
  close FILE;
}

sub add_rules {
  my $bot = shift;

  $bot->{rules} ||= {};

  for my $rule (@_) {
    my $match = 0;
    for (@types) {
      if($_ =~ /$rule->{itype}/i) {
	$match = 1;
	if($_ eq 'normal'
	   || $_ eq 'emote'
	   || $_ eq 'modwarning'
	   || $_ eq 'ban'
	   || $_ eq 'modannounce'
	   ) {
	  $bot->{rules}->{$_} ||= {};
	  for my $c (@channels) {
	    $bot->{rules}->{$_}->{$c} ||= [];
	    if($c =~ /$rule->{ichannel}/i) {
	      push @{$bot->{rules}->{$_}->{$c}},$rule;
	    }
	  }
	} else {
	  $bot->{rules}->{$_} ||= [];
	  push @{$bot->{rules}->{$_}},$rule;
	}
      }
    }
    print "Rule matched no known types\n"
      unless $match;
  }
}

##########################################
## Scanners
##########################################

sub new_login_hook {
  my $bot = shift;

  print "AWM DEBUG, new_login_hook\n";
  $bot->enter_chat();
}

sub start_rollover_hook {
  my $bot = shift;

  print "AWM DEBUG, start_rollover_hook\n";
}

sub main {
  my $bot = shift;

  if ($bot->{db_connect}) {
    &{$bot->{db_connect}}($bot);
    $bot->{local_reset} = $bot->{db_connect};
  }

  #
  # GET STATE, a state manager is required.
  #

  $bot->{chat_state} = $bot->sm->get('!','state_hold');

  $bot->chat_state->{time} ||= 0;
  exists($bot->chat_state->{url}) 
    or $bot->chat_state->{url} = $bot->{config}->{site}->[0];

  $bot->{time} = $bot->chat_state->{time};
  $bot->{lasttime} = $bot->chat_state->{time};

  $bot->{url} = $bot->chat_state->{url};

  # actual running

  # should actually only evaluate once
  while(1) {

    $bot->ensure(sub {

      # Hrm.  Repeat every time?
      $bot->get_pwd();

      # get hash
      print "Bot connected, ",$bot->url,"\n";

      # AWM: ?
      $bot->{time} = $bot->chat_state->{time};

      print "Bot running.\n";

      $bot->scan_chat();
    });

    # should never be here
    print "UNKNOWN ERROR $@, waiting\n";
    sleep 3*60;
  }
}

# AWM untested
sub scan_kmail {
  my $bot = shift;

  $bot->{time} = 0;

  while (1==1) {
    my @kmail = $bot->read_kmail();

    # AWM: update time here from most recent kmail?

    exists($bot->config->{heartbeat})
      and &{$bot->config->{heartbeat}}($bot);

    # for my $km (reverse @kmail) {
    # for my $km (sort {$a->id <=> $b->id} @kmail) {
    for my $km (@kmail) {
      &{$bot->config->{process_message}}($bot,$km);
    }

    sleep($bot->config->{delay} || 60);
  }
}

##########################################
## Processors
##########################################

sub scan_hook_pre {
  my $bot = shift;

  defined($bot->config->{scan_hook_pre}) 
    and &{$bot->config->{scan_hook_pre}}($bot);
}

sub scan_hook_mid {
  my $bot = shift;

  $bot->{chat_state} = {time => $bot->{time},
			url => $bot->{url}};
  $bot->sm->set('!','state_hold',$bot->chat_state);
}

sub scan_hook_post {
  my $bot = shift;

  defined($bot->config->{heartbeat})
    and &{$bot->config->{heartbeat}}($bot);
}

sub send_chat_message {
  my $bot = shift;
  my $pid = shift;
  my $text = shift;

  my $message = new KoLBot::ChatMessage (
    user => $pid,
    message => $text,
    type => 'private',);
  print "  out: $message\n";
  $bot->send_message($message);
}

sub process_message {
  my $bot = shift;
  my $message = shift;

  my $myname = $bot->config->{user};

  if($message->{user} eq $bot->config->{user}) {
    # don't respond to yourself!
    return;
  }
  # AWM: grey chat from sickness
  $message->{message} =~ s/<font color=#333333>//gi;
  $message->{message} =~ s/<font color=#666666>//gi;
  $message->{message} =~ s/<font color=gray>//gi;
  $message->{message} =~ s/<font color=.*?>//gi;
  $message->{message} =~ s/<\/font>//gi;
  $message->{message} =~ s/\s+/ /gi;
  $message->{message} =~ s/^\s+//gi;

  if($message->{type} eq 'normal' 
     || $message->{type} eq 'emote'
     || $message->{type} eq 'modwarning'
     || $message->{type} eq 'ban'
     || $message->{type} eq 'modannounce'
     ) {
    for my $rule (@{$bot->{rules}->{$message->{type}}->{lc($message->{channel})}}) {
#       if($message->{type} eq 'normal' && $message->{channel} eq 'clan') {
# 	print "Processing a rule, ",$rule->actions->[0]->action,
# 	" ",$rule->name,"\n";
#       }
      if($rule->process(
			message => $message,
			bot     => $bot,
			limiter => $bot->ltr,
			),
	 ) {
	return;
      }
    }
  } else {
    for my $rule (@{$bot->{rules}->{lc($message->{type})}}) {
      if($rule->process(
			message => $message,
			bot     => $bot,
			limiter => $bot->ltr,
			),
	 ) {
	return;
      }
    }
  }
}


1;
