###################################################################

# Copyright (c) 2004 Alan W. Mahoney. All rights reserved.
# This program is free software.  You may modify and/or
# distribute it under the same terms as Perl itself.
# This copyright notice must remain attached to the file.
#

# Fnord7's The Kingdom of Loathing (www.kingdomofloathing.com) chat bot
#
# eP2 indicates bulk contributions from ePeterso2 (he's contributed in so
# many small ways it's impossible to list them all)

use warnings;
use strict;

package KoLBot;

use POSIX qw( floor );

use KoLBot::KMail;
use KoLBot::Item;
use Time::Local;
use HTTP::Cookies::Netscape;

use Carp;

use Digest::MD5 qw(md5_hex);
use Digest::HMAC_MD5 qw(hmac_md5_hex);

BEGIN
{
  use Exporter ();
  our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $ID);

  $VERSION     = 0.5;
  $ID          = '$Id: KoLBot.pm 195 2010-10-11 08:51:59Z alan $';

  @ISA         = qw(Exporter);
  @EXPORT      = qw( koltime kmailtime next_rollover);
  %EXPORT_TAGS = ( );

  @EXPORT_OK   = qw();
}

our @EXPORT_OK;
our ($VERSION,$ID);

use LogUserAgent;
# use LWP::UserAgent;

use KoLBot::ChatMessage;


##########################################
## element access
##########################################

for my $field (qw( config
		   ua
		   time
		   lasttime
		   items
		   recipes
		   url
		   url_i
		   drunk
		   hash
		   sm
		   dbh
		   )) {
  no strict "refs";
  *$field = sub {
    my $self = shift;
    $self->{$field} = shift if @_;
    return $self->{$field};
  }
}

##########################################
## Constructor:
##########################################

# Needs to be called with a full config hashref.  See examples.

sub new {
  my $invocant = shift;
  my $class = ref($invocant) || $invocant;
  my $bot = { url_i => 0,
	      drunk => 0,
	      @_ };

  exists($bot->{config}) or die "KoLBot NEEDS a config passed";
  $bot->{config}->{agent} ||= "KoLBot.pm/$VERSION";
#  $bot->{config}->{cookie_file} ||= 'cookies.txt';# GAH
  ref($bot->{config}->{site}) eq 'ARRAY' 
    or $bot->{config}->{site} = [$bot->{config}->{site}];
  $bot->{config}->{timeout} ||= 30;
  $bot->{config}->{timeout_login} ||= 180;
  $bot->{url} = $bot->{config}->{site}->[0];

#  $bot->{ua} = LWP::UserAgent->new(agent => $bot->{config}->{agent});
  $bot->{ua} = LogUserAgent->new(agent => $bot->{config}->{agent});
  $bot->{ua}->timeout($bot->{config}->{timeout});

  push @{ $bot->{ua}->requests_redirectable }, 'POST';

  my $cookie_jar;
  if(exists($bot->{config}->{cookie_file})) {
    $cookie_jar = HTTP::Cookies::Netscape->new(
			       file => $bot->{config}->{cookie_file},
					       ignore_discard => 1);
    $cookie_jar->load();
  } else {
    $cookie_jar = {};
  }
 
  $bot->{ua}->cookie_jar($cookie_jar);
  if(exists($bot->{config}->{headers})) {
    $bot->{ua}->default_header(@{$bot->{config}->{headers}});
  }

  bless $bot, $class;
}

##########################################
## The rest
##########################################

sub login {
  my $bot = shift;
  my %params = @_;

  if($bot->{config}->{login_old}) {
    return $bot->login_old(%params);
  }

  return $bot->login_secure(%params);
}

# combines old get_pwd and login scripts.  Attempts to be robust.
sub login_old {
  my $bot = shift;
  my %params = @_;

GET_PWD:
{
  # check whether we are logged in
  print "login: getting lchat " . $bot->url . "\n";
  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'lchat.php';
  my $response = $bot->ua->get($url);

  unless($response->is_success) {
    print "login: lchat HTTP error " . $response->status_line . "\n";
    sleep ($bot->{config}->{timeout})
      unless $response->status_line =~ /^500/; # 500 is timeout.
    redo GET_PWD;
  }

  if($response->content =~ /pwd=([a-f0-9]+)\&/) {
    $bot->{hash} = $1;
    $bot->fix_cookies();    
    return;
  }

  unless ($response->content =~ m{<b><font size=\+2>The Kingdom of Loathing</font></b>}) {
    print "login: lchat content error, " . $response->content . "\n";
    sleep ($bot->{config}->{timeout_login});
    redo GET_PWD;
  }

  # We're not logged into the server in our URL.  Try logging in,
  # there might be a redirect coming.

  # NEVER LOG IN WITHOUT GETTING "not logged in" first.
  # otherwise we might have to end up waiting 15 minutes.

  my $name = $bot->config->{user};
  $name .= '/q'
    if $bot->config->{login_quiet};

  $url = $bot->url . 'login.php';
  print "login: login " . $bot->{url} . " " . $name . "\n";
  $bot->ua->timeout($bot->config->{timeout_login});
  $response = $bot->ua->post($url,
			     [ loggingin => 'Yup.',
			       loginname => $name,
			       password  => $bot->config->{password},
			       ],
			     );
  $bot->ua->timeout($bot->config->{timeout});

  unless($response->is_success) {
    print "login: login HTTP error " . $response->status_line . "\n";
    sleep ($bot->{config}->{timeout_login})
      unless $response->status_line =~ /^500/; # 500 is timeout.
    redo GET_PWD;
  }

  if($response->content =~ /<frame name=charpane src="charpane.php">/) {
    # we're logged in: get pwd.
    redo GET_PWD;
  }

  if($response->content =~ m|<b>Nightly Maintenance...</b>|) {
    print "login: Nightly maintenance.  Waiting.\n";
    sleep($bot->{config}->{timeout_login});
    redo GET_PWD;
  }

  # OUCH
  if($response->content =~ m|Please wait fifteen minutes and try again.|) {
    print "login: Darn 15 minute rule\n";
    sleep(15*60 + 10);
    redo GET_PWD;
  }

  if($response->content =~ m|Redirecting to (www\d?)\.|) {
    die "Would redirect to $1, but can't."
      if exists($bot->{config}->{block_login_redirect});
    print "login: Redirecting to $1\n";
    $bot->{url} ='http://' . $1 . '.kingdomofloathing.com/';
    redo GET_PWD;
  }

  if($response->content =~ m|Please wait a minute|) {
    print "login: One minute wait before logging in again\n";
    sleep(61);
    redo GET_PWD;
  }

  print "login: login content unknown ", $response->content,"\n";
  sleep($bot->{config}->{timeout_login});
  redo GET_PWD;}
}

sub new_login_hook {
  my $bot = shift;
}

# secured login
sub login_secure {
  my $bot = shift;
  my %params = @_;


  my $redirect;
  my $new_login = 0;
GET_PWD:
{
  # check whether we are logged in
  print "login: getting charpane " . $bot->url . "\n";
  my $resp;
  eval {
    $resp = $bot->charpane();
  };

  unless ($@ =~ /^NOTLOGGEDIN/) {
    if($@ ne '') {
      print "login: charpane error: " . $@ . "\n";

      sleep ($bot->{config}->{timeout_login})
	unless $@ =~ /500/;
      redo GET_PWD; # 500 is timeout
    }

    if(lc($resp->{name}) eq lc($bot->config->{user})) {
      $bot->{hash} = $resp->{pwd};
      $bot->fix_cookies();
      if($new_login) {
	$bot->new_login_hook();
      }
      return;
    }
  
    die $resp->{name} . " is already logged in on " . $bot->{url}
      . " with cookie file " . $bot->config->{cookie_file};
  }

  # We're not logged into the server in our URL.  Try logging in,
  # there might be a redirect coming.

  # NEVER LOG IN WITHOUT GETTING "not logged in" first.
  # otherwise we might have to end up waiting 15 minutes.

  print "Get a challenge, login.php\n";
  my $url = $bot->url . 'login.php';
  $bot->ua->timeout($bot->config->{timeout_login});
  my $response = $bot->ua->get($url);
  $bot->ua->timeout($bot->config->{timeout});

  unless($response->is_success) {
    print "login: challenge HTTP error " . $response->status_line . "\n";
    sleep ($bot->{config}->{timeout_login})
      unless $response->status_line =~ /^500/; # 500 is timeout.
    redo GET_PWD;
  }

  if($response->content =~ m|<b>Nightly Maintenance...</b>|) {
    print "login: Nightly maintenance.  Waiting.\n";
    $bot->{rollover} = 1;
    sleep($bot->{config}->{timeout_login});
    redo GET_PWD;
  }

  redo GET_PWD 
    unless $response->content =~ m|<input type=hidden name=challenge value="([0-9a-f]+)">|; # Get challenge

  # compute a response
  
  my $challenge = $1;
#   my $rsp = hmac_md5_hex($challenge,md5_hex($bot->config->{password}));
  my $rsp = md5_hex(md5_hex($bot->config->{password}) . ':' . $challenge);

  print "C, r: ",$challenge," ",$rsp,"\n";

  # login

  my $name = $bot->config->{user};
  $name .= '/q'
    if $bot->config->{login_quiet};

  $url = $bot->url . 'login.php';
  print "login: login " . $bot->{url} . " " . $name . "\n";
  $bot->ua->timeout($bot->config->{timeout_login});
  if(defined($redirect)) {
    print "GETTING\n";
    $response = $bot->ua->get($redirect);
    undef $redirect;
  } else {
    print "POSTING\n";

    my @args = (loggingin => 'Yup.',
		secure    => 1,
		loginname => $name,
		challenge => $challenge,
		response  => $rsp,
		password  => '',
		);

    # we want to see and follow the redirects ourself.
    @{$bot->{ua}->requests_redirectable} = ('GET','HEAD');
    $response = $bot->ua->post($url,\@args);
    push @{ $bot->{ua}->requests_redirectable }, 'POST';
  }

  $bot->ua->timeout($bot->config->{timeout});
  # fix redirects so we don't forget

  unless($response->is_success) {
    if($response->status_line =~ /^302/) { # Found, there's a redirect.
      $redirect = $response->headers->{location};
      unless($redirect =~ m|(www\d?)\.|) {
	if($redirect eq 'main.html'
	   ||
	   $redirect eq 'game.html'
	   ||
	   $redirect eq 'main_c.html'
	   ||
	   $redirect eq 'game.php'
	   ) { # We did log in
	  if($redirect eq 'main_c.html') {
	    # take us out, scotty
	    $url = $bot->url . 'account.php?action=togglecompact';
	    $response = $bot->ua->head($url);
	  }
	  undef $redirect;
	  redo GET_PWD;
	}
	print "login_secure: All kinds of bad things.  We got a redirect, but where?\n";
	print "  location header is $redirect .\n";
	undef $redirect;
	sleep (180);
	redo GET_PWD;		# Hope it fixes itself
      }

      $bot->{url} ='http://' . $1 . '.kingdomofloathing.com/';
      print "login: Redirecting to $1\n";
      die "no redirect" if $bot->config->{no_redirect};
      redo GET_PWD;
    }

    print "login: login HTTP error " . $response->status_line . "\n";
    sleep ($bot->{config}->{timeout_login})
      unless $response->status_line =~ /^500/; # 500 is timeout.
    redo GET_PWD;
  }

  if($response->content =~ /<frame name=charpane src="charpane.php">/) {
    # we're logged in: get pwd.
    $new_login = 1;
    redo GET_PWD;
  }

  if($response->content =~ m|<b>Nightly Maintenance...</b>|) {
    print "login: Nightly maintenance.  Waiting.\n";
    $bot->{rollover} = 1;
    sleep($bot->{config}->{timeout_login});
    redo GET_PWD;
  }

  # OUCH
  if($response->content =~ m|You may try again in 15 minutes\.|
     ||
     $response->content =~ m|Please wait fifteen minutes and try again\.|) {
    print "login: Darn 15 minute rule\n";
    sleep(15*60 + 10);
    redo GET_PWD;
  }

#   if($response->content =~ m|Redirecting to (www\d?)\.|) {
#     die "Would redirect to $1, but can't."
#       if exists($bot->{config}->{block_login_redirect});
#     print "login: Redirecting to $1\n";
#     $bot->{url} ='http://' . $1 . '.kingdomofloathing.com/';
#     redo GET_PWD;
#   }

  if($response->content =~ m|Please wait a minute|) {
    print "login: One minute wait before logging in again\n";
    sleep(61);
    redo GET_PWD;
  }

  if($response->content =~ m|<b>Login failed.\s+Bad password.</b>|) {
    print "login: bad password, waiting 15 minutes to keep IP safe\n";
    sleep (15*60+10);
    redo GET_PWD;
  }

  if($response->content =~ m|<center><b><font size=\+2>The Kingdom of Loathing</font></b><br>|) {
    print "login: got main page? Grr.\n";
    redo GET_PWD;
  }

  if($response->content =~ m|Whoops -- it looks like you had a recent session open that didn\'t get logged out of properly.|) {
    print "login: other session, waiting 3 minutes\n";
    sleep (3*60+10);
    redo GET_PWD;
  }

  print "login: login content unknown ", $response->content,"\n";
  sleep($bot->{config}->{timeout_login});
  redo GET_PWD;}
}

sub fix_cookies {
  my $bot = shift;

  my $s = $bot->ua->cookie_jar->as_string();
#  my @ss = split(/\n/,$s);
  my @ss = split(/^/,$s);
  for(@ss) {
    next unless /PHPSESSID=(.*?);.*(www\d?|dev).(kingdomofloathing|loathing2).com/i;
    my $key = 'PHPSESSID';
    my $val = $1;
    my $path = '/';
    my $domain = $2.'.'.$3.'.com';
    my $version = 0;
    my $path_spec = 1;
    my $maxage = 147368433;
    my ($port,$secure,$discard);
    $bot->ua->cookie_jar->set_cookie($version,$key,$val,$path,$domain,$port,
				     $path_spec,$secure,$maxage,$discard);
  }

  $bot->ua->cookie_jar->save();
}

sub rotate {
  my $bot = shift;

  my $t = shift @{$bot->config->{site}};
  push @{$bot->config->{site}},$t;
  $bot->{url} = $bot->config->{site}->[0];
}

sub get_pwd {
  my $bot = shift;

  my $resp = $bot->charpane();

  if(lc($resp->{name}) eq lc($bot->config->{user})) {
    $bot->{hash} = $resp->{pwd};
    $bot->fix_cookies();    
    return;
  }

#  print "AWM DEBUG name = ", $resp->{name}, " user = " , $bot->config->{user},"\n";
  
  die $resp->{name} . " is already logged in on " . $bot->{url}
    . " with cookie file " . $bot->config->{cookie_file};

}


sub netscape_cookie {
  my $bot = shift;

  # now, set the cookies to not be deleted, so we can use them with mozilla.
  my $s = $bot->ua->cookie_jar->as_string();
  my @ss = split(/^/,$s);
  for(@ss) {
    next unless /PHPSESSID=(.*?);.*(www\d?|dev).(kingdomofloathing|loathing2).com/;
    next unless  'http://' . $2.'.'.$3.'.com/' eq $bot->url;
    my $key = 'PHPSESSID';
    my $val = $1;
    my $domain = $2.'.'.$3.'.com';
    my $exptime = 1242859979;
    return "$domain\tFALSE\t/\tFALSE\t$exptime\tPHPSESSID\t$val";
  }
  return undef;
}

sub logout {
  my $bot = shift;

  my $url = $bot->url . 'logout.php';
  my $response = $bot->ua->head($url);

  $response->is_success or die "HTTP "
    . $response->status_line;
}

sub check_response {
  my $bot = shift;
  my $response = shift;
  my $from = shift || '';

  if($bot->{config}->{log_requests}) {
    print "HTTP request, $from\n";
  }

#  $response->is_success or confess "from $from HTTP " . $response->status_line;
  $response->is_success or die "from $from HTTP " . $response->status_line;

  # Matches on the login page, but not ours
  if($response->content =~ m{<title>The Kingdom of (Loathing|Salad)</title>}) {
    die 'ROLLOVER' if $response->content =~ m|<b>Nightly Maintenance...</b>|;
    die 'NOTLOGGEDIN';
  }

  # not really
  die 'DBSERVERDOWN Too many'
    if $response->content =~ m|mysql_connect\(\): Too many connections|;
}

sub start_rollover_hook {
  my $bot = shift;
}

#
# this works best with reading, or as a big wraparound
#
sub ensure {
  my $bot = shift;
  my $fcn = shift;

  while(1) {
    my @rv1 = eval {
      &$fcn;
    };

    return @rv1 unless $@ ne '';

    warn "ensure: exception $@\n";

    redo if($@ =~ /^HTTP/);

    if($@ =~/^ROLLOVER/) {
      $bot->start_rollover_hook();
    }

    if($@ =~/^NOTLOGGEDIN|^ROLLOVER/) {
      $bot->login();
      redo;
    }
    
    sleep ($bot->{config}->{timeout} || 180);
    # Might be a local problem, possibly DB.
    &{$bot->{local_reset}}($bot,$@) if exists($bot->{local_reset});
    redo;
  }
}

########################################################################
# Chat interfaces
########################################################################

sub enter_chat {
  my $bot = shift;

  my $url = $bot->url . 'lchat.php';
  my $response = $bot->ua->head($url);

  $response->is_success or die "HTTP "
    . $response->status_line;
}

## AWM: this will rollover far too often, {time} isn't well set without
## Chatbot.pm
sub get_chat {
  my $bot = shift;

  my $messages = [];

  $bot->{time} ||= 0;
  if($bot->{time} < $bot->{lasttime}) {
    &{$bot->{rollover_fcn}}($bot) if exists($bot->{rollover_fcn});
    $bot->{time} = 0;
  }

  $bot->{lasttime} = $bot->{time};

KLUDGE:

  my $url = $bot->url . 'newchatmessages.php?lasttime='
    . ($bot->lasttime || 0);
  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP "
    . $response->status_line;

  my $content = $response->content;

    goto KLUDGE
      if ($content =~/Could not open game database/);

  if ($content eq '') {
    die "NOTLOGGEDIN";
  }

  $bot->check_response($response,'get_chat');

  $content =~ s/class=nounder//g;
  $content =~ s/\s+/ /g;

  # first fix for red ban messages---keep all code on one line
  $content =~ s/<br><\/b><\/font>/<br>/g;
  # with orange ones need this too
  $content =~ s/<br><\/font>/<br>/g;
  $content =~ s/<br \/>//g;
  $content =~ s/&quot;/\"/g;
# move these to after pattern matching to isolate HTML
#  $content =~ s/&lt;/</g;
#  $content =~ s/&gt;/>/g;
#  $content =~ s/\&amp;/\&/g;
  $content =~ s/<font color=red><b>V<\/b><\/font>/V/g;
  $content =~ s/<font color=red><b>v<\/b><\/font>/v/g;
  $content =~ s/<!--<font color=red><b>v<\/b><\/font>iva-->//ig;
  $content =~ s/<!--<b>v<\/b>iva-->//ig;
  $content =~ s/<b>v<\/b>/v/g;
  $content =~ s/<b>V<\/b>/V/g;
  $content =~ s/<!--viva-->//ig;
  $content =~ s/<a class=itemlink href='javascript:descitem\((\d+)\)'>(.*?)<\/a>/$2/ig;
  my @lines = split /<br>/i,$content;

  my $l = pop @lines;


  unless ($l =~ /<!--lastseen:(\d+)-->/) {

    print " content = " . $content . "\n";
#    . "\n\$l = $l\n";
    die " Should be a timestamp here!"
  }

  $bot->{time} = $1;

  while (@lines) {
    $_ = shift (@lines);
    my $message = new KoLBot::ChatMessage(
					  channel => '',
					  message => '',
					  type    => 'none',
					  user    => '',
					  user_id => 0,
					  );

    # pname and a_user are a little loose, but I don't think that
    # devious user names could fool them.

    # This is really messy.  What do pros do here?
    my $a = '<a.*?>';
    my $b = '<b>';
    my $i = '<i>';
    my $e_b = '<\/b>';
    my $e_a = '<\/a>';
    my $e_f = '<\/font>';
    my $e_i = '<\/i>';
    my $pname = '\w[\w\ \d_]{0,28}\w';
    my $f_color = '<font color="?(\w+)"?>';
    my $f_green = '<font color="?green"?>';
    my $f_blue = '<font color="?blue"?>';
    my $f_red = '<font color="?red"?>';
    my $f_black = '<font color="?black"?>';
    my $f_blueviolet = '<font color="?blueviolet"?>';
    my $f_orange = '<font color="?orange"?>';
    my $f_none = '<font color=\"none\">';
    my $f = '<font.*?>';
    my $channel = '\[(\w+)\]';

    my $a_user = '<a [^>]*href=\"?\'?showplayer.php\?who=(-?\d+)\"?\'?[^>]*>';
    # will also have a "target=mainpane" and possibly a "style='stuff'"
    my $a_read = '<a [^>]*href=\'?\"?messages.php\"?\'?[^>]*>';

    my $a_offer = '<a target=mainpane\ href=\"makeoffer.php\">';
    my $private = '\ \(private\):';
    my $a_bet = '<a target=mainpane\ href=\'?\"?bet.php\'?\"?>';
#    my $a_bet_nounder = "<a href='bet.php' target=mainpane class=nounder>";
    my $a_clan_hall = '<a target=mainpane\ href=\"clan_hall.php\">';
    my $mwarn = 'Mod Warning';
    my $mannounce = 'Mod Announcement';
    my $smsg = 'System Message';
    my $msfm   = 'New message received from ';
    my $smbuff = '(.*) has been conjured around you by ';
    my $atbuff = 'has played a song \((.*)\) for you\.';
    my $ttbuff = 'has (imbued|fortified|given) you (with|the) (.*)\.';
    my $bet_to_hagnks = "(, which was delivered to Hagnk's)?";
    my $carol = "has sung an Ancient Yuletide Troll Carol for you\.";
    my $refresh = "<!--refresh-->";

    ## Convert clan buff name to stat name
    my %clanbuff2stat = (
			 'Enraged'               => 'Muscle',
			 'Ancestral Disapproval' => 'Mysticality',
			 'Whitened Teeth'        => 'Moxie',
			 );
    ## Convert substat name to stat name
    my %substat2stat = (
			'Beefiness'         => 'Muscle',
			'Fortitude'         => 'Muscle',
			'Muscleboundedness' => 'Muscle',
			'Strengthliness'    => 'Muscle',
			'Strongness'        => 'Muscle',
			
			'Enchantedness'     => 'Mysticality',
			'Magicalness'       => 'Mysticality',
			'Wizardliness'      => 'Mysticality',
			'Mysteriousness'    => 'Mysticality',
			
			'Smarm'             => 'Moxie',
			'Chutzpah'          => 'Moxie',
			'Sarcasm'           => 'Moxie',
			'Roguishness'       => 'Moxie',
			'Cheek'             => 'Moxie',
			);

    if (/$f_green$channel$e_f/) {
      $message->{channel} = $1;
      s/$f_green$channel$e_f//;
    }

    if (/who=(\d+)\'>$f_green$b($pname)$e_b$e_f$e_a\ logged\ on.$e_f$/) {
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'logon';
      $message->{message} = '';
    } elsif (/^($f$e_f)? ?$b$a_user$f_black($pname)$e_f$e_b$e_a:\ (.*)/s) {
      $message->{user_id} = $2;
      $message->{user} = $3;
      $message->{type} = 'normal';
      $message->{message} = $4;
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^($f$e_f)? ?$f$e_f$b$a_user($pname)$e_b$e_a:\ (.*)/s) {
      $message->{user_id} = $2;
      $message->{user} = $3;
      $message->{type} = 'normal';
      $message->{message} = $4;
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^($f$e_f)? ?$b$i$a_user$f_color($pname)$e_b$e_f$e_a(\ (.*)$e_i)?/s) {
      $message->{color} = $3;
      $message->{user_id} = $2;
      $message->{user} = $4;
      $message->{type} = 'emote';
      $message->{message} = $6;
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^ ?$a_user$f_blue$b($pname)$private$e_b$e_a\ (.*)$e_f$/s) {
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'private';
      $message->{message} = $3;
    } elsif (/^ ?$b$a_user$f_color($pname)$e_f$e_b$e_a:\ (.*)/s) {
      # Normal message from Mr Skullhead/Jick
      $message->{user_id} = $1;
      $message->{color} = $2;
      $message->{user} = $3;
      $message->{type} = 'normal';
      $message->{message} = $4;
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^ ?$f_red ?$b$a_user$f_red$mwarn$e_f$e_b$e_a:\ +($pname)\ has been banned from the chat \((.*)\)/) {
      $message->{user_id} = $1;	# Who banned, number
      $message->{type} = 'ban';
      $message->{message} = "$2 ($3)"; # Who was banned, how long
      $message->{channel} ||= $bot->config->{channel},
    } elsif (/^ ?$f_red ?$b$a_user$f_red$mwarn$e_f$e_b$e_a:(.*)/) {
      $message->{user_id} = $1;	# Who banned, number
      $message->{type} = 'modwarning';
      $message->{message} = $2; # message
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^ ?$f_green ?$b$a_user$f_green$mannounce$e_f$e_b$e_a:(.*)/) {
      $message->{user_id} = $1;	# Who banned, number
      $message->{type} = 'modannounce';
      $message->{message} = $2; # message
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^ ?$f_red$b$b$a_user$f_red$smsg$e_f$e_b$e_a:\ (.*)/s) {
      $message->{user_id} = $1; # ID , not name
      $message->{type} = 'sysmessage';
      $message->{message} = $2;
      $message->{channel} ||= $bot->config->{channel};
    } elsif (/^ ?$a_read$f_green$msfm$a_user$f_green($pname)$e_f$e_a\./) {
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'kmail';
      $message->{message} = '';
    } elsif (/^ ?$f_green$smbuff$a_user$b$f_green($pname)$e_f$e_b$e_a$e_f/) {
      $message->{message} = $1;
      $message->{user_id} = $2;
      $message->{user} = $3;
      $message->{message} =~ s/&ntilde;/n/;
      $message->{type} = 'buff';
    } elsif (/^ ?$f_green$a_user$b$f_green($pname)$e_f$e_b$e_a $atbuff$e_f/) {
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = $3;
      $message->{type} = 'buff';
    } elsif (/^ ?$f_green$a_user$b$f_green($pname)$e_f$e_b$e_a $ttbuff$e_f/) {
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = $5;
      $message->{type} = 'buff';
    } elsif (/^ ?$f_green$a_user($pname)$e_a $carol$refresh$e_f/) {
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = 'An Ancient Yuletide Carol';
      $message->{type} = 'buff';

    } elsif (/^ ?$a_offer$f_green($pname) has cancelled a trade offer to you.$e_f$e_a/) {
      $message->{message} = "";
      $message->{type} = 'offer_cancelled';
      $message->{user} = $1;
    } elsif (/^ ?$a_offer$f_green($pname) has offered to sell you (\d+) of item (.+) for (\d+) Meat.$e_f$e_a/) {
      $message->{message} = "$2\@$3 for $4 ";
      $message->{type} = 'offer';
      $message->{user} = $1;
    } elsif (/^ ?$a_offer$f_green($pname) has proposed a trade offer to you.$e_f$e_a/) {
      $message->{message} = "$2\@$3 for $4 ";
      $message->{type} = 'offer';
      $message->{user} = $1;
    } elsif (/^ ?$f_green$a_user$f_green$b($pname)$e_b$e_f$e_a logged off\.$e_f/) {
      $message->{message} = "";
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'logged_off';
    } elsif (/^ ?$f_green$a_user$b$f_green($pname)$e_f$e_b$e_a has given you (\d+) boxes of sunshine/) {
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'mister_a';
      $message->{message} = "The Smile of Mister A ($3)";
    } elsif (/^ ?$f_green$a_user$b$f_green($pname)$e_f$e_b$e_a has given you a box of sunshine/) {
      $message->{user_id} = $1;
      $message->{user} = $2;
      $message->{type} = 'mister_a';
      $message->{message} = "The Smile of Mister A";
#     } elsif (/^ ?$f_green$a_user$b$f_green($pname)$e_f$e_b$e_a has bestowed the Smile of Mr\. A\. on you\.$e_f/) {
# #      $message->{user_id} = $1;
#       $message->{user} = $2;
#       $message->{type} = 'mister_a';
#       $message->{message} = "The Smile of Mister A";
    } elsif (/^ ?${f_green}$a_user$a_bet$b$f_green(.*)$e_f$e_b$e_a took your (.*) Meat bet, and you won, earning you (.*) Meat$bet_to_hagnks\./) {
      $message->{user} = $2;
      $message->{user_id} = $1;
      $message->{type} = 'betwon';
      $message->{message} = join (" ", $3, $4, $5 ? 'hagnk' : 'inv');
    } elsif (/^ ?${f_green}$a_user$a_bet$b$f_green(.*)$e_f$e_b$e_a took your (.*) Meat bet, and you ${f_red}lost\.$e_f Better luck next time\.$e_a$e_f/) {
      $message->{user} = $2;
      $message->{user_id} = $1;
      $message->{type} = 'betlost';
      $message->{message} = $3;
    } elsif (/^ ?${f_green}You gained (\d+) adventures\' worth of an effect \((.*)\) from a clan purchase\.$e_f$/) {
      $message->{type} = 'clanbuff';
      $message->{user} = '';
      $message->{user_id} = -1;
      $message->{message} = "$1 $2";
    } elsif (/^ ?$a_clan_hall${f_green}A new announcement has been posted in your Clan Hall\.$e_f$e_a$/) {
      $message->{type} = 'clanannounce';
      $message->{user} = '';
      $message->{user_id} = -1;
      $message->{message} = 'A new announcement has been posted in your Clan Hall.';
    } elsif (/^ ?${f_green}You gained (\d+) (.*) from a clan purchase\.$e_f$/) {
      $message->{type} = 'clanstat';
      $message->{user} = '';
      $message->{user_id} = -1;
      $message->{message} = "$1 $2";
    } elsif (/^ ?${f_green}You have been granted leadership of your clan\.$e_f$/) {
      $message->{type} = 'clanleader';
      $message->{user} = '';
      $message->{user_id} = -1;
      $message->{message} = "";
    } elsif (/^ ?${f_green}(.*) has used a WANG on you<!--refresh-->$e_f$/) {
      $message->{type} = 'wanged';
      $message->{user} = $1;
      $message->{message} = "";
    } elsif (/^$a$f_green$a_user$f_green($pname)$e_f$e_a $a${f_green}has busted out your (.*?) window pane with a brick.$e_f$e_a$e_f$e_a$/) {
      $message->{type} = 'brick';
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = "";
    } elsif (/^$f_green$a_user$f_green($pname)$e_f$e_a.*toilet paper/i) {
      $message->{type} = 'toilet_paper';
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = "";
    } elsif (/^$a_read$f_green$a_user($pname)$e_a ${a_read}has sent you a candy heart!/i) {
      $message->{type} = 'candy_heart';
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = "";
    } elsif (/^$f_green$a_user($pname)$e_a has given you the right half/i) {
      $message->{type} = 'heart_necklace';
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = "";
    } elsif (/^$f_green$a_user($pname)$e_a has hit you in the face with a cream pie/i) {
      $message->{type} = 'cream_pie';
      $message->{user_id} = $1;
      $message->{user}    = $2;
      $message->{message} = "";
    } elsif (/^${f_green}You have been (blessed|cursed) by The RNG/i) {
      $message->{type} = 'rng_essing';
      $message->{user_id} = 1000000;
      $message->{user}    = 'The RNG';
      $message->{message} = $1;
    } elsif (@{$messages} && /<i>.*<\/i>$/ ) {
      ## If there is a previous message in the queue
      ## And this appears to be gothy, tack on
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } elsif (@{$messages} && /^\&nbsp;/) {
      ## If there is a previous message in the queue
      ## And this appears to be gothy, tack on
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } elsif (@{$messages} && $messages->[-1]->{channel} eq 'haiku'
	     && /^[^<]*<a.*?><font color=blue>\[link\]$e_f$e_a[^<]+$/) {
      ## If there is a previous haiku channel in the queue
      ## And this appears to be no message, tack on
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } elsif (@{$messages} && $messages->[-1]->{channel} eq 'haiku'
	     && !/</) {
      ## If there is a previous haiku channel in the queue
      ## And this appears to be no message, tack on
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } elsif (@{$messages} && $messages->[-1]->{channel} eq 'haiku'
	     && $messages->[-1]->{type} eq 'emote'
	     && m|</i>$|) {
      ## If there is a previous haiku channel in the queue
      ## And this appears to be no message, tack on
      s|</i>$||;
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } elsif (@{$messages} && $messages->[-1]->{user_id} == 7007) {
      ## Toot Oriole can speak in haiku anywhere
      $messages->[-1]->{message} .= "<br>" . $_;
      next;
    } else {
      warn "Couldn't find the goods in this line!\n"
	. "channel = " . $message->{channel} . "\n($_)\n";
      @$messages and warn "Last message = " . $messages->[-1] . "\n";
      next;
    }
    if(!defined($message->{message})) {
      warn "AWM DEBUG, !defined {message}";
      print $_,"\n";
    } else {
#      if($message->{message} =~ showplayer.php
      $message->{message} =~ s|<a style='color: blue' href='showplayer.php\?who=(\d+)' target=mainpane>&gt;&gt;(\d+)</a>|&gt;&gt;$1|g;

      $message->{message} =~ s|<a style='color: blue' href='showplayer.php\?who=(\d+)' target=mainpane>\(\#(\d+)\)</a>|\(\#$1\)|g;

      $message->{message} =~ s|<a target=_blank href="(.*?)">.*?</a>||g;

      $message->{message} =~ s/&lt;/</g;
      $message->{message} =~ s/&gt;/>/g;
      $message->{message} =~ s/\&amp;/\&/g;
    }
    push @$messages, $message;
  }
  return $messages;
}

sub scan_chat {
  my $bot = shift;

  while (1) {
    my $time = time();

    $bot->scan_hook_pre();		# Allows overriding in derived classes

    my $messages = $bot->get_chat();

    $bot->scan_hook_mid();		# Allows overriding in derived classes

    for(@$messages) {
      &{$bot->config->{process_message}}($bot,$_);
    }

    $bot->scan_hook_post();		# Allows overriding in derived classes

    my $del = $time - time() + ($bot->config->{delay} || 60);

    print "  del = $del\n" if ($del < -15);
    sleep($del) if ($del > 0);
  }
}

sub scan_hook_pre { };
sub scan_hook_mid { };

sub scan_hook_post {
  my $bot = shift;

  defined($bot->config->{heartbeat})
    and &{$bot->config->{heartbeat}}($bot);
}

sub send_message_raw {
  my $bot = shift;
  my $msg = shift;
  my $ret = shift || 0;

  my $url = $bot->url . 'submitnewchat.php?pwd='
    . $bot->hash
    . '&graf=' . $msg;

  if ($ret) {
    my $response = $bot->ua->get($url);
    $response->is_success 
      or die "HTTP " . $response->status_line;
    $bot->check_response($response,'send_message_raw');

    return $response->content;
  } else {
    my $response = $bot->ua->head($url);
    $response->is_success 
      or die "HTTP " . $response->status_line;
    return;
  }
}

sub send_message {
  my $bot = shift;
  my $msg = shift;
  my $ret = shift || 0;

  # I hate checking type
#  $msg->isa('KoLBot::ChatMessage') or die "Not a ChatMessage!";

  defined ($msg->type) or die "send_message with no type!";

  # don't modify message in place:
  my $message = $msg->{message};

  # There should be an escape function around somewhere.
  # FIRST do % signs so they aren't redone.
  $message =~ s/%/%25/g;

  $message =~ s/#/%23/g;
  # $ 24
  $message =~ s/&/%26/g;
  $message =~ s/\+/%2B/g;

  if ($msg->type eq 'emote') {
    $bot->send_message_raw('/' . $msg->{channel}. ' /em ' . $message,
			   $ret);
  } elsif ($msg->type eq 'normal') {
    # AWM talk like a Pirate day.
#    filter_text(\$message);
    $bot->send_message_raw(
       '/' . $msg->channel . ' ' . $message,
			   $ret);
  } elsif ($msg->type eq 'private') {
    my $user = $msg->user;

    # AWM talk like a Pirate day.
#    filter_text(\$message);

    $user =~ s/\ /_/g;
    # AWM Should make this a parameter :)
    if(!int(rand(5)) && $bot->{drunk}) {
      if(int(rand(2))) {
	$bot->send_message_raw('/msg '.$user.' '.$message . ' -tic-',
			       $ret);
      } else {
	$bot->send_message_raw('/msg '.$user.' '.$message . ' -toc-',
			       $ret);
      }
    } else {
      $bot->send_message_raw('/msg '.$user.' '.$message,$ret);
    }
  } else {
    die "unknown type: " . $msg->type;
  }
}

sub curse {
  my $bot = shift;
  my %params = (whichitem => 1649, # Default brick
		@_);

  my $url = $bot->url . 'curse.php';

  my $args =
      [ action          => 'use',
	pwd             => $bot->hash,
	whichitem       => $params{whichitem},
	targetplayer    => $params{user},
	message         => $params{message},
	];

  $bot->ua->max_size(0) unless($params{check_response});
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  $response->is_success or die "HTTP " . $response->status_line;

  return 1 unless($params{check_response});

  $bot->check_response($response,'curse');

  return;
}

sub chat_who {
  my $bot = shift;
  my $channel = shift;

  my $content = $bot->send_message_raw("/who $channel",1);

  # clean up

  my @content = split(/, /,$content);
  my %whos = ();
  for(@content) {
    /<a target=mainpane href=\"showplayer.php\?who=(\d+)\"><font color=(.*?)>(.*?)<\/font><\/a>/ or warn "No match $_";
    my ($id,$color,$name) = ($1,$2,$3);
    $whos{$name} = { id => $id,
			color => $color,
			name => $name };
  }
  
  return %whos;
}

########################################################################
# KMail interfaces
########################################################################


# If check_response is chosen, the return code means something
# returns 1 on success: item known sent
# returns 0 on failure: item known unsent, {error} set to reason.
# Otherwise dies, with some clue as to why, either 
#   HTTP 
#   NOTLOGGEDIN
#   Or don't know.
sub send_kmail {
  my $bot = shift;
  my $kmail = shift;
  my $savecopy = shift || 0;
  my $check_response = shift || 0;

  $bot->{error} = undef;

  # I hate checking type
  ref($kmail) =~ /KoLBot::KMail/ or die "Not a Kmail!, " . ref($kmail);

  my $url = $bot->url . 'sendmessage.php';

  my $args =
      [ action          => 'send',
	pwd             => $bot->hash,
	towho           => $kmail->user,
	contact         => 0,
	message         => $kmail->message || '',
	sendmeat        => $kmail->meat || 0,
	];

  croak 'Items passed to send_kmail must now be array of items'
    if(ref($kmail->items) ne 'ARRAY');
  my $i = 0;
  for(@{$kmail->items}) {
    push @$args, 'howmany'.(($i)+1) ,$_->{number};
    push @$args, 'whichitem'.(($i)+1) ,$_->{id};
    $i++;
  }

  push @{$args},(savecopy => 'on') if($savecopy);

  $bot->ua->max_size(0) unless($check_response);
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  $response->is_success or die "HTTP " . $response->status_line;

  return 1 unless($check_response);

  $bot->check_response($response,'send_kmail');

  $response->content =~ m|<center>Message sent.</center>|i
    and return 1;

  $response->content =~
    /send less than one of something./
    and $bot->{error}="send_kmail: less than one",return 0;

  $response->content =~
    /items cannot be transferred/
    and $bot->{error}="send_kmail: untransferrable item",return 0;

  $response->content =~
    /That player cannot receive Meat or items from other players./
    and $bot->{error}="send_kmail: ronin",return 0;

  $response->content =~
    /You don\'t have enough of one of the items you\'re trying to send./
    and $bot->{error}="send_kmail: don't have that many",return 0;

  $response->content =~
    /You don\'t have that much meat./
    and $bot->{error}="send_kmail: not enough meat" , return 0;

  $response->content =~
    /<td>Invalid PlayerID.<\/td>/
    and $bot->{error}="send_kmail: Invalid PlayerID. = " . $kmail->user , 
      return 0;

  $response->content =~
    /player\'s ignore list./
    and $bot->{error}="send_kmail: Ignored." , return 0;

  $response->content =~
    /<br>You twiddle your thumbs.<p>/
    and $bot->{error}="send_kmail: In combat." , return 0;

  $response->content =~ /action=choice.php/
    and $bot->{error}="send_kmail: Choice adventure." , return 0;

  $response->content =~ m|img src="http://images\.kingdomofloathing\.com/otherimages/main/maintop\.gif" width=350 height=50|i
    and $bot->{error}="send_kmail: Main page" , return 0;

  # don't know what might happen, lets take a look
  die "unknown response content, sendmessage.php\n",$response->content,"\n";
}

sub town_sendgift {
  my $bot = shift;
  my %params = ( whichpackage => 1,
		 items => [],
		 fromwhere => 0,
		 meat => 0,
		 do_it => 1,
		 full_return => 0,
		 @_ );

  if(exists($params{item_id})) {
    die "KoLBot::town_sendgift now reqires an array of Items";
  }

  my $url = $bot->url . 'town_sendgift.php';

  # just get types of packages sendable
  unless($params{do_it}) {
    my $response = $bot->ua->get($url);

    $bot->check_response($response,'town_sendgift');
    my $content = $response->content;
    my $allowed_types = {};
    while($content =~ /<input type=radio name=whichpackage value=(\d+)/g) {
      $allowed_types->{$1} = 1;
    }
    return $allowed_types
      unless $params{full_return};

    # Parse rest
    my $items = [];
    my $hmeat = 0;
    my $hitems = [];
    if($content =~ m{<select name=whichitem1 id='clonesel1'><option value=0>-select an item-</option>(.*?)</select></td>}) {
#      print "AD1\n";
      my $itembits = $1;

      while($itembits =~ m|<option value="?'?(\d+)'?"? descid="?'?(\d+)'?"?>(.*?)( \(([\d,]+)\))?</option>|g) {
	my $id = $1;
	my $descid = $2;
	my $name = $3;
	my $number = $5 || 1;
	$number =~ s/,//g;
	push @$items,{id=>$id,name=> $name,number=>$number,
		      desc => $descid};
      }
    }
       
    if($content =~ m{<select name=hagnks_whichitem1 id='clonesel2'><option value=0>-select an item-</option>(.*?)</select></td>}) {
#      print "AD2\n";
      my $itembits = $1;

      while($itembits =~ m|<option value="?'?(\d+)'?"? descid="?'?(\d+)'?"?>(.*?)( \(([\d,]+)\))?</option>|g) {
	my $id = $1;
	my $descid = $2;
	my $name = $3;
	my $number = $5 || 1;
	$number =~ s/,//g;
	push @$hitems,{id=>$id,name=> $name,number=>$number,
		      desc => $descid};
      }
    }
    if($content =~ /You have ([\d,]+) Meat in Hagnk\'s/) {
      $hmeat = $1;
      $hmeat =~ s/,//g;
    }

    return ($allowed_types,$items,$hmeat,$hitems);
  }


  if(!exists($params{note}) && exists($params{message})) {
    $params{note} = $params{message};
  }
  if(!exists($params{insidenote})) {
    $params{insidenote} = $params{note};
  }

  my $note = $params{note};
  my $insidenote = $params{insidenote};
  $note =~ s/\n/\r\n/g;
  $insidenote =~ s/\n/\r\n/g;

  my $args =
     [ towho           => $params{user},
	contact         => 0,
	note            => $note,
	insidenote      => $insidenote,
	whichpackage    => $params{whichpackage},
       fromwhere        => $params{fromwhere},
	pwd             => $bot->hash,
	action          => 'Yep.',
	sendmeat        => $params{meat},
       ];

  my $i = 0;
  for(@{$params{items}}) {
    push @$args, 'howmany'.(($i)+1) ,$_->{number};
    push @$args, 'whichitem'.(($i)+1) ,$_->{id};
    $i++;
  }

  $bot->ua->max_size(0) unless($params{check_response});
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  $response->is_success or die "HTTP " . $response->status_line;

  return 1 unless($params{check_response});

  $bot->check_response($response,'town_sendgift');

  $response->content =~ m|<td>Package sent.</td>|i
    and return 1;

  $response->content =~ m|his town isn\'t big enough for the both of us|
    and return (0,'too many items in package');

  $response->content =~ m|Invalid package|
    and return (0,'invalid package');

  $response->content =~ m|Invalid player selected|
    and return (0,'invalid player selected');

  return (0,$response->content);
}

# offload into KMail, or per KINGDOM config file
# our $kmail_offset = -5*60*60;  # winter
our $kmail_offset = -7*60*60;
# these UTC
our $rollover_hour = 3;
our $rollover_minute = 30;

sub read_kmail {
  my $bot = shift;
  my $params = { page_limit => 100,
		 box        => 'Inbox',
		 min_id     => 0,
		 dont_die   => 0,
		 no_items   => 0,
		 @_ };

  if($params->{box} !~ /^(Inbox|Outbox|Saved)$/i) {
    die "Bad box type passed to read_kmail";
  }
  my $min_id = $params->{min_id};

  my %kmail = ();

  # Check if there are more pages
  my $page = 1;
  my $page_limit = $params->{page_limit};

  while($page <= $page_limit) {

    $bot->ua->timeout($bot->config->{timeout});
    my $url = $bot->url . 'messages.php?box='
      . $params->{box} .'&begin='
      . $page;
    my $response = $bot->ua->get($url);

    $response->is_success
      or die 'HTTP ' . $response->status_line;

    $bot->check_response($response,'read_kmail');

    die 'unknown response in read_kmail, ',$response->content
      unless $response->content =~ /cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=(blue|green)><b>Messages:/;

    my $content = $response->content;

    last 
      unless ($content =~ m|(<tr><td valign=top>.*?</blockquote></td></tr>)|s);

    while($content =~ m|(<tr><td valign=top>.*?</blockquote></td></tr>)|gs) {
      my $rawmsg = $1;

      my $mail = {};

      $rawmsg =~ m{<input type=checkbox name="sel(\d+)">} 
        or die "no sel part in message";

      $mail->{id} = $1;

      my $from_b = '<td class=small><b>(?:From|To)</b> <a href="showplayer.php\?who=(\d+)">(.*?)</a> \(#\d+\)';
      $rawmsg =~ m{$from_b} or die "no from part in message";

      $mail->{user_id} = $1;
      $mail->{user} = $2;

      my $day = '(?:Friday|Saturday|Sunday|Monday|Tuesday|Wednesday|Thursday)';
      my $month = '(July|August|September|October|November|December|January|February|March|April|May|June)';

      # each is made up of
      my $date = '<br><b>Date:</b> ' . $day . ', ' .  $month . ' (\d\d), (\d\d\d\d), (\d\d):(\d\d)(AM|PM)</b><!--(\d\d)\/(\d\d)\/(\d\d) (\d\d):(\d\d):(\d\d)--><br>';

      unless($rawmsg =~ m{$date}) {
	print $rawmsg;
	die "Can't find date in message.";
      }

      my $months = {'January' => 1,'February' => 2, 'March' => 3, 'April' => 4,
		    'May' => 5, 'June' => 6, 'July' => 7,'August' => 8,
		    'September' => 9,'October'=>10, 'November' => 11,
		    'December' => 12};
      !exists($months->{$1}) and die "no month ",$1;
      my $mth = $months->{$1} - 1;

      my $mtime;
#      my $mtime_old = timegm(0,$5,($4+(($6 eq 'PM')?12:0))%24,$2,$mth,$3-1900)
#	  - $kmail_offset;
      $mtime = timegm($12,$11,$10,$8,$7-1,$9+100)-$kmail_offset;

      $mail->{time} = $mtime;

      $rawmsg =~ m{<blockquote>(.*?)</blockquote></td></tr>}s
	or die "no bulk";
 
      my $bulk = $1;

      # get rid of HTML in messages
      # line breaks
      $bulk =~ s{<br />\n}{\n}g;
      # inlined profile links
      $bulk =~ s{<a class=nounder style='color: blue' href='showplayer.php\?who=\d+' target=mainpane>(.*?)</a>}{$1}g;
      # normal URLs
      $bulk =~ s{<a target=_blank href="[^\"]*"><font color=blue>\[link\]</font></a>}{}g;

      if($bulk =~ m|<span class=small>\(\* interjection denoting surprise\)</span>|) {
	$mail->{brick} = 1;
	$bulk =~ s|<center><img src='http://images.kingdomofloathing.com/adventureimages/\w+\.gif'></center>|\n|;
	$bulk =~ s|<p><center><div style='display: table; border: 2px inset; padding: 10px;'>|\n|;
	$bulk =~ s|</div></center><p><span class=small>|\n|;
      }
      # valentines
      if($bulk =~ m{<center><table><tr><td><img src="http://images\.kingdomofloathing\.com/adventureimages/(\w+)\.gif" width=100 height=100></td><td valign=center>(.*?)</td></tr></table></center>}) {
	my $repl = "$1.gif $2";
	$repl =~ s/<br>/\n/ig;
	$bulk =~ s{<center><table><tr><td><img src="http://images\.kingdomofloathing\.com/adventureimages/(\w+)\.gif" width=100 height=100></td><td valign=center>(.*?)</td></tr></table></center>}{$repl};
      }
      if($bulk =~ m{<center><table><tr><td><img src="http://images\.kingdomofloathing\.com/itemimages/(\w+)\.gif" width=30 height=30></td><td valign=center>(.*?)</td></tr></table></center>}) {
	my $repl = "$1.gif $2";
	$repl =~ s/<br>/\n/ig;
	$bulk =~ s{<center><table><tr><td><img src="http://images\.kingdomofloathing\.com/itemimages/(\w+)\.gif" width=30 height=30></td><td valign=center>(.*?)</td></tr></table></center>}{$repl};
      }

      # candy hearts
      if($bulk =~ m{<center><table[^>]*><tr><td[^>]*><img src="http://images\.kingdomofloathing\.com/otherimages/heart/hearttop\.gif"}) {

	$mail->{heart_message} = '';

	# Loses line breaks? Punctuation?
	while($bulk =~ m{<img src="http://images.kingdomofloathing.com/otherimages/letters/(.*?).gif"}g) {
	  my $c = $1;
	  $c =~ s/space/ /;
	  $c =~ s/exclamation/!/;
	  $c =~ s/question/?/;
	  $c =~ s/lessthan/</;
	  $c =~ s/period/./;
	  $c =~ s/caret/^/;
	  $c =~ s/hyphen/-/;
	  $c =~ s/singlequote/\'/;
	  $c =~ s/equals/=/;
	  $c =~ s/rightparen/)/;
	  $c =~ s/leftparen/(/;
          $c =~ s/colon/:/;
	  die $c if length($c) > 1;
	  $mail->{heart_message} .= $c;
	}

	$mail->{message} = 'HEART: ' . $mail->{heart_message};
	$bulk =~ s{<center>.*?</center>}{};
      # cups
      } elsif($bulk =~ m{<center><table[^>]*><tr><td[^>]*><img src="http://images\.kingdomofloathing\.com/otherimages/heart/cuptop\.gif"}) {

	$mail->{cup_message} = '';

	# Loses line breaks? Punctuation?
	my @bulks = split(/<\/tr><tr>/,$bulk);
	for my $cupline (@bulks) {
	  next if $cupline eq '<td height=10></td>';
	  next if $cupline =~ /cupbottom.gif/;
	  while($cupline =~ m{<img src="http://images.kingdomofloathing.com/otherimages/letters/(.*?).gif"}g) {
	    my $c = $1;
	    $c =~ s/space/ /;
	    $c =~ s/exclamation/!/;
	    $c =~ s/question/?/;
	    $c =~ s/lessthan/</;
	    $c =~ s/period/./;
	    $c =~ s/caret/^/;
	    $c =~ s/hyphen/-/;
	    $c =~ s/singlequote/\'/;
	    $c =~ s/equals/=/;
	    $c =~ s/rightparen/)/;
	    $c =~ s/leftparen/(/;
	    $c =~ s/colon/:/;
	    die $c if length($c) > 1;
	    $mail->{cup_message} .= $c;
	  }
	  $mail->{cup_message} .= "\n";
	}

	$mail->{message} = 'COFFECUP: ' . $mail->{cup_message};
	$bulk =~ s{<center>.*?</center>}{};
      } else {
	# match message
	$bulk =~ m{^([^<]*)};
	$mail->{message} = html_to_text($1);
      }

      # separate first and second parts
      my ($outer,$inner);
      if($bulk =~ /^(.*)<p>Inside Note:<p>(.*)$/m) {
	$outer=$1;
	$inner=$2;
	$inner =~ m{^([^<]*)};
	$mail->{insidenote} = html_to_text($1);
      } else {
	$outer = $bulk;
	undef $mail->{insidenote};
      }

      my ($meat_o,$items_o);
      my ($meat_i,$items_i);
      unless ($params->{no_items}) {
	($meat_o,$items_o) = $bot->parse_attachments($outer);
	if(defined($inner)) {
	  ($meat_i,$items_i) = $bot->parse_attachments($inner);
	}
      }

      $kmail{$mail->{id}} = 
	new KoLBot::KMail(rawmsg     => $rawmsg,
			  id         => $mail->{id},
			  time       => $mail->{time},
			  user       => $mail->{user},
			  user_id    => $mail->{user_id},
			  message    => $mail->{message},
			  meat       => $meat_o,
			  items      => $items_o,

			  insidenote => $mail->{insidenote},
			  i_meat     => $meat_i,
			  i_items    => $items_i,

# 			  item_id    => [@item_id],
# 			  item_name  => [@item_name],
# 			  number     => [@item_num],

# 			  i_item_id  => [@i_item_id],
# 			  i_item_name => [@i_item_name],
# 			  i_number    => [@i_item_num],
			  );
#      print $kmail{$mail->{id}};
#      die;
    }

    $page++;
  }
  return sort {$a->{id} <=> $b->{id}} values %kmail;
}

sub html_to_text {
  my $text = shift;

  $text =~ s/&lt;/</g;
  $text =~ s/&gt;/>/g;
  $text =~ s/&quot;/\"/g;
  $text =~ s/&amp;/&/g;

  return $text;
};

sub parse_attachments {
  my $bot = shift;
  my $text = shift;

  my $meat = 0;
  my $items = [];

  if($text =~ m{<td valign=center>You gain ([\d,]+) Meat.</td>}) {
    $meat = $1;
    $meat =~ s/,//g;
    # remove meat to ease further processing
    $text =~ s|<center><table><tr><td><img src="http://images.kingdomofloathing.com/itemimages/meat.gif" height=30 width=30 alt="Meat"></td><td valign=center>You gain [\d,]+ Meat.</td>.*$||;
  }

  while($text =~ m{<tr>(.*?)</tr>}g) {
    my $line = $1;
    push @$items, $bot->item_from_acquire($line);
  }

  return ($meat,$items);
}

sub item_from_acquire {
  my $bot = shift;
  my $text = shift;

  $text =~ m{descitem\((\d+)\)} or die "no description: $text";
  my $desc = $1;

  my ($name,$number);
  if($text =~ m{You acquire an item: <b>(.*?)</b>}) {
    $name = $1;
    $number = 1;
  }
  
  if($text =~ m{You acquire <b>([\d,]+) (.*?)</b>}) {
    $number = $1;
    $name = $2;
    $number =~ s/,//g;
  }

  die "No item found $text"
    unless defined($name);

  my $item = {desc => $desc, name => $name, number => $number};

  if(defined($bot->{items}) && $bot->{items}->isa("KoLBot::Items")) {
    $item = 
      $bot->items->get_item(%$item)
      or die 'FATAL: didn\'t find item in database ' . $desc . ', '.$name;
    $item->{number} = $number;
  }
  $item->{id} ||= 0;

  return $item;
}

sub saving_kmail {
  my $bot = shift;
  my $box = shift;
  my @kmail = @_;

  my $url = $bot->url . 'messages.php';

  my $args = 
      [ the_action          => 'save',
	pwd             => $bot->hash,
	box             => $box,
	];

  for(@kmail) {
    push @$args, 'sel'.$_->{id},'on';
  }

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
				$args,
				     );
  delete($bot->ua->{max_size});

  $response->is_success or die 'HTTP ' . $response->status_line;
}

sub delete_kmail {
  my $bot = shift;
  my $box = shift;
  my @kmail = @_;

  my $url = $bot->url . 'messages.php';

  my $args = 
      [ the_action          => 'delete',
	pwd             => $bot->hash,
	box             => $box,
	];

  for(@kmail) {
    push @$args, 'sel'.$_->{id},'on';
  }

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
				$args,
				     );
  delete($bot->ua->{max_size});

  $response->is_success or die 'HTTP ' . $response->status_line;
}

# Obsolete, takes only one mail.
sub del_kmail {
  my $bot = shift;
  my $kmail = shift;

  my $url = $bot->url . 'messages.php';
  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ the_action          => 'delete',
	pwd             => $bot->hash,
	box             => 'Inbox', # For now
	'sel'.$kmail->{id} => 'on',
	]
				     );
  delete($bot->ua->{max_size});

  $response->is_success or die 'HTTP ' . $response->status_line;
}

# Obsolete, takes only one mail.
sub save_kmail {
  my $bot = shift;
  my $kmail = shift;

  my $url = $bot->url . 'messages.php';
  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ the_action          => 'save',
	pwd             => $bot->hash,
	box             => 'Inbox', # For now
	'sel'.$kmail->{id} => 'on',
	]
				     );
  delete($bot->ua->{max_size});

  $response->is_success or die 'HTTP ' . $response->status_line;
}

sub open_gift {
  my $bot = shift;
  my $params = { item_id    => 1167,
		 @_ };

  if($params->{item_id} < 1167 || $params->{item_id} > 1177
     && $params->{item_id} != 1460 ) {
    die "I don't think that is a giftbox in open_gift";
  }

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'inv_use.php?pwd=' . $bot->{hash}
  . '&which=3&whichitem=' . $params->{item_id};
  my $response = $bot->ua->get($url);

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'open_gift');

  $_ = $response->content;

  m|<blockquote>You don't have the item you're trying to use.</blockquote>|
    and return(undef,'None of that item to open.');

  s/&nbsp;/ /g;

  my $pname = '\w[\w\ ]{0,28}\w';

  my $user;

  if(m|<blockquote>.*?<p>From: <b><a class=nounder href="showplayer.php\?who=\d+">($pname)</a></b>.(.*?)<table width=95%\s+cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=blue><b>Inventory:|) {
    $user = $1;
    $_ = $2;
  } elsif (m|<blockquote>.*?<p>From: <b></b>.(.*?)<table width=95%  cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=blue><b>Inventory:|) {
    $user = '';
    $_ = $1;
  } else {
    print("ERROR open_gift: Not really a gift box?\n$_"), die;
  }

  my $content = '';
  my $meat = 0;
  m|<center><p><table><tr><td style='border: 1px solid black; padding: 10px'>(.*?)</td></tr></table></center>|
    and $content = $1;
  m|<center><table><tr><td><img src="http://images.kingdomofloathing.com/itemimages/meat.gif" height=30 width=30 alt="Meat"></td><td valign=center>You gain ([\d,]+) Meat.</td></tr></table>|
    and $meat = $1, $meat =~ s/,//g;
    
  # AWM replace and test
  # my ($meat,$items) = $bot->parse_attachments($_);

  my @item_name = ();
  my @item_id = ();
  my @item_desc = ();
  my @i_num = ();

  my $done = 0;

  s/<\/?td.*?>//g;
  s/<\/td.*?>//g;
  s/<\/?center>//g;
  s/<\/?table>//g;

  while(1) {
    if(/<img[^>]*?descitem\((\d+)\)[^>]*?>You acquire <b>([\d,]+) ([^<]*?)<\/b>/) {
      push @item_desc, $1;
      push @item_name, $3;
      my $num = $2;
      $num =~ s/,//g;
      push @i_num, $num;
      s/<img[^>]*?descitem\((\d+)\)[^>]*?>You acquire <b>([\d,]+) ([^<]*?)<\/b>//;

    } elsif(/<img[^>]*?descitem\((\d+)\)[^>]*?>You acquire an item: <b>([^<]*?)<\/b>/) {
      push @item_desc, $1;
      push @item_name, $2;
      push @i_num, 1;
      s/<img[^>]*?descitem\((\d+)\)[^>]*?>You acquire an item: <b>([^<]*?)<\/b>//;

    } else {
      /acquire/i and print "ERROR open_gift Missed items in $_\n";

      last;
    }
  }

  $content =~ s/<br>/\n/g;
#  $content =~ s/&lt;/</g;
#  $content =~ s/&gt;/>/g;
  $content =~ s/&amp;/&/g;
  $content =~ s/&quot;/\"/g;
  
  my @items;

  if($params->{no_items}) {
    @item_id = ();
    @item_name = ();
    @i_num = ();
  } else {
    for(0..$#i_num) {
      my $item = 
	$bot->items->get_item(desc => $item_desc[$_],
			      name => $item_name[$_],
			      number => $i_num[$_])
#	  || 
#	  $bot->find_item(desc => $item_desc[$_],
#			  name => $item_name[$_])
	|| 
	die 'FATAL: didn\'t find '.$item_desc[$_].','.$item_name[$_];

      push @items,$item;
    }
  }

  return new KoLBot::KMail(time      => 0,
			   user      => $user,
			   message   => $content,
			   id        => 0,
			   user_id   => 0,
			   meat      => $meat,
			   items     => [@items],
			   );
}

##########################################
## Clan interface
##########################################

sub clan_buff {
  my $bot = shift;
  my $parms = { @_ };

  my $content;
  my $url = $bot->url . 'clan_buytraining.php';

  my $response = $bot->ua->post($url,
				[ 
				  action  => 'buyround',
				  pwd     => $bot->{hash},
				  members => $parms->{members},
				  # 1 small, 2 med, 3 expensive
				  size => $parms->{size},
				  #next is 1 musc, 2 myst, 3 mox
				  # temp 4-5-6
				  whichgift => $parms->{whichgift},
				  ]
				);
  $bot->check_response($response,'clan_buff');

  return unless($parms->{check_response});

  $content = $response->content;
}

# good actions are 'addgoodies'
# 'takegoodies'

sub clan_stash {
  my $bot = shift;
  my $parms = { @_ };

  my $content;
  my $url = $bot->url . 'clan_stash.php';

  if($parms->{action}) {
    # code from tomek
    my $response;
    if($parms->{action} eq "addgoodies") {
      $response = $bot->ua->post($url,
	    [ 
	      action => $parms->{action},
	      qty1 => $parms->{quantity},
	      item1 => $parms->{whichitem},
	      pwd    => $bot->{hash},
	      ]);
    } else {
      $response = $bot->ua->post($url,
	 [ 
	      action => $parms->{action},
	      quantity => $parms->{quantity},
	      whichitem => $parms->{whichitem},
	      pwd    => $bot->{hash},
	      ]);
    }

    $bot->check_response($response,'clan_stash');

    return unless($parms->{check_response});

    $content = $response->content;
  } else {
    my $response = $bot->ua->get($url);

    $bot->check_response($response,'clan_stash');

    $content = $response->content;
  }

  my ($inv,$stash) = ([],[]);
  # get rid of head and tail, keep only the juicy fillings

  if($content =~ m|<tbody id='additems'>(.*?)</tbody>|) {
    my $acont = $1;
     
    while($acont =~ m|<option value="(\d+)">(.*?) \(([\d,]+)\)</option>|g) {
      my $id = $1;
      my $name = $2;
      my $number = $3;
      $number =~ s/,//g;

      push @$inv,new KoLBot::Item(id=>$id,name=> $name,number=>$number);
    }
  }

  if($content =~ m|<form name=takegoodies action=clan_stash.php method=post>(.*?)</form>|) {
    my $acont = $1;

    while($acont =~ m|<option value="?(\d+)"? descid=(\d+)>(.*?)( \(([\d,]+)\))?</option>|g) {
      my $id = $1;
      my $descid = $2;
      my $name = $3;
      my $number = $5 || 1;
      $number =~ s/,//g;
      push @$stash,new KoLBot::Item(id=>$id,name=> $name,number=>$number,
				    desc => $descid);
    }
  } 

  return ($stash,$inv);
}

sub clan_applications {
  my $bot = shift;
  my $parms = { @_ };

  my $content;
  my $url = $bot->url . 'clan_applications.php';

  if($parms->{id}) {
    my $value = $parms->{accept}?1 : $parms->{decline} ? 2 : return undef;
    my $response = $bot->ua->post($url,
				  [ 
			    action => 'process',
			    pwd       => $bot->hash,
			    'request' . $parms->{id} => $value,
				    ]
				  );
    $bot->check_response($response,'clan_applications');

    return unless($parms->{check_response});

    $content = $response->content;
  } else {
    my $response = $bot->ua->get($url);

    $response->is_success or die 'HTTP ' . $response->status_line;

    $bot->check_response($response,'clan_applications');

    $content = $response->content;
  }

  $content =~ s/<td>//g;
  $content =~ s/<\/td>//g;

  my $applicants = [];
  # get rid of head and tail, keep only the juicy fillings

  unless($content =~ m|Reject</u></b></tr><tr><a href=\"showplayer.php\?who=(.*)</tr><tr><td colspan=3|) {
    return $applicants;
  }

  my $middle = $1;

  my @apps = split m{<a href=\"showplayer.php\?who=},$middle;

  for (@apps) {
    m{^(\d+)\">(.*?)</a>}
    or die "Not an app: $_";

    my $id = $1;
    my $name = $2;
    
    push @$applicants,{id => $id, name => $name};

  }

  return $applicants;
}

sub clan_members {
  my $bot = shift;
  my $params = { action => 'modify',
		@_ };

  my $content;
  my $url = $bot->url . 'clan_members.php';

  if($params->{id}) {
    my $args = [ 
		 pwd       => $bot->hash,
		 action    => $params->{action},
		 ];
    push @$args,'pids[]'=> $params->{id};
    push @$args, begin => 1;
    if($params->{boot}) {
      push @$args, 'boot' . $params->{id} => 'on';
    } else {
      if($params->{rank} && !$params->{boot}) {
	push @$args, 'level' . $params->{id} => $params->{rank};
      } else { 
	push @$args, 'level' . $params->{id} => 0;
      }
      if($params->{title}) {
	push @$args, 'title' . $params->{id} => $params->{title};
      } else {
	push @$args, 'title' . $params->{id} => '';
      }
    }

    my $response = $bot->ua->post($url,$args);

    $bot->check_response($response,'clan_members');

    return;
  }

  my $page = '';

  my $members = [];
 PAGE: {
   my $response = $bot->ua->get($url . $page);

   $response->is_success or die 'HTTP ' . $response->status_line;

   $bot->check_response($response,'clan_members_2');

   $content = $response->content;

   $content =~ s/<td>//g;
   $content =~ s/<\/td>//g;

   # get rid of head and tail, keep only the juicy fillings

   unless($content =~ m|(<a class=nounder href.*)<tr><td colspan=4|) {
     return $members;
   }

   my $middle = $1;

   my @mems = split m{</tr><tr>},$middle;

   for (@mems) {
#    print $_,"\n";
     m{who=(\d+)\">(.*?)</a>} or die "Not a member: $_";


     my $m = {};
     $m->{id} = $1 if $1;
     $m->{name} = $2 if $2;

     if(m{<option value=(\d+) selected>(.*?)</option>}) {
       $m->{rank} = $1;
       $m->{rank_name} = $2;
     }
     if(m{title\d+ size=20 maxlength=30 value=\"(.*?)\">}) {
       $m->{title} = $1;
     }

     push @$members,$m;
   }
   if($content =~ /clan_members.php(\?begin=\d+)\">next 20/) {
     $page = $1;
     redo PAGE;
   }
 }
  return $members;
}

sub clan_admin {
  my $bot = shift;
  my $parms = { @_ };

  if($parms->{action} eq 'transfer') {
    my $content;
    my $url = $bot->url . 'clan_admin.php';

    my $args = [ 
		 pwd       => $bot->hash,
		 action    => 'changeleader',
		 newleader => $parms->{id},
		 confirm   => 'on',
		 ];

    my $response = $bot->ua->post($url,$args);

    $bot->check_response($response,'clan_admin');

    return;
  }
}

sub clan_war {
  my $bot = shift;
  my $parms = { @_ };

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'clan_war.php';

  delete($bot->ua->{max_size});
  my $response;
  if($parms->{goodies}) {
    my $p = [ 
	      action => 'Yep',
	      goodies=> $parms->{goodies},
	      oatmeal => 0,
	      cinderblocks => 0,
	      grunts => 0,
	      flyers => 0,
	      recliners => 0,
	      archers => 0,
	      ];
    $response = $bot->ua->post($url,$p);
  } else {
    $response = $bot->ua->get($url);
  }

  $bot->check_response($response,'clan_war');

  my $resp = {};

  my $content = $response->content;

  $content =~ s/&nbsp;/ /g;
  $content =~ s/ width=\d+//g;
  $content =~ s/ height=\d+//g;
  $content =~ s/(\d),(\d)/$1$2/g;
  $content =~ s/<td.*?>/<td>/g;
  $content =~ s/<font.*?>/<font>/g;
  $content =~ s/\s+/ /g;


  $content =~ m{Bags of Goodies:</td><td><b>(\d+)}i
    and $resp->{goodies} = $1;

  return $resp;
}

sub clan_board {
  my $bot = shift;
  my $parms = { @_ };

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'clan_board.php';

  delete($bot->ua->{max_size});
  my $response;
  if($parms->{delete}) {
    $url .= '?action=delete&msgid=' . $parms->{msgid};
  }
  $response = $bot->ua->get($url);
  $bot->check_response($response,'clan_board');

  my $content = $response->content;

  my $pname = '\w[\w\ ]{0,28}\w';

  my @messages = ();
  while ($content =~ m|<font size=2><p><b>Date: (.*?)<br>From: <a href="showplayer.php\?who=(\d+)">($pname)</a> \(\#(\d+)\)</b><p><blockquote>(.*?)</blockquote><p><center><a href="sendmessage.php\?toid=(\d+)">reply \(private message\)</a>(&nbsp;&nbsp;<a href="clan_board.php\?action=delete&msgid=(\d+)">delete</a>)?</font></center>|g) {
    push @messages,{dstr => $1,user_id => $2,user => $3,text => $5,
		     msgid =>$8};
#    my $dstr = $1;
#    my $pid = $2;
#    my $pname = $3;
#    my $text = $5;
#    my $msgid = $7;
    # Our clan was attacked by The Knights of the Realm. They were able to loot 1 Bags of Goodies.
# Our clan was attacked by disco banits. They were unable to loot any Goodies.
  }
  return @messages;
}

sub clan_whitelist {
  my $bot = shift;
  my %params = (level => 0,
		title => '',
		@_);

  my $url = $bot->url . 'clan_whitelist.php';

  my $response;
  if($params{action} eq 'add') {
    my $p = [ 
	      action => $params{action},
	      addwho => $params{user},
	      level  => $params{level},
	      title  => $params{title},
	      ];
    $response = $bot->ua->post($url,$p);
  } else {
    die "clan_whitelist only adds";
  }

  $response->is_success or die 'HTTP ' . $response->status_line;
  return;
}

# topicid => 
# postid => 
# message => 
# mode => 'edit' or 'reply'
# 

sub clan_fpost {
  my $bot = shift;
  my %params = (mode => 'reply',
		@_);

  my $url = $bot->url . 'clan_fpost.php';

  my $response;

  my $p = [ 
	    pwd       => $bot->hash,
	    mode      => $params{mode},
	    action    => $params{mode},
	    message   => $params{message},
	    subject   => $params{subject},
	    ];

  if($params{mode} eq 'edit') {
    die "No postid to edit" unless exists $params{postid};
    push @$p,'postid' => $params{postid};
  } elsif ($params{mode} eq 'reply') {
    die "No topicid to reply" unless exists $params{topicid};
    push @$p,'topicid' => $params{topicid};
  } else {
    die "Can't recognize " . $params{mode};
  }

  $response = $bot->ua->post($url,$p);

  $response->is_success or die 'HTTP ' . $response->status_line;

  return;
}

sub clan_raidlogs_raw {
  my $bot = shift;

  my $url = $bot->url . 'clan_raidlogs.php';
  my $response = $bot->ua->get($url);
  $bot->check_response($response,'clan_raidlogs');
  my $content = $response->content;

}

sub clan_raidlogs {
  my $bot = shift;

  my $url = $bot->url . 'clan_raidlogs.php';
  my $response = $bot->ua->get($url);
  $bot->check_response($response,'clan_raidlogs');
  my $content = $response->content;

  my $results = {};
  for(qw{sewer_water sewer_grates 
	   ee_water ee_water_bb ee_yodel
	   pl_flimflam pl_bamboozle pl_trash_th
	   bg_flowers
	   th_trashcano th_compost_bg
	   bb_steam
	   ts_hobos
	 }) {
    $results->{$_} = 0;
  }

  # this populates $_.  There should be an easier way.
  for($content) {
    # count various things.
    # sewers
    while(/\) lowered the water level (\d+ times )?\((\d+) turns?\)<br>/g) {
      $results->{sewer_water} += $2;
    }
    while(/\) opened (\d+|a) sewer grates? \((\d+) turns?\)<br>/g) {
      $results->{sewer_grates} += $2;
    }

    # exposure
    while(/\) diverted some cold water to Burnbarrel Blvd. \((\d+) turns?\)<br>/g) {
      $results->{ee_water_bb} += $1;
    }
    while(/\) diverted some cold water out of Exposure Esplanade \((\d+) turns?\)<br>/g) {
      $results->{ee_water} += $1;
    }
    while(/\) yodeled like crazy \((\d+) turns?\)<br>/g) {
      $results->{ee_yodel} += $1;
    }
      # fridge raiding

    # purple
    while(/\) flimflammed some hobos \((\d+) turns?\)<br>/g) {
      $results->{pl_flimflam} += $1;
    }
    while(/\) bamboozled some hobos \((\d+) turns?\)<br>/g) {
      $results->{pl_bamboozle} += $1;
    }
    while(/\) sent some trash to The Heap \((\d+) turns?\)<br>/g) {
      $results->{pl_trash_th} += $1;
    }

    # burial
    while(/\) sent (\d+|a) bunch(es)? of flowers to The Heap \((\d+) turns?\)<br>/g) {
      $results->{bg_flowers} += $3;
    }
      # tomb raiding

    # burnbarrel
    while(/\) diverted some steam away from Burnbarrel Blvd. \((\d+) turns?\)<br>/g) {
      $results->{bb_steam} += $1;
    }

    # the Heap
    while(/\) started (\d+|a) trashcano eruptions? \((\d+) turns?\)<br>/g) {
      $results->{th_trashcano} += $2;
    }
    while(/\) sent (\d+|a) batch(es)? of compost to the Burial Ground \((\d+) turns?\)<br>/g) {
      $results->{th_compost_bg} += $3;
    }

    # buried treasure

    # town square
    while(/defeated\s+Normal hobo (x \d+ )?\((\d+) turns?\)<br>/gi) {
      $results->{ts_hobos} += $2;
    }
  }
  $results->{size} = $content =~ s/<br>//gi;

  return $results;
}

sub clan_hobopolis_richard {
  my $bot = shift;

  my $url = $bot->url . 'clan_hobopolis.php?place=3&action=talkrichard&whichtalk=3';
  my $response = $bot->ua->get($url);
  $bot->check_response($response,'clan_hobopolis_richard');
  my $content = $response->content;

  my $results = {
                 boots => 0,
		 eyes => 0,
		 guts => 0,
    	         skulls => 0,
	         crotches => 0,
		 skins => 0,};

  $content =~ /<b>(\d+)<\/b> pairs? of charred hobo boots/
    and $results->{boots} = $1;
  $content =~ /<b>(\d+)<\/b> pairs? of frozen hobo eyes/
    and $results->{eyes} = $1;
  $content =~ /<b>(\d+)<\/b> piles? of stinking hobo guts/
    and $results->{guts} = $1;
  $content =~ /<b>(\d+)<\/b> creepy hobo skull/
    and $results->{skulls} = $1;
  $content =~ /<b>(\d+)<\/b> hobo crotch/
    and $results->{crotches} = $1;
  $content =~ /<b>(\d+)<\/b> hobo skin/
    and $results->{skins} = $1;

  return $results;
}

########################################################################
# PVP
########################################################################

sub pvp {
  my $bot = shift;
  my $parms = { @_ };

  my $content;
  my $url = $bot->url . 'pvp.php';

  if($parms->{id}) {
    my $p = [ 
	      action => 'Yep',
	      pwd    => $bot->hash,
	      who    => $parms->{id},
	      # 1,2,3 std
	      stance => $parms->{stance},
	      # dignity, meat, lootbooze, lootfood, lootwhatever
	      attacktype => $parms->{attacktype},
	      winmessage => $parms->{winmessage},
	      losemessage => $parms->{losemessage},
	      ];
    my $response = $bot->ua->post($url,$p);

    $bot->check_response($response,'pvp');

    return unless($parms->{check_response});

    $content = $response->content;


    if($content =~ /failed to yield/i) {
      print "AWM DEBUG FAILED TO YIELD\n";
    }

    if($content =~ /No attacking\./) {
      return (1,'hippy');
    }
    if($content =~ /ranked too highly/) {
      return (1,'too high');
    }

    $content =~ /<center><table><tr>(.*?)<\/tr><\/table><table width/
      or return(1,"can\'t match $content");

    $content = $1;
    
    $content =~ s/<b>/ /g;
    $content =~ s/<\/b>/ /g;
    $content =~ s/<td.*?>/ /g;
    $content =~ s/<\/td>/ /g;
    $content =~ s/<\/table>/ /g;
    $content =~ s/<center>/ /g;
    $content =~ s/<\/center>/ /g;
    $content =~ s/<font.*?>/ /g;
    $content =~ s/<\/font>/ /g;
    $content =~ s/\s+/ /g;
    
    my $r;
    my @lines=split(/<p>/,$content);
    for(@lines) {
      /^\s*$/ and next;

      /Attacking (.*) \.\.\./
	and $r .= "attacking $1\n",next;
      /call (.*) out/
	and $r .= "attacking $1\n",next;

      /by his house/i
	and $r .= "house\n",next;
      /pretty flowers/i
	and $r .= "flowers\n",next;
      /scarecrow/i
	and $r .= "scarecrow\n",next;
      /wire fence/i
	and $r .= "barbed-wire fence\n",next;
      /white picket/i
	and $r .= "white picket fence\n",next;
      /feng shui/i
	and $r .= "feng shui\n",next;

      /spell bounces right back at you/
	and $r .= "buffed myst: loss\n",next;
      /magical gambit is successful/
	and $r .= "buffed myst: win\n",next;
      /opponent feel like a loser/
	and $r .= "buffed moxie: win\n",next;
      /one-ups you with a well-timed joke/
	and $r .= "buffed moxie: loss\n",next;
      /ignores your taunts.*punches you/
	and $r .= "buffed moxie: loss\n",next;
      /spell, which also burns/
	and $r .= "buffed moxie: loss\n",next;

      /raw Mysticality.*You lose the round/i
	and $r .= "raw magic: loss\n",next;
      /raw Moxie vs\. raw Moxie. You lose/
	and $r .= "raw moxie: loss\n",next;
      /raw Moxie vs\. raw Moxie\. You win/
	and $r .= "raw moxie: win\n",next;

      /armwrestling match. You struggle and groan/
	and $r .= "buffed muscle: loss\n",next;
      /armwrestling.*\bwin\b/
	and $r .= "buffed muscle: win\n",next;
      /beat the band, and deftly win/
	and $r .= "buffed moxie: win\n",next;

      /Mysticality outmatches yours/
	and $r .= "buffed myst: loss\n",next;
      /mad magical skills/
	and $r .= "buffed myst: win\n",next;

      /drunk more White Canadians/
	and $r .= "canadian: loss\n",next;
      /way more balanced than your opponent/
	and $r .= "diet: win\n",next;
      /eat significantly more pie/
	and $r .= "eating: win\n",next;
      /your opponent is able to consume more pie than you/
	and $r .= "eating: loss\n",next;
      /weighs (more )?than your opponent/
	and $r .= "familiar: win\n",next;
      /your familiar. Your familiar doesn\'t have/
	and $r .= "familiar: loss\n",next;
      /far more stylish than your opponent/
	and $r .= "fashion: win\n",next;
      /deem that your style is outmoded/
	and $r .= "fashion: loss\n",next;
      /more flowers than you/
	and $r .= "flowers: loss\n",next;
      /Your headache knows no bounds/
	and $r .= "hangover: win\n",next;
      /significantly hunger over than you/
	and $r .= "hangover: loss\n",next;
      /more blood than your opponent/
	and $r .= "HP: win\n",next;
      /you easily outburp him/
	and $r .= "MP: win\n",next;
      /less threatened by him/
	and $r .= "popularity: loss\n",next;
      /fewer active effects on you/
	and $r .= "purity: win\n",next;
      /purity.*opponent has fewer active/
	and $r .= "purity contest: loss\n",next;
      /staggering array of tattoos/
	and $r .= "tattoo: win\n",next;
      /your stack is higher/
	and $r .= "trophy: win\n",next;
      /taken fewer vacations than your opponent/
	and $r .= "work ethic: win\n",next;
      /ethic contest. Your opponent has taken fewer/
	and $r .= "work ethic: loss\n",next;


      /ve won the fight!/
	and $r .= "You win.\n",next;
      /You lost the fight!/
	and $r .= "You lose.\n",next;
      /decreased by (\d+) points/
	and $r .= "PvP: -$1\n",next;
      /increased by (\d+) points/
	and $r .= "PvP: +$1\n",next;

      if(/You (lost|steal) some dignity/) {
	for my $l (split /<tr>/,$_) {
	  $l =~ /You (lost|steal) some dignity/
	    and next;
	  $l =~ /You gain (\d+) (Beef|Stren|Strong|Muscle|Fortit)/
	    and $r .= "Muscle: +$1\n",next;
	  $l =~ /You gain (\d+) (Enchant|Magic|Myst|Wizard)/
	    and $r .= "Myst: +$1\n",next;
	  $l =~ /You gain (\d+) (Smarm|Rogui|Chutz|Cheek|Sarcasm)/
	    and $r .= "Moxie: +$1\n",next;
	  $l =~ /You lose (\d+) (Beef|Stren|Strong|Muscle|Fortit)/
	    and $r .= "Muscle: -$1\n",next;
	  $l =~ /You lose (\d+) (Enchant|Magic|Myst|Wizard)/
	    and $r .= "Myst: -$1\n",next;
	  $l =~ /You lose (\d+) (Smarm|Rogui|Chutz|Cheek|Sarcasm)/
	    and $r .= "Moxie: -$1\n",next;
	  $l =~ /^\s*$/
	    and next;

	  $r .= "AWM: Unknown part: $l\n";
	}
	next;
      }

      $r .= "AWM: Unknown line: $_\n";
    }
    return(0,$r);
  }
}

##########################################
## Utility
##########################################


# only sets, doens't read
sub account_profile {
  my $bot = shift;
  my $parms = { @_ };

  my $url = $bot->url . 'account_profile.php';

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ action          => 'updating',
	pwd             => $bot->hash,
	age             => $parms->{age},
	gender          => $parms->{gender},
	location        => $parms->{location},
	website         => $parms->{website},
	quote           => $parms->{quote},
	]
				     );
  delete($bot->ua->{max_size});
}

# only sets, doens't read
sub account_avatar {
  my $bot = shift;
  my $parms = { @_ };

  my $url = $bot->url . 'account_avatar.php';

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ action          => 'Yep',
	pwd             => $bot->hash,
	whichavatar     => $parms->{whichavatar},
	]
				     );
  delete($bot->ua->{max_size});
}

sub account_contactlist {
  my $bot = shift;

  my $url = $bot->url . 'account_contactlist.php';

  my $response = $bot->ua->get($url);
  $bot->check_response($response,'account_contactlist');

  my $contact = [];
  my $ignore = [];
  my $content = $response->content;

  unless($content =~ m{<td style="color: white;" align=center bgcolor=blue><b>Contact List</b></td>(.*)<td style="color: white;" align=center bgcolor=blue><b>Ignore List</b></td>(.*)}s) {
    print $content;
    die;
  }
  my $part1 = $1;
  my $part2 = $2;

  while ($part1 =~ m|<tr><td><input type=checkbox name=pids\[\] value=(\d+)></td><td><a href="showplayer.php\?who=(\d+)" class=nounder><b>([\w ]+)</b></a></td></tr>|g){
    push @$contact,{user    => $3,
		    user_id => $1,};
  }
  while ($part2 =~ m|<tr><td><input type=checkbox name=pids\[\] value=(\d+)></td><td><a href='showplayer.php\?who=(\d+)' class=nounder><b>([\w ]+)</b></a></td></tr>|g){
    push @$ignore,{user    => $3,
		   user_id => $1,};
  }

  return $contact,$ignore;
}

sub add_contact {
  my $bot = shift;
  my $parms = { @_ };

  my $url = $bot->url . 'account_contactlist.php';

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ action          => 'add',
	pwd             => $bot->hash,
	who             => $parms->{user},
	]
				     );
  delete($bot->ua->{max_size});
  $response->is_success or die "HTTP " . $response->status_line;
}

sub remove_contact {
  my $bot = shift;
  my $parms = { @_ };

  if(!defined($parms->{user_id})) {
    warn "No user id!";
    return;
  }

  my $url = $bot->url . 'account_contactlist.php';
  my $args = [ pwd       => $bot->hash,
	       'pids[]'  => $parms->{user_id},
	       action    => 'remove',
	       which     => 'contact',
	       ];

  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;
}

sub add_ignore {
  my $bot = shift;
  my $parms = { @_ };

  my $url = $bot->url . 'account_contactlist.php';

  $bot->ua->max_size(0);
  my $response = $bot->ua->post($url,
      [ action          => 'add',
	ignore          => 'yes',
	pwd             => $bot->hash,
	who             => $parms->{user},
	]
				);
  delete($bot->ua->{max_size});
  $response->is_success or die "HTTP " . $response->status_line;
}

sub remove_ignore {
  my $bot = shift;
  my $parms = { @_ };

  if(!defined($parms->{user_id})) {
    warn "No user id!";
    return;
  }

  my $url = $bot->url . 'account_contactlist.php';
  my $args = [ pwd       => $bot->hash,
	       'pids[]'  => $parms->{user_id},
	       action    => 'remove',
	       which     => 'ignore',
	       ];

  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;
}

#
#
#

sub rename_familiar {
  my $bot = shift;
  my $params = { @_ };

  my $check_response = $params->{check_response} || 0;

  my $url = $bot->url . 'familiar.php';

  $bot->ua->max_size(0) unless $check_response;
  my $response = $bot->ua->post($url,
      [ action          => 'rename',
	pwd             => $bot->hash,
	newname         => $params->{newname},
	]
				     );
  delete($bot->ua->{max_size});
  $response->is_success or die "HTTP " . $response->status_line;

  return 0 unless $check_response;

  $bot->check_response($response,'rename_familiar');

  # AWM: check more here

}

sub take_familiar {
  my $bot = shift;
  my $params = { @_ };

  my $check_response = $params->{check_response} || 0;

  my $url = $bot->url . 'familiar.php';

  $bot->ua->max_size(0) unless $check_response;
  my $response = $bot->ua->post($url,
      [ pwd             => $bot->hash,
	action          => 'newfam',
	newfam          => $params->{newfam},
	]
				);
  delete($bot->ua->{max_size});

  $response->is_success or die "HTTP " . $response->status_line;

  return 0 unless $check_response;

  $bot->check_response($response,'take_familiar');

  # combine rest with familiar?
}

sub familiar {
  my $bot = shift;
  my $params = { @_ };

  my $url = $bot->url . 'familiar.php';

  delete($bot->ua->{max_size});
  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'familiar');

  my $ret = { familiars => {},
	      current   => {},
	    };
  my $content = $response->content;
  if($content =~ m{Current Familiar:<br><img src="http://images.kingdomofloathing.com/itemimages/(.*?)" class=hand onClick='fam\((\d+)\)'><br><b>(.*?)</b><br>(\d+)-pound (.*?) \(([\d,]+) experience, ([\d,]+) kills\)<table>}) {
    ## current familiar equipment not read

    $ret->{current} = {pic  => $1,
		       id   => $2,
		       name => $3,
		       weight => $4,
		       type => $5,
		       exp => $6,
		       kills => $7,};

    $ret->{current}->{exp} =~ s/,//g;
    $ret->{current}->{kills} =~ s/,//g;

    $ret->{familiars}->{$ret->{current}->{id}} = $ret->{current};
  }

  while($content =~ m{<input type=radio name=newfam value=(\d+)></td><td valign=center><img src="http://images.kingdomofloathing.com/itemimages/(.*?)" class=hand onClick='fam\(\d+\)'></td><td valign=center><b>(.*?)</b>, the (\d+)-pound (.*?) \(([\d,]+) exp, ([\d,]+) kills\)}g) {
    my $newfam = { id => $1,
		   pic => $2,
		   name => $3,
		   weight => $4,
		   type => $5,
		   exp => $6,
		   kills => $7,
		 };
    $newfam->{exp} =~ s/,//g;
    $newfam->{kills} =~ s/,//g;
    $ret->{familiars}->{$newfam->{id}} = $newfam;
  }

  return $ret;
}

sub get_users_from_skills {
  my $bot = shift;

  my $url = $bot->url . 'skills.php';

  my $response = $bot->ua->get($url);
  $bot->check_response($response,'get_users_from_skills');

  $response->content =~ /---Online Players---<\/options>(.*?)<\/select>/;
  my $content = $1;
  my @lines=split(/<\/option>/,$content);
  my $ids = [];
  my $names = [];
  for(@lines) {
    /value=(\d+)>(.*)/;
    push @$ids,$1;
    push @$names,$2;
  }
  return ($ids,$names);
}

# 3 is smile of Mr. A.

sub use_skill {
  my $bot = shift;
  my $params = { castings => 1,
		 @_ };

  my $url = $bot->url . 'skills.php';

  my $args = 
      [ pwd             => $bot->hash,
	action          => 'Skillz',
	whichskill      => $params->{skill},];
  if(exists($params->{tplayer})) {
    push @$args, specificplayer  => $params->{tplayer},
                 bufftimes       => $params->{castings};
  } else {
    push @$args, quantity => $params->{castings};
  }

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'use_skill');

  return $bot->parse_response($response->content);
}

#
# Do recipe is an overall function that can do lots of stuff
#

sub do_recipe {
  my $bot = shift;
  my $r = shift;
  my $check_response = shift || 0;

  my ($rv,$text,@prods) 
    = $bot->do_general($r->reagents,$r->action,$check_response);

  return ($rv,$text,@prods) unless ($rv);

  my $error = 0;
  my @cprods = @prods;
 LOOPI:
  for my $i (0..$#{$r->products}) {
    for my $j (0..$#cprods) {
      if($r->products->[$i] == $cprods[$j]) {
	splice @cprods,$i,1;
	next LOOPI;
      }
    }
    $error = 1;
    last LOOPI;
  }
  if($error or @cprods) {
    warn "recipe on file doesn't match items received, prods = (",
     join(',',@prods),")";
    $bot->{error} = "recipe on file doesn't match items received";
    return (1,$text,@prods);
  }
  return (1,$text,@prods);
}

# item stuff

sub inv_equip_save_custom {
  my $bot = shift;
  my $name = shift;

  my $url = $bot->url . 'inv_equip.php?which=2';
  my $args = [action => 'customoutfit',
	      outfitname => $name,
	      pwd  => $bot->{hash},
	      ];
  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;

  $response->content =~ /<option value=(-\d+)>$name<\/option>/
    and return $1;

  return 0;
}

sub inv_equip_outfit {
  my $bot = shift;
  my $outfit_id = shift;

  my $url = $bot->url . 'inv_equip.php';
  my $args = [action => 'outfit',
	      whichoutfit => $outfit_id,
	      which => 2,
	      pwd  => $bot->{hash},
	      ];
  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;

  $response->content =~ /You put on an Outfit:/ and return (0,'');

  $response->content =~ 
    /You don\'t have one or more of the items from that set./
    and return (1,"Missing item(s)");
  die "Don't know equip response " . $response->content;
}

sub equip {
  my $bot = shift;
  my $item_id = shift;
  my $slot = shift;

  my $url = $bot->url
    . 'inv_equip.php?'
    . 'pwd=' . $bot->{hash} 
    . '&which=2&action=equip&whichitem=' . $item_id;
  $url .= "&slot=$slot" if defined $slot;

  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  return $bot->parse_response($response->content);
}

sub unequip {
  my $bot = shift;
  my $type = shift;

  $type =~ /^(hat|weapon|pants|acc1|acc2|acc3|familiarequip|shirt)$/ 
    or die "Unequip: unknown type";

  my $url = $bot->url
    . 'inv_equip.php?which=2&action=unequip&type=' . $type
    . '&pwd=' . $bot->{hash};
  my $response = $bot->ua->head($url);
  $response->is_success or die "HTTP " . $response->status_line;
}

sub unequipall {
  my $bot = shift;

  my $url = $bot->url
    . 'inv_equip.php?&pwd=' . $bot->{hash} . '&action=unequipall';
  my $response = $bot->ua->head($url);
  $response->is_success or die "HTTP " . $response->status_line;
}

sub inventory_makestuff {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'inventory.php';
  my $args = [quantity => $params{quantity} || 1,
	      action => 'makestuff',
	      pwd  => $bot->{hash},
	      whichitem => $params{whichitem},
	      ];
  my $response = $bot->ua->post($url,$args);
  $response->is_success or die "HTTP " . $response->status_line;
  return parse_response($response->content);
}

# AWM: get rid of this
# migrated to use new generic format, which sucks!
sub use {
  my $bot = shift;
  my $item = shift;
  my $check_response = shift || 0;

  return $bot->do_general([$item],'use',$check_response);
}

sub multiuse {
  my $bot = shift;
  my $item = shift;
  my $quant = shift;
  my $check_response = shift || 0;

  my $url = $bot->url . 'multiuse.php';
  $bot->ua->max_size(0) unless($check_response);
  my $args =
      [ action          => 'useitem',
	whichitem       => $item,
	pwd             => $bot->hash,
	quantity        => $quant,
	];
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  $bot->check_response($response,'multiuse');

  return 0 unless $check_response;

  return $bot->parse_response($response->content);
}


# general use an inventory item.
# returns (error code, descriptive text, obtained items)
sub do_general {
  my $bot = shift;
  my $reagents = shift;
  my $action = shift;
  my $check_response = shift || 0;

  my $item_name='([01-9a-z \-\'()]|&trade;|&ntilde;)+';
#  my $item_name='([a-z ]|&trade;|&ntilde;)+';

  if($action eq 'use') {
    my $url = $bot->url . 'inv_use.php?'
      . 'pwd=' . $bot->{hash}
      . '&which=3&whichitem=' . $reagents->[0];
    my $response;

    if(!$check_response) {
      $response = $bot->ua->head($url);
      $response->is_success or die "HTTP " . $response->status_line;
      return 1;
    }

    $response = $bot->ua->get($url);

    $response->is_success or die "HTTP " . $response->status_line;

    $bot->check_response($response,'do_general_use');

    return 0 unless $check_response;

    return $bot->parse_response($response->content);
  }
  die "didn't recognize command";
}

sub eat_item {
  my $bot = shift;
  my $item_no = shift;
  my $check_response = shift || 0;

  my $url = $bot->url . 'inv_eat.php?'
    . 'pwd=' . $bot->{hash} . '&'
    . 'which=1&whichitem=' . $item_no;

  my $response;
  if($check_response) {
    $response = $bot->ua->get($url);
  } else {
    $response = $bot->ua->head($url);
  }

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return 0 unless $check_response;

  return $bot->parse_response($response->content);
}

sub drink_item {
  my $bot = shift;
  my $item_no = shift;
  my $check_response = shift;

  my $url = $bot->url 
    . 'inv_booze.php?'
    . 'pwd=' . $bot->{hash}
    . '&which=1&whichitem=' . $item_no;

  my $response;
  if($check_response) {
    delete($bot->ua->{max_size});
    $response = $bot->ua->get($url);
  } else {
    $response = $bot->ua->head($url);
  }

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return 0 unless $check_response;

  return $bot->parse_response($response->content);
}

sub starchart {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'starchart.php';
  my $args =
      [ action          => 'makesomething',
	pwd             => $bot->hash,
	numstars        => $params{numstars},
	numlines        => $params{numlines},
	];
  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;

  return $bot->parse_response($response->content);
}

sub cook {
  my $bot = shift;
  my $params = {quantity => 1,
		check_response => 0,
		@_};

  my $url = $bot->url . 'craft.php';
  $bot->ua->max_size(0) unless($params->{check_response});
  my $args =
      [ mode            => 'cook',
	action          => 'craft',
	pwd             => $bot->hash,
	a               => $params->{item1},
	b               => $params->{item2},
	qty             => $params->{quantity},
	];
  ($params->{makemax}) and push @$args,('max' => 'on');
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return unless $params->{check_response};

  $bot->check_response($response,'cook');

  return $bot->parse_response($response->content); 
}

sub cocktail {
  my $bot = shift;
  my $params = {quantity => 1,
		check_response => 0,
		@_};

  my $url = $bot->url . 'craft.php';
  $bot->ua->max_size(0) unless($params->{check_response});
  my $args =
      [ mode            => 'cocktail',
	action          => 'craft',
	pwd             => $bot->hash,
	a               => $params->{item1},
	b               => $params->{item2},
	qty             => $params->{quantity},
	];
  ($params->{makemax}) and push @$args,('max' => 'on');
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return unless $params->{check_response};

  $bot->check_response($response,'cocktail');

  return $bot->parse_response($response->content); 
}

sub smith {
  my $bot = shift;
  my $params = {quantity => 1,
		check_response => 0,
		@_};

  my $url = $bot->url . 'craft.php';
  my $args =
      [ mode            => 'smith',
	action          => 'craft',
	pwd             => $bot->hash,
	a               => $params->{item1},
	b               => $params->{item2},
	qty             => $params->{quantity},
	];
  ($params->{makeall}) and push @$args,('max' => 'on');
  my $response = $bot->ua->post($url,$args);

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return unless $params->{check_response};

  $bot->check_response($response,'smith');

  return $bot->parse_response($response->content); 
}

sub combine {
  my $bot = shift;
  my $params = {quantity => 1,
		check_response => 0,
		@_};

  unless(exists($params->{item1})) {
    $params->{check_response} = 1;
  }

  my $url = $bot->url . 'craft.php';
  my $args =
      [ mode            => 'combine',
	action          => 'craft',
	pwd             => $bot->hash,
	a               => $params->{item1},
	b               => $params->{item2},
	qty             => $params->{quantity},
	];
  ($params->{makemax}) and push @$args,('max' => 'on');
  my $response;
  if(exists($params->{item1})) {
    $bot->ua->max_size(0) unless($params->{check_response});
    $response = $bot->ua->post($url,$args);
    delete($bot->ua->{max_size});
  } else {
    if($params->{check_response}) {
      $response = $bot->ua->get($url,$args);
    } else {
      # Why would you do this?  You can't now.
      $response = $bot->ua->head($url,$args);
    }
  }

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  return unless $params->{check_response};

  $bot->check_response($response,'combine');

  my ($rv,$txt,$prods) = $bot->parse_response($response->content);

  # parse items
  my $inv = {};

  my $content = $response->content;
  # No meat paste
  $content =~ /<option value=0>-select an item-<\/option>/
    or return($rv,$txt,$prods);
  $content =~ s/<option value=0>-select an item-<\/option>//g;
  while($content =~ m{<option value=(\d+)>(.+?) \((\d+)\)</option>}g) {
    next if $1 == 0;
    $inv->{$1} = $3;
  }

  return ($rv,$txt,$prods,$inv);
}

sub combine_for_inventory {
  my $bot = shift;

  die "combine_for_inventory broken";

  my $url = $bot->url . 'combine.php';
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'combine_for_inventory');

  # parse items
  my $inv = {};

  my $content = $response->content;
  # No meat paste
  $content =~ /<option value=0>-select an item-<\/option>/
    or return;			# UNDEF means no meat paste
  $content =~ s/<option value=0>-select an item-<\/option>//g;
  while($content =~ m{<option value=(\d+)>(.+?) \((\d+)\)</option>}g) {
    next if $1 == 0;
    $inv->{$1} = {id => $1,
		  name => $2,
		  number => $3};
  }

  return ($inv);
}

sub uneffect {
  my $bot = shift;
  my $effect = shift;

  my $url = $bot->url . 'uneffect.php';
  my $args =
      [
       pwd => $bot->hash,
       using => 'Yep.',
       whicheffect => $effect,
	];
  my $response = $bot->ua->post($url,$args);
  $response->is_success or die "HTTP " . $response->status_line;
  $bot->check_response($response,'uneffect');

  return $bot->parse_response($response->content); 
}
  

sub parse_response {
  my $bot = shift;
  my $content = shift;

  my $text = '';
  my $prods = {};

  confess "no content" unless($content);

  my $content_save = $content;

  # burn scripts and such
  $content =~ /<body>\s*(.*?)\s*$/is or die "Nobody cares. $content";
  $content = $1;
  $content =~ s/<\/body>$//i;

  # dev code
  $content =~ s/<table><tr><td style='border: 1px solid black;'><font size=1 color=green>.*?<\/table>//ig;
  $content =~ s/\s*<div style='max-height:200px;overflow:auto;margin-left:2.5\%;margin-right:2.5\%'><font size ?= ?1>.*?<\/font><\/div>//ig;
  $content =~ s/<span class=tiny>.*?<\/span>//ig;

  $content =~ s/\s+/ /ig;
  $content =~ s/<center>//ig;
  $content =~ s/<\/center>//ig;
  $content =~ s/<td.*?>//ig;
  $content =~ s/<\/td>//ig;
  $content =~ s/<\/table>/\t/ig;
  $content =~ s/<table.*?>/\t/ig;
  $content =~ s/<br>/\t/gi;
  $content =~ s/<p>/\t/gi;
  $content =~ s/<tr>/\t/ig;
  $content =~ s/<\/tr>/\t/g;
  $content =~ s/<tr>/\t/g;

  $content =~ s/&quot;/\"/g;

  if($content =~ m{<b>The Kingdom of (Loathing|Salad)</b>}) {
    # got main.php
    return (1,"main",{});
  }

  # I'm not sure whether we need all of these.  Others may be using them
  # now so I'm not deleting them yet.
  my $mode;
  my %mode_map = ( 'Inventory:'      => 'inventory',
		   'Make Cocktails:' => 'cocktails',
		   'Cook Stuff:'     => 'cook',
		   'Smith Stuff:'     => 'smith',
		   'Combine Items:'  => 'combine',
		   'Use Multiple Items' => 'use_multiple',
		   'Use Skills:'     => 'skills',
		   'Star Chart'      => 'star_chart',
		   'The Hermit'      => 'hermit',
		   'Doc Galaktik\'s Medicine Show' => 'npc_store',
		   'The Untinker'    => 'npc_store',
		   'The Demon Market' => 'npc_store',
		   'Gouda\'s Grimoire and Grocery' => 'npc_store',
		   'The Meatsmith\s Shop' => 'npc_store',
		   'The Knob Goblin Laboratory' => 'npc_store',
		   'White Citadel'   => 'npc_store',
		   'The Entrance Cavern \(2\)' => 'ec2',
		   'Dvorak\'s Revenge' => 'dvorak',
		   'The Junkyard'    => 'junkyard',
		   'Fernswarthy\'s Basement, Level \d+' => 'basement',
		   );

# <td style="color: white;" align=center bgcolor=blue><b>Combat!</b></td>

  for(keys %mode_map) {
    if($content =~ m{<b>(Market )?Results:</b>(.*?)<b>$_</b>}) {
      $mode = $mode_map{$_};
      $content = $2;
      last;
    }
  }

  if(defined($mode) && $mode eq 'dvorak') {
    return parse_dvorak($bot,$content_save);
  }

  # Weird.
  if(!defined($mode)) {
    if ($content =~ 
	/<b>Results:<\/b>(.*?)<b>Results:<\/b>.*Innabox the Smith/i) {
      $mode = 'innabox';
      $content = $1;
    } elsif ($content =~ 
	     /<b>Results:<\/b>(.*?)<b>Results:<\/b>.*The Plunger/) {
      $mode = 'plunger';
      $content = $1;
    }
  }

  if(!defined($mode) 
     &&
     $content =~ /<form name=choiceform\d? action=choice\.php method=post>/) {
    $mode = 'adv_choice';
    if($content =~ m{^\s*<b>Results:</b>\s*<b>(.*?)</b>(.*)}) {
      $prods->{encounter} = $1;
      $content = $2;
    } elsif ($content =~ m{^\s*<b>(.*?)</b>(.*)}) {
      $prods->{encounter} = $1;
      $content = $2;
    } else {
      die "adv_choice, can\'t match $_";
    }
  }

  if(!defined($mode)
     &&
     $content =~ m{^\s*<b>Combat!</b>(.*)}) {

    if($bot->{new_combat}) {
      return $bot->parse_combat($content_save);
    }

    $mode = 'combat';
    $content = $1;
    $content =~ m{(.*?)<a href="adventure.php\?snarfblat=(\d+)">Adventure Again}
      and $content = $1,$prods->{adventure_again} = $2;
    $content =~ m{(.*?)<a href="(\w+).php">(Go back|Return)}
      and $content = $1,$prods->{return_to} = $2;
    if($content =~ m{(.*)<form name.*value=\"Attack with your .*?\">(.*)}) {
      $prods->{in_combat} = 1;
      $content = $1;
      my $options = $2;
    }
    # consider parsing options
  }

  # adventure results are results from adventuring.  This is kinda strange.

  if(!defined($mode)
     &&
     $content =~ m{^\s*<b>(Adventure )?Results:</b>\s*<b>(.*?)</b>(.*)<a href="adventure.php\?snarfblat=(\d+)">Adventure Again \((.*?)\)</a>\s*<a href="(\w+\.php(\?[^\"]+)?)">Go back to (.*?)</a>}) {
    $mode = 'adv_results';
    $prods->{encounter} = $2;
    $content = $3;
    $prods->{adventure_again} = $4; # $5 is text
    $prods->{return_to} = $6;	# $7 is text
  }

  if(!defined($mode)
     &&
     $content =~ m{^\s*<b>(Adventure )?Results:</b>(.*)<a href="adventure.php\?snarfblat=(\d+)">Adventure Again \((.*?)\)</a>\s*<a href="(\w+\.php(\?[^\"]+)?)">Go back to (.*?)</a>}) {
    $mode = 'adv_results';
    $content = $2;
    $prods->{adventure_again} = $3; # $4 is text
    $prods->{return_to} = $5;	# $6 is text
  }

  # different style adventure again/go back to links
  if(!defined($mode)
     &&
     $content =~ m{^\s*<b>(Adventure )?Results:</b>(.*)<a href="adventure.php\?snarfblat=(\d+)">Adventure Again \((.*?)\)</a>\s*<a href="(\w+\.php(\?[^\"+])?)">Go back to (.*?)</a>}) {
    $mode = 'adv_results';
    $content = $2;
    $prods->{adventure_again} = $3; # $4 is text
    $prods->{return_to} = $5;	# $6 is text
  }

  # Rats.  They're not nice.
  if(!defined($mode)) {
    if ($content =~ 
	/<b>Results:<\/b>\s+<b>(.*?)<\/b>(.*)<b>The Cellar of the Typical Tavern<\/b>/) {
      $mode = 'adv_results';
      $prods->{encounter} = $1;
      $content = $2;
      $prods->{adventure_again} = 'rats';
    }
  }
  if(!defined($mode)) {
    if ($content =~ 
	/<b>Results:<\/b>(.*left the rat faucet on down here.*)<b>The Cellar of the Typical Tavern<\/b>/) {
      $mode = 'adv_results';
      $prods->{encounter} = 'faucet';
      $content = $1;
      $prods->{adventure_again} = 'rats';
    }
  }

  if(!defined($mode)) {
    if ($content =~ 
	/<b>Results:<\/b>.*?<img src='http:\/\/images.kingdomofloathing.com\/otherimages\/hiddencity\/altar(\d).gif' alt='(An altar with a carving of a god of .*?)' title='An altar with a carving of a god of (.*?)'>(.*?)<b>The Hidden City<\/b>/i) {
      $mode = 'adv_results';
      $prods->{encounter} = $2;
      $prods->{desc} = $2;
      $content = $4;		# After that can be parsed in hiddencity()
      $prods->{adventure_again} = 'hiddencity';
      $prods->{done} = 0;
      $content =~ /Place something in the depression/i
	and $prods->{done} = 1;
    }
  }

  if(!defined($mode)) {
    if ($content =~ 
	/<b>Results:<\/b>(.*?)<b>The Hidden City<\/b>/i) {
      $mode = 'adv_results';
      $content = $1;
      $prods->{adventure_again} = 'hiddencity';
    }
  }


  if(!defined($mode)
     &&
     $content =~ /<b>Adventure Results:<\/b>(.*?)$/
     &&
     $content =~ /adventure again/i
     ) {
    print "AWM DEBUG, failed to parse adventure again\n";
    print $content,"\n";
    die;
    $mode = 'adv_results';
    $content = $1;
  }

  if(!defined($mode)
     &&
     $content =~ /<b>Adventure Results:<\/b>(.*?)$/) {
    $mode = 'adv_results';
    $content = $1;
  }

  if(!defined($mode)
     &&
     $content =~ /<b>Results:<\/b>(.*)<b>Remove an effect<\/b>/) {
    $mode = 'uneffect';
    $content = $1;
  }

  if(!defined($mode)
     &&
     $content =~ /<b>Results:<\/b>(.*?)$/) {
    return (1,"can't tell type of results, $content");
  }

  if(!defined($mode)) {
    return (1,"can't tell what happened, $content");
  }

  if($content =~ /You don\'t have any more green fluffy/) {
    return(1,"Not enough items.");
  }
  if($content =~ /That doesn\'t make any sense. Please select an item before you hit that button./) {
    return (1,"No sense (items attack)");
  }
  if($content =~ /You can\'t afford that much medicine, my friend!/) {
    return(1,"Not enough meat.");
  }
  if($content =~ /You don\'t have enough of one of the/) {
    return(1,"Not enough items.");
  }
  if($content =~ /You don\'t have that many of that item/) {
    return(1,"Not enough items.");
  }
  if($content =~ /You don\'t have the item you\'re trying to use/) {
    return(1,"Not enough items.");
  }
  if($content =~ /You don\'t have enough one of the/) {
    return(1,"Not enough items.");
  }
  if($content =~ /You don\'t have that many adv/) {
    return(1,"Not enough adventures.");
  }
  if($content =~ /Those (two )?items don\'t combine/) {
    return(1,"Nothing interesting.");
  }
  if($content =~ /Those (two )?items cannot be Meatsmithed/) {
    return(1,"Nothing interesting.");
  }
  if($content =~ /You don\'t have that skill/) {
    return(1,"You don't have that skill.");
  }
  if($content =~ /You don\'t have the skill/) {
    return(1,"You don't have the skill.");
  }
  if($content =~ /No such skill./) {
    return(1,"No such skill.");
  }
  if($content =~ /You don\'t have enough (Muscularity|Mana|Mojo) Points to use that skill./) {
    return(1,"Not enough MP.");
  }
  $content =~ /You\'re too full/i
    and return(1,"Too full.");
  $content =~ /You can\'t afford that many of that item/i
    and return(1,"You can't afford that item.");
  $content =~ /You can\'t afford that item/i
    and return(1,"You can't afford that item.");
  $content =~ /is bad for your spleen/i
    and return(1,"Bad spleen.");
  $content =~ /This part of the cyrpt is already/i
    and return(1,"already undefiled");
  $content =~ /can\'t find any additional ducks/i
    and return(1,"no more ducks");
  $content =~ /You\'ve already found the White Citadel/i
    and return(1,"Already found WC");
  $content =~ /no more ducks here/i
    and return(1,"no more ducks");
  $content =~ /no Frat soldiers left -- the way to their house is clear/i
    and return(1,"no more frats");
  $content =~ /You shouldn\'t be here/i
    and return(1,"Should not be here");
  $content =~ /more than 30 rounds,/i
    and return(1,"More than 30 rounds",{content => $content});
  if($content =~ /You must have at least (\d+) base (Moxie|Muscle|Mysticality) to equip/i) {
    return(1,"Stat fail $1 $2");
  }
  $content =~ /<!--skipagainmenu-->/
    and die "Skipagain, probably bad, $content";
  if($content =~ /You already have too many songs stuck in your head/) {
    return(1,"Too many songs.");
  }
  if($content =~ /You\'re not healthy enough to play this game\. Come back when you\'re a little better rested/i) {
    return(1,"Need HP for skeleton key");
  }
  if($content =~ /You don\'t have the item you\'re trying to equip\./) {
    return(1,"Don't have item being equipped.");
  }

  my $epic = '<img.*?adventureimages/(.*?)\.gif.*?>';
  my $ipic = '<img.*?descitem\((\d+)\).*?>';
  my $item_name='([01-9a-z \-\'()\.]|&trade;|&ntilde;)+';
  my $pl_item_name='([01-9a-z \-\'\.]|&trade;|&ntilde;)+';

  $content =~ s/\t(\(duration: (\d+) Adventures\))/$1/g;

  my @content = split(m{\t},$content);
 
  my $f = 0;
  my $open = 0;
  while(@content) {
    local $_ = shift @content;
    next if(/^\s*$/);

    $f++;

    if(/Ah, here\'s the problem. Somebody evidently left the rat faucet on down here\./i) {
      $prods->{encounter} = 'faucet';
      next;
    }

    $mode eq 'adv_results' 
      && /After all that happens, you look down and notice a ten-leaf clover/ 
      and $prods->{marmot_clover} = 1,next;

    if(m{${ipic}You acquire an item: <b>($item_name)</b>}i) {
      my $result_desc = $1;
      my $result_name = $2;
      my $ri =  $bot->items->get_item(desc => $result_desc,
				      name => $result_name,
				      number => 1);
      if(!defined($ri)) {
	print "UNDEFINED, $result_desc, $result_name\n";
	next;
      }
      $prods->{$ri->id} += 1;
      next;
    }

    if(m{${ipic}You acquire some items: <b>($pl_item_name) \(([\d,]+)\)</b>}i) {
      my $result_desc = $1;
      my $result_name = $2;
      my $number = $4;
      $number =~ s/,//g;
      my $ri =  $bot->items->get_item(desc => $result_desc,
				      name => $result_name,
				      number => $number);
      $prods->{$ri->id} += $number;
      next;
    }

    if(m{${ipic}You acquire <b>([\d,]+) ($item_name)</b>}i) {
      my $result_desc = $1;
      my $result_name = $3;
      my $number = $2;
      $number =~ s/,//g;
      my $ri =  $bot->items->get_item(desc => $result_desc,
				      name => $result_name,
				      number => $number);
      die "Couldn't get item $result_desc $result_name" unless defined($ri);
      $prods->{$ri->id} += $number;
      next;
    }

    $mode eq 'combat' 
      and $f == 1
      and m{${epic}You\'re fighting <span id='monname'>(.*)</span>}i
      and $prods->{pic} = $1,$prods->{encounter} = $2, next;

#    $mode eq 'adv_results' 
#      and !exists($prods->{encounter})
#      and m{<b>(.*)</b>}i
#      and $prods->{encounter} = $1,next;

    $mode eq 'adv_results' 
      and m|^\s*$epic\s*$|i
      and $prods->{pic} = $1,next;

    $mode eq 'combat' 
      and m{<blockquote>(.*)</blockquote>}
      and $prods->{desc} = $1,next;

    $mode eq 'adv_results' 
      and m{<blockquote>(.*)</blockquote>}
      and $prods->{desc} = $1,next;

    $mode eq 'combat' 
      and m{<blockquote>(.*)}
      and $prods->{desc} = $1,$open = 1,next;

    $mode eq 'adv_results' 
      and m{<blockquote>(.*)}
      and $prods->{desc} = $1,$open = 1,next;

    $mode eq 'use_multiple'
      and m{<blockquote>(.*)}
      and $prods->{desc} = $1,$open = 1,next;

    $mode eq 'combat' 
      and $open == 1
      and m{(.*)</blockquote>}
      and $prods->{desc} .= " " . $1,$open = 0,next;

    $mode eq 'adv_results' 
      and $open == 1
      and m{(.*)</blockquote>}
      and $prods->{desc} .= " " . $1,$open = 0,next;

    $mode eq 'combat' 
      and $open == 1
      and $prods->{desc} .= " " . $_,next;

    $mode eq 'adv_results'
      and $open == 1
      and $prods->{desc} .= " " . $_,next;

    # Hrm.  May miss things.
    $mode eq 'adv_results'
      and !exists($prods->{desc})
      and $prods->{desc} = $_,next;


    # Combat section
    if($mode eq 'combat') {
      /You attempt to run away like a sissy little coward, but you don\'t quite make it./ 
	and $prods->{failed_runaway} = 1,next;

      /You run away, like a sissy little coward\./
	and $prods->{runaway} = 1,next;

      /You twiddle your thumbs./i
	and $prods->{twiddle} = 1,next;

      /You conjure a saucy salve and quickly apply it to your wounds\./
	and $prods->{skill} = $_,next;

      my %spell_type = ( red => 'hot',blue => 'cold',
			 gray => 'spooky',green => 'stench',
			 blueviolet => 'sleaze');

      /You cast a (.*) at your opponent, which does <font color=(red|blue|gray|green|blueviolet)><b>(\d+)<\/b><\/font> damage/
	and $prods->{spell} = $1, $prods->{damage_spell} = $3,
	$prods->{damage_spell_type} = $spell_type{$2},
	next;

      /You summon a (.*), which (erupts beneath|hits|washes over) your opponent for <font color=(red|blue|gray|green|blueviolet)><b>(\d+)<\/b><\/font> damage/
	and $prods->{spell} = $1, $prods->{damage_spell} = $4,
	$prods->{damage_spell_type} = $spell_type{$3},
	next;

      /You call forth a (.*?), which strikes your opponent for (\d+) damage./
	and $prods->{spell} = $1, $prods->{damage_spell} = $2,next;

      if(/\b(\d+)\s+(?:\([^\(]+?\)\s+)*(damage|notches|points).*?(bam|barf|biff|bonk|boof|kapow|kerblam|pow|smack|socko|splat|squeak|whack|wham|whammo|zap|zot)/i 
	 ||
	 /dealing (\d+)\s+(?:\([^\(]+?\)\s+)*to your opponent.*?(bam|barf|biff|bonk|boof|kapow|kerblam|pow|smack|socko|splat|squeak|whack|wham|whammo|zap|zot)/i
	 ) {
	$prods->{damage} = $1;
	m{\(<b>\+(\d+)</b>\)}i
	  and $prods->{damage_bonus_offhand} = $1;
	m{<font color=red>CRITICAL HIT!</font>}i
	  and $prods->{critical_hit} = 1;
	while(m{<font color=(\w+)><b>\+(\d+)</b></font>}ig) {
	  $1 eq 'red' and $prods->{damage_bonus_hot} = $2,next;
	  $1 eq 'blue' and $prods->{damage_bonus_cold} = $2,next;
	  $1 eq 'gray' and $prods->{damage_bonus_spooky} = $2,next;
	  $1 eq 'green' and $prods->{damage_bonus_stench} = $2,next;
	  $1 eq 'blueviolet' and $prods->{damage_bonus_sleaze} = $2,next;
	  print "UNKNOWN DAMAGE COLOR, $1\n";
	  $prods->{damage_bonus_other} += $2;
	}
	if(exists($prods->{damage}) && $prods->{damage} == 0) {
	  print "AWM DEBUG, $_\n";
	}
	next;
      }

      if(/You work up a Thrust-Smack.*You failed to hit the monster./i) {
	$prods->{miss} = 1;
	$prods->{thrust_smack} = 1;
	next;
      }
      if(/You work up a Lunging Thrust-Smack\. You failed to hit the monster./i) {
	$prods->{lunging_thrust_smack} = 1;
	$prods->{miss} = 1;
      }

      /You work up a Thrust-Smack\./
	and $prods->{thrust_smack} = 1,next;
      /You work up a Lunging Thrust-Smack\./i
	and $prods->{lunging_thrust_smack} = 1,next;
      /You can\'t Lunging Thrust-Smack with a ranged weapon\./i
	and $prods->{failed_lunging_thrust_smack} = 1,next;
      /You can\'t Thrust-Smack with a ranged weapon\./i
	and $prods->{failed_thrust_smack} = 1,next;
#      m{hits your opponent for (\d+) damage}i
#	and $prods->{damage} = $1, next;

      m{You failed to hit the monster}i
	and $prods->{miss} = 1, next;

      m{<b>FUMBLE!</b>.*doing (\d+) damage}i
	and $prods->{fumble_damage} = $1,next;

      m{(They) get the jump on you\.}i
	and $prods->{monster_init} = 1, next;
      m{(He|It|She) gets the jump on you\.}i
	and $prods->{monster_init} = 1, next;
      m{(You) get the jump on}i
	and $prods->{monster_init} = -1, next;

      if(m{^(.*)<b><font color=red>( Eek!| Ouch!| Ugh!| Oof!| Ow!| Ooh!| Argh!){1,}</font></b>}i) {
	# previous row
	$prods->{in_damage_desc} = pop @{$prods->{unmatched}};
	next;
      }

      if(m{<img src="http://images.kingdomofloathing.com/itemimages/hp.gif" height=30 width=30>You lose (\d+) hit points?\.}i) {
	$prods->{in_damage} = $1;
	m{(cold|hot|sleaze|spelling|shadow|spooky|stench) damage}i
	  and $prods->{in_damage_type} = $1;
	next;
      }
      m/You lose (\d+) (Mana|Muscularity|Mojo) Points/i
	and $prods->{mp_loss} = $1,next;

      m{You win the fight!<!--WINWINWIN-->}i
	and $prods->{win} = 1, next;

      m{Starlight shines through your opponent, doing (\d+) damage, and pours into your body.}
        and $prods->{damage_familiar} = $1,next;
      m{shouts "Yarrrr!" and battens your opponent\'s hatches for him. Violently. For (\d+) damage.} 
        and $prods->{damage_familiar} = $1,next;

      m{kicks your opponent in the shin \(or shin-analogue\) with her clear plastic high-heels, for <font color=blueviolet><b>(\d+)</b></font> damage}i
        and $prods->{damage_familiar} = $1,next;
      m{shoots a beam of ice-cold water at your opponent, hitting him for <font color=blue><b>(\d+)</b></font> damage}i
        and $prods->{damage_familiar} = $1,next;
      m{produces a gout of flame from her fingertips, singeing your opponent for <font color=red><b>(\d+)</b></font> damage}i
        and $prods->{damage_familiar} = $1,next;

      m{pumpkin bucket.*(\d+) damage}i
	|| m{plastic bucket.*(\d+) damage}i
	|| m{shoelaces together.*takes (\d+) damage as}i
	|| m{trick.*most sensitive area.*and does (\d+) damage\.}i
	|| m{drops a bunch of ice cubes.*<font color=blue><b>(\d+)</b>}i
	|| m{it a wet willy.*<font color=blue><b>(\d+)</b>}i
	|| m{a balloon filled with water.*<font color=blueviolet><b>(\d+)</b>}i
	|| m{snaps its bra strap.*<font color=blueviolet><b>(\d+)</b>}i
	|| m{tells it a spooky story\..*<font color=gray><b>(\d+)</b>}i
	|| m{shines a flashlight.*<font color=gray><b>(\d+)</b>}i
	|| m{a creepy skeleton mask.*<font color=gray><b>(\d+)</b>}i
	|| m{throws a rotten egg.*<font color=green><b>(\d+)</b>}i
	|| m{into smelling your feet..*<font color=green><b>(\d+)</b>}i
	|| m{throws a rotten egg at it..*<font color=green><b>(\d+)</b>}i
	|| m{eats a clove of garlic.*<font color=green><b>(\d+)</b>}i
	|| m{a week-old strip of bacon.*<font color=green><b>(\d+)</b>}i
	|| m{Native American Rugburn.*<font color=red><b>(\d+)</b>}i
	|| m{a ballpoint pen really fast.*<font color=red><b>(\d+)</b>}i
      and $prods->{damage_trick} += $1,next;

      m{bleeps, "MEDICAL SYSTEMS ACTIVATED," and fires a dart into your neck. You suddenly feel much better.}
        and $prods->{fam_heal} = $1,next;

      # Use items
      m{You use the spices.} and next;

      m{You shake some spices into your opponent\'s eyes, doing (\d+) damage\. (It|He|She) winces\.}i
	and $prods->{damage_useitem} += $1,next;
      
      /hose down your opponent for <font color=red><b>(\d+)<\/b><\/font> damage\./
	and $prods->{damage_useitem} += $1,next;

      /suffering (\d+) points of spelling damage/
	and $prods->{damage_useitem} += $1,next;

      # more general


      # familiar stuff
      /gains a pound!<\/b>/
	and $prods->{fam_gain_pound}++,next;
      /does a little fairy dance\./i 
	|| /around the maypole/i
	and $prods->{fam_item}++,next;
      /flits around like a hummingbird on crack/
	and $prods->{fam_item}++,
	$prods->{fam_meat}++,next;

      /The air briefly glows octarine/
	and $prods->{fam_item}++,next;

      /hace una sonrisa brillante con sus labios de cera/
	and $prods->{fam_wax_lips}++,next;
      /curious \(and curiouser\) smile/
	and $prods->{fam_wax_lips}++,$prods->{fam_stat}++,next;
      /wax lips/
	and $prods->{fam_wax_lips}++,next;
      /bends its brim into an approximation of a smile/
	and $prods->{fam_stat}++,next;
      /disappears for a moment/i
	and $prods->{fam_meat}++,next;
      /smiles approvingly\./i
	and $prods->{fam_stat}++,next;
      /smiles unsettlingly\./i
	and $prods->{fam_stat}++,next;
      /whiff of Ovaltine/
	and $prods->{fam_stat}++,next;

      /chanting, and you feel some of your power move from you to him./
	and next;
      /familiar45.gif.*?sits motionless./
	and next;

      # spheres
      /Your Jalape&ntilde;o Saucesphere does (\d+) damage to your opponent, and infuses you with renewed health./
	and $prods->{damage_sphere} += $1,next;
      m{Your opponent takes (\d+) damage from your Saucesphere\.}i
	and $prods->{damage_sphere} += $1,next;
      /Your Jaba&ntilde;ero Saucesphere does (\d+) damage to your opponent, and infuses you with renewed energy./
	and $prods->{damage_sphere} += $1,next;

      # items
      /NanoMechaMech.*(\d+) damage/
	and $prods->{damage_nano} += $1,next;
      /leaps off of your back and sinks its fangs into your opponent, dealing (\d+) damage/
	and $prods->{damage_vampire_cape} += $1,next;


      m{You lose. You slink}i
	and $prods->{win} = -1, next;

      # Er, kludge miss bits
      m{They pause and look at each other, confused.} and next;
      m{"Hay\?" he shouts, "Where!\?"} and next;

#      my @mon_miss_patterns = (
#        'but (you|he|she|it|they|its|sees|passes|only|finds|missed|fails|misses)',
#        'but (for some|appears)',
# 			       'Nothing (bad )?happens',
# 			       'It runs straight toward you, which also happens to be straight away from you, so nothing bad happens',
#			       );
#      my $pat = join('|',@mon_miss_patterns);
#      /$pat/oi && !exists($prods->{in_damage})
# 	and $prods->{monster_miss} = 1,next;

    }
    
    # unmatched damage
    /<font color=\w+><b>(\d+)<\/b><\/font> damage/
      and die "parse_response, unmatched damage" . $_;
    /<b>(\d+)<\/b> damage/
      and die "parse_response, unmatched damage" . $_;
    /(\d+) damage/
      and die "parse_response, unmatched damage" . $_;

    if($mode eq 'adv_results') {
      /\(You must have at least (\d+) (Moxie|Mysticality|Muscle) to adventure here.\)/
	and $prods->{stat_requirement} = $1,next;
    }

    if($mode eq 'skills') {
      /A cocoon of cannelloni envelops you/
	and $prods->{desc} = $_,next;
      /You contemplate the Cosmic Sauce/
	and $prods->{desc} = $_,next;
      /make with the empathy for you/
	and $prods->{desc} = $_,next;
      /You muster up some tenacity for yourself/
	and $prods->{desc} = $_,next;
      /You summon a Leash of Linguini./
	and $prods->{desc} = $_,next;
      /You envelop your body in Springy Fusilli/
	and $prods->{desc} = $_,next;
      /play (The Polka of Plenty|The Moxious Madrigal|Fat Leon\'s Phat Loot Lyric|Jackasses\' Symphony of Destruction|Aloysius\' Antiphon of Aptitude|The Sonata of Sneakiness|Ur-Kel\'s Aria of Annoyance), with yourself as your audience/i
	and $prods->{desc} = $_,next;
      /You conjure an Elemental Saucesphere around yourself/
	and $prods->{desc} = $_,next;
      /You conjure a Jaba&ntilde;ero Saucesphere around yourself/ 
	and $prods->{desc} = $_,next;
      /You practice the accordion. Moxiousness wells up within you\./
	and $prods->{desc} = $_,next;
      /You relax\. You chill\. You become\.\.\. smooth\./
	and $prods->{desc} = $_,next;
      /You lick your wounds\./
	and $prods->{desc} = $_,next;
    }

    if($mode eq 'use_multiple' || $mode eq 'inventory') {
      /ou use the hair spray to stiffen your coiffure/
	and $prods->{desc} = $_,next;
    }

    if(m{<img class=hand src="http://images.kingdomofloathing.com/itemimages/(\w+)\.gif" onClick=\'eff\(\"([\da-f]+)\"\);\'.*?>You acquire an effect: <b>([^<]+)</b>\(duration: (\d+) Adventures\)}i) {
      $prods->{effects} ||= {};
      $prods->{effects}->{lc($3)} = {
	image => $1,
	id    => $2,
	name  => $3,
	turns => $4,
      };
      next;
    }

    if(m{<img class=hand src="http://images.kingdomofloathing.com/itemimages/(\w+)\.gif" onClick=\'eff\(\"([\da-f]+)\"\);\'.*?>You acquire an effect: <b>([^<]+)</b>}) {
      my ($image,$id,$name) = ($1,$2,$3);
      if($content[0] =~ m{Duration: (\d+) Adventures?}i) {
	shift @content;
 	my $turns = $1;
	$prods->{effects} ||= {};
	$prods->{effects}->{lc($name)} = {
	  image => $image,
	  id    => $id,
	  name  => $name,
	  turns => $turns,
	};
	next;
      }
      # even through we matched part, consider it unmatched
    }

    /You gain (\d+) (Mana|Mojo|Muscularity) points?/i
      and $prods->{mp} ||= 0, $prods->{mp} += $1, next;
    /You lose (\d+) (Mana|Mojo|Muscularity) points?/i
      and $prods->{mp} ||= 0, $prods->{mp} -= $1, next;

    if(/You gain ([\d,]+) Hit points?/i) {
      $prods->{hp} ||= 0;
      my $hpgain = $1;
      $hpgain =~ s/,//;
      $prods->{hp} += $hpgain;
      next;
    }
    if(/You lose ([\d,]+) Hit points?/i) {
      my $hploss = $1;
      $hploss =~ s/,//;
      $prods->{hp} ||= 0;
      $prods->{hp} -= $hploss;
      next;
    }

    if(m{You gain (\d+) meat}i) {
      $prods->{meat} += $1;
      $prods->{meat_count}++;
      next;
    }
    if(m{You lose ([\d,]+) meat}i) {
      my $m = $1;
      $m =~ s/,//g;
      $prods->{meat} -= $m;
      $prods->{meat_count}++;
      next;
    }

    if(/You\s+gain\s+([\d,]+)\s+adventures?/i) {
      my $a = $1;
      $a =~ s/,//g;
      $text .= "($a)";
      $prods->{adventures} = $a;
      next;
    }

    /Smoke begins to pour/
      and $text .= 'Box destroyed.',next;

    /<blockquote>You place the chef-in-the-box at your campsite. He makes himself at home\.<\/blockquote>/
      and $prods->{citb} = 1;

    /You gain (\d+) (Beef|Stren|Strong|Muscle|Fortit)/i
      and $prods->{muscle} += $1, next;
    /You gain (\d+) (Enchant|Magic|Myst|Wizard)/i
      and $prods->{myst} += $1,next;
    /You gain (\d+) (Smarm|Rogui|Chutz|Cheek|Sarcasm)/i
      and $prods->{moxie} += $1, next;
    /You lose (\d+) (Beef|Stren|Strong|Muscle|Fortit)/i
      and $prods->{muscle} -= $1, next;
    /You lose (\d+) (Enchant|Magic|Myst|Wizard)/i
      and $prods->{myst} -= $1, next;
    /You lose (\d+) (Smarm|Rogui|Chutz|Cheek|Sarcasm)/i
      and $prods->{moxie} -= $1, next;

    /<b>You gain a Level!<\/b>/
      and $prods->{level_gain} = 1,next;
    /You gain a (Mysticality|Muscle|Moxie) point/i
      and $prods->{lc($1) . '_point'} = 1,next;
    /You gain some (Mysticality|Muscle|Moxie) points/i
      and $prods->{lc($1) . '_point_more'} = 1,next;

    /You\s+gain\s+([\d,]+)\s+drunk/i
      and $prods->{drunk} = $1, next;

    /You cook up a delicious new item\./
      and next;
    /The meat paste fuses your items together\./
      and next;
    /You hear an unhealthy-sounding clicking noise coming from inside your chef-in-the-box\./
      and next;

    /^\s*<\/blockquote>\s*$/i and next;
    /You mix up a refreshing cocktail/i and next;
#    m|^<img src="http://images.(kingdomofloathing|loathing2).com/itemimages/\w+.gif" width=30 height=30>$| and next;
    m|^<img src="http://images.kingdomofloathing.com/itemimages/\w+.gif" width=30 height=30>$| and next;
    /^<blockquote>You drink the/ and next;
    /^<blockquote>You take the shot of schnapps. Shnappy!/ and next;
    /^<blockquote>You read the scroll of drastic healing.*crumbles to powder/ 
      and $prods->{sdh_gone}=1, next;
    /^<blockquote>You read the scroll of drastic healing.*you roll the scroll/i
      and $prods->{sdh_kept}=1, next;

    if($mode eq 'adv_choice') {
      m{^<img src=".*?adventureimages/(\w+)\.gif" width=100 height=100>}
        and $prods->{pic} = $1,next;

      while(m{<form name=choiceform\d? action=choice\.php method=post><input type=hidden name=pwd value='[\da-f]+'><input type=hidden name=whichchoice value=(\d+)><input type=hidden name=option value=(\d+)><input class=button type=submit value="(.*?)"></form>}g) {
	if(exists($prods->{choice_value}) && $prods->{choice_value} != $1) {
	  die "choice adventure multiple values";
	}
	$prods->{choice_value} = $1;
	exists($prods->{'choice_' . $2}) and die "same option multiple times";
	$prods->{'choice_' . $2} = $3;
      }
      /^<\/body><\/html>$/ and next;
      
      # Huh?
      $prods->{desc} ||= '';
      s{<form name=choiceform\d? action=choice\.php method=post><input type=hidden name=pwd value='[\da-f]+'><input type=hidden name=whichchoice value=(\d+)><input type=hidden name=option value=(\d+)><input class=button type=submit value="(.*?)"></form>}{}g;
      $prods->{desc} .= $_;
      next;
    }

    if($mode eq 'inventory') {
      m{You equip the <b>(.*?)</b>\.} 
        and $prods->{equip} = $1,next;
    }
    warn "couldn't match fragment $mode $f $_";
    $prods->{unmatched} ||= [];
    push @{$prods->{unmatched}},$_;
  }

  $prods->{mode} = $mode;
  return (0,$text,$prods);
}

my $IMG_FAM = '<img src="http://images.kingdomofloathing.com/itemimages/(familiar\d+.gif|hat2.gif|slgfairy.gif|hounddog.gif)" width=30 height=30>';

my %spell_type = ( red => 'hot',blue => 'cold',
		   gray => 'spooky',green => 'stench',
		   blueviolet => 'sleaze');

sub parse_combat {
  my $bot = shift;
  my $content = shift;

  my %mode_map = ('Combat!' => 'combat');

  # just strip stuff as we come to it.

  # mostly used in descriptions.
  $content =~ s/<\/?i>//g;

  my $prods;

  $content =~ m{<body>\s*(.*?)\s*</body>}s;
  $content = $1;
  $content =~ s/<table><tr><td style='border: 1px solid black;'><font size=1 color=green>(.*?)<\/font><\/td><\/tr><\/table>//g;
  $content =~ s/\s*<div style='max-height:200px;overflow:auto;margin-left:2.5\%;margin-right:2.5\%'><font size ?= ?1>.*?<\/font><\/div>//ig;
  $content =~ s/<span class=tiny>.*?<\/span>//ig;
#  $content =~ s/^<\/center><font size=1>(.*?)<\/font><center>//; #d

  # start of a blue box
  $content =~ m|^<center><table width=95%  cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=blue><b>([^<]*)</b></td></tr><tr><td style="padding: 5px; border: 1px solid blue;"><center><table><tr>(.*)$|s 
    or die "Doesn't start with a blue box! " . $content;

  $prods->{mode_desc} = $1;
  exists $mode_map{$1} or die "No mode_map for $1";
  $prods->{mode} = $mode_map{$1};
  $content = $2;

  # fighting a monster
  $content =~ s|^<td><center><table><tr><td><img id='monpic' src=\"http://images.kingdomofloathing.com/adventureimages/([^\"]+)\" width=100 height=100></td><td valign=center>You\'re fighting <span id='monname'>([^<]*)</span></td></tr></table><br>|| 
    or die "no monster " . $content;

  $prods->{pic} = $1;
  $prods->{encounter} = $2;

  # probably first round, monster description
  my $first_round = 0;
  if($content =~ s|^</center><blockquote>(.*?)</blockquote>||s) {
    $first_round = 1;
    $prods->{desc} = $1;

    # initiative
    if($content =~ s/^<center><p>You get the jump on (them|it|him|her)\.//) {
      $prods->{monster_init} = -1;
      $prods->{mon_did} = 'lost_init';
      $prods->{we_did} = 'first_round';
    } elsif ($content =~ s/^<center><p>(They|He|She|It) gets? the jump on you\.//) {
      $prods->{monster_init} = 1;
      $prods->{we_did} = 'lost_init';
    } else {
      die "I think it's first round, but no initiative " . $content;
    }
  }

  $content =~ s/^(<p>)//;

  # our action

  my $fight_finish = 0;

  # we used an item (do two later)
# <table><tr><td valign=center><img src="http://images.kingdomofloathing.com/itemimages/sphererough.gif" width=30 height=30></td><td valign=center><p>You hold the rough stone sphere up in the air.<p>It radiates a bright red light, and a gout of flame blasts out of it, roasting your opponent for <font color=red><b>9</b></font> damage.</td></tr></table>
  if($content =~ s|^<table><tr><td valign=center><img src="http://images.kingdomofloathing.com/itemimages/([^\"]+)" width=30 height=30></td><td valign=center>(.*?)</td>(</tr>)?</table>||) {
    $prods->{item1} = {};
    $prods->{item1}->{pic} = $1;
    my $text = $2;

    # kludge: rather this came out of parse_results
    $prods->{item1}->{desc} = $text; 

    $prods->{item1}->{results} = $bot->parse_results($text);

    for(keys %{$prods->{item1}->{results}}) {
      next unless $_ =~ /^(hp|\d+)$/;
      $prods->{$_} += $prods->{item1}->{results}->{$_};
    }
    # AWM: kludgy
    $prods->{damage_useitem} += $prods->{item1}->{results}->{damage} || 0;
  } elsif($content =~ /^(.*?)<b><font color=blue>( <font color=red>CRITICAL HIT!<\/font> )?( bam!| barf!| biff!| boink!| bonk!| boof!| kapow!| kerblam!| pow!| smack!| socko!| splat!| squeak!| whack!| wham!| whammo!| zap!| zot!){1,}<\/font><\/b>(.*)$/is) {

    my $check = $1;
    $content = $4;
    $prods->{critical_hit} = 1 if(defined $2);

    $check =~ s/^You work up a Thrust-Smack\. <p>//
      and $prods->{thrust_smack} = 1,$prods->{we_did} = 'thrust_smack';

    $check =~ s/^You work up a Lunging Thrust-Smack\. <p>//
      and $prods->{lunging_thrust_smack} = 1,
      $prods->{we_did} = 'lunging_thrust_smack';

    die "missed something before we acted check = $check"
      if $check =~ /<(p|tr|br|td)>/;

    defined $prods->{we_did} or $prods->{we_did} = 'attack';

    $check =~ /\b(\d+)/ or die;
    $prods->{damage} = $1;

    $check =~ m{\(<b>\+(\d+)</b>\)}i
      and $prods->{damage_bonus_offhand} = $1;
    
    while($check =~ m{<font color=(\w+)><b>\+(\d+)</b></font>}ig) {
	  $1 eq 'red' and $prods->{damage_bonus_hot} = $2,next;
	  $1 eq 'blue' and $prods->{damage_bonus_cold} = $2,next;
	  $1 eq 'gray' and $prods->{damage_bonus_spooky} = $2,next;
	  $1 eq 'green' and $prods->{damage_bonus_stench} = $2,next;
	  $1 eq 'blueviolet' and $prods->{damage_bonus_sleaze} = $2,next;
	  die "UNKNOWN DAMAGE COLOR, $1\n";
	  $prods->{damage_bonus_other} += $2;
	}
    if(exists($prods->{damage}) && $prods->{damage} == 0) {
      die "had bonus damage, but no natural damage?, $check\n";
    }

  } elsif ($content =~ /^<font color=red><b>FUMBLE!<\/b><\/font>[^<]+<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose (\d+) hit points.<\/td><\/tr><\/table><\/center>(.*)$/s) {
    $prods->{fumble_damage} = $1;
    $content = $2;
  } elsif ($content =~ s/^You call forth an? ([^<]+), which (?:strikes|hits) your opponents? for (<font color=(\w+)><b>)?(\d+)(<\/b><\/font>)? damage\.//
	   ||
	   $content =~ s/^You call forth an? ([^<]+), dealing (<font color=(\w+)><b>)?(\d+)(<\/b><\/font>)? damage to your opponents?\.//
	   ||
	   $content =~ s/^You summon an? ([^<]+), dealing (<font color=(\w+)><b>)?(\d+)(<\/b><\/font>)? damage to your opponents?\.//
	   ||
	   $content =~ s/^You summon an? ([^<]+), which (?:strikes|hits|drenches) your opponents? for (<font color=(\w+)><b>)?(\d+)(<\/b><\/font>)? damage\.//
	   ||
	   $content =~ s/^You cast an? ([^<]+), dealing (<font color=(\w+)><b>)?(\d+)(<\/b><\/font>)? damage.//
	   ) {
    $prods->{we_did} = $1;
    if(defined $3){
      exists $spell_type{$3} or die "No spell type for $3";
      $prods->{damage_spell_type} = $spell_type{$3},
    } # otherwise it's physical
    $prods->{damage_spell} = $4;

    if($content =~ s/^(<p>)*The Jalape&ntilde;o saucesphere scintillates as your spell passes through it.  You feel warmer.//) {
      $content =~ s/^<center>(.*?)<\/center>// or die $content;
      my $match = $1;
      my $skill_results = $bot->parse_results($match);
      for (keys %$skill_results) {
	die "unknown result" unless $_ eq 'hp';
	print "  $_:",$skill_results->{$_},"\n";
	$prods->{$_} = $skill_results->{$_};
      }
    }
    if($content =~ s/^(<p>)*The Jaba&ntilde;ero saucesphere glistens as your spell passes through it.  You feel recharged.//) {
      $content =~ s/^<center>(.*?)<\/center>// or die $content;
      my $match = $1;
      my $skill_results = $bot->parse_results($match);
      for (keys %$skill_results) {
	die "unknown result" unless $_ eq 'mp';
	print "  $_:",$skill_results->{$_},"\n";
	$prods->{$_} = $skill_results->{$_};
      }
    }
  } elsif ($content =~ s/^You conjure up a mass of sticky noodles to entangle your opponents?\.//) {
    $prods->{we_did} = 'entangling noodles';
  } elsif ($content =~ /^This fight is being ended because it took more than 30 rounds, which is sort of unreasonable.You twiddle your thumbs.<p>You lose.  You slink away, dejected and defeated.(.*)$/) {
    $prods->{win} = -1;
    $prods->{over_30_rounds} = 1;
    $content = $1;
    $fight_finish = 1;
  } elsif ($content =~ /^You work up a Thrust-Smack\. You failed to hit the monster.(.*)$/s) {
    $prods->{we_did} = 'ts_miss';
    $prods->{thrust_smack} = 1;
    $prods->{miss} = 1;
    $content = $1;
  } elsif ($content =~ /^You work up a Lunging Thrust-Smack\. You failed to hit the monster.(.*)$/s) {
    $prods->{we_did} = 'lts_miss';
    $prods->{lunging_thrust_smack} = 1;
    $prods->{miss} = 1;
    $content = $1;
  } elsif ($content =~ /^You failed to hit the monster.(.*)$/s) {
    $prods->{we_did} = 'miss';
    $prods->{miss} = 1;
    $content = $1;
  } elsif ($content =~ /^You run away, like a sissy little coward.(.*)$/s) {
    $prods->{we_did} = 'runaway';
    $prods->{mon_did} = 'runaway';
    $prods->{runaway} = 1;
    $content = $1;
    $fight_finish = 1;
  } elsif ($content =~ /^You attempt to run away like a sissy little coward, but you don\'t quite make it.(.*)$/s) {
    $prods->{we_did} = 'runaway_fail';
    $prods->{failed_runaway} = 1;
    $content = $1;
  } elsif ($content =~ /^This fight is being ended because it took more than 30 rounds, which is sort of unreasonable.You twiddle your thumbs.<p>You lose.  You slink away, dejected and defeated.(.*)$/) {
    $prods->{win} = -1;
    $prods->{over_30_rounds} = 1;
    $content = $1;
  } elsif ($content =~ s/^You quickly conjure a saucy salve and apply it to your wounds.<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) hit points.<\/td><\/tr><\/table><\/center>//) {
    $prods->{we_did} = 'salve_quick';
    $prods->{hp} += $1;
    $prods->{mon_did} = 'salve_quick';
  } elsif ($content =~ s/^You conjure a saucy salve and apply it to your wounds.<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) hit points.<\/td><\/tr><\/table><\/center>//) {
    $prods->{we_did} = 'salve';
    $prods->{hp} += $1;
  } elsif ($content =~ s/^You take a deep breath, allowing your opponent\'s scent to fill your entire being.//) {
    $content =~ s/^<center>(.*?)<\/center>// or die;
    my $match = $1;
    $prods->{we_did} = 'skill_19'; # trascendent olfaction
    my $skill_results = $bot->parse_results($match);
    for (keys %{$skill_results->{effects}}) {
      print "  $_:",$skill_results->{effects}->{$_},"\n";
      $prods->{effects}->{$_} = 
	$skill_results->{effects}->{$_};
    }
  } else {
    die "Can't figure out what we did, $content"
      unless $first_round;
  }

  $content =~ s/^(<p>)+//;

  if($content =~ s/^With a muffled &quot;blahhhh,&quot; your cape suddenly leaps off of your back and sinks its fangs into your opponent, dealing (\d+) damage\.  It returns to your back, and you feel strangely refreshed\.//) {
    $prods->{damage_vampire_cape} += $1;

    $content =~ s/^<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) hit points?.<\/td><\/tr><\/table><\/center>//;
    $prods->{hp} += $1;
  }

  $content =~ s/^(<p>)+//;

  # combat familiars act now, combat effects of familiar equipment
  # note, one captured field in $IMG_FAM

  while($content =~ m|^<!--familiarmessage--><center><table><tr><td align=center valign=center>$IMG_FAM</td><td valign=center>(.*?)</td></tr></table></center>(.*)$|s) {
    $prods->{familiar_pic} = $1;
    $prods->{familiar_desc} ||= [];
    push @{$prods->{familiar_desc}}, $2;
    $content = $3;
    $content =~ s/^(<p>)//;
    if($prods->{familiar_desc}->[-1] =~ m{<font color=(red|blue|gray|green|blueviolet)><b>(\d+)</b></font> damage}) {
      $prods->{damage_familiar} += $2;
    } elsif ($prods->{familiar_desc}->[-1] =~ m|<b>(\d+)</b> damage|) {
      $prods->{damage_familiar} += $1;
    } elsif ($prods->{familiar_desc}->[-1] =~ m|(\d+) damage|) {
      # due to plastic pumpkin bucket
      $prods->{damage_familiar} += $1;
    } else {
      die "No familiar damage? " . join(':',@{$prods->{familiar_desc}});
    }
  }

  $content =~ s/^(<p>)+//;

  # DUPLICATED CODE: LATER, after monster attacks
  my @continuing_damage_desc = 
    ( "You\'re on fire!",
      "Can\'t... breathe...  Talking... like... Shatner...",
      "No, seriously.  You don\'t feel so well.  Those toads are bad for you.",
      );
				
  $content =~ s/^(<p>)+//;
  for my $cdd (@continuing_damage_desc) {
    if($content =~ /^$cdd<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose ([\d,]+) hit points. \(<font color=red><b>hot damage<\/b><\/font>\)<\/td><\/tr><\/table><\/center>(.*)$/s) {
      my $hploss = $1;
      $prods->{hp} -= $hploss;
      $content = $2;
      $hploss =~ s/,//g;
    }
    if($content =~ /^$cdd<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose ([\d,]+) hit points.<\/td><\/tr><\/table><\/center>(.*)$/s) {
      my $hploss = $1;
      $prods->{hp} -= $hploss;
      $content = $2;
      $hploss =~ s/,//g;
    }
  }

  $content =~ s/^(<p>)+//;

  if($content =~ s/^You pat out the flames\.//) {
    $prods->{flames_out} = 1;
  }

  $content =~ s/^(<p>)+//;

  # what does the monster do?
  if($content =~ m|^<center>You win the fight!<!--WINWINWIN-->(.*)$|s) {
    $prods->{mon_did} = 'lost';
    $prods->{win} = 1;
    $content = $1;
    $fight_finish = 1;
  }

  $content =~ s/^(<p>)+//;

  if($fight_finish) {

    # "Gains a pound" messages come here.
    $content = $bot->parse_combat_familiar($content,$prods);

    if($content =~ s/^(<p>)?You burp, and it tastes like snowcone.\s+Refreshing!//) {
      $content =~ s/^<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) hit points?\.<\/td><\/tr><\/table><\/center><center><Table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/mp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) Mana Points?\.<\/td><\/tr><\/table><\/center>// or die "snowcone bit wrong";
      $prods->{hp} += $1;
      $prods->{mp} += $2;
      $prods->{purple_snowcone} = 1;
    }

    if($content =~ s/^(<p>)?The mystical pyramid dissipates\. As your third eye closes, it spots some extra items lying nearby\.//) {
      $prods->{navel_drop} = 1;
    }
    if($content =~ s/^(<p>)?You notice some lint in your navel\.  You go to pick it out, and get a shock of static electricity!//) {
      $content =~ s/^<center><Table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/mp.gif" height=30 width=30><\/td><td valign=center class=effect>You gain (\d+) Mana Points.<\/td><\/tr><\/table><\/center>// or die;
      $prods->{mp} += $1;
    }

    if($prods->{win} && $content =~ /^([^<]+)(<.*)$/s) {
      my $extra_kill = $1;
      $content = $2;

      my %extra_kill_map = 
	(
	 # As Frat
	 'drive up, shove three hippies into' => 3,
	 'vodka and pain for a trio of charging hippies' => 3,
	 'Sergeant hose down three hippies with' => 3,
	 'frat brothers line up three hippies' => 3,
	 'brothers douse a trio of nearby hippies' => 3,
	 'lob a sake bomb into a trio' => 3,

	 # As Hippy
	 'three attacking frat boy warriors' => 3,
	 'watch him and two of his fortunate buddies' => 3,
	 'take down three frat boys in a whirl' => 3,
	 'urine-based bombing run over three frat boys' => 3,
	 'which turns into a three-way concussion' => 3,
	 'three frat boys abruptly vanish' => 3,

	 );
      for(keys %extra_kill_map) {
	if($extra_kill =~ /$_/i) {
	  $prods->{extra_kill} = $extra_kill_map{$_};
	}
      }
      die "Extra kill content $extra_kill unmatched."
	unless defined $prods->{extra_kill};
    }
    if($content =~ s{^<table><tr><td><center><img src="http://images.kingdomofloathing.com/adventureimages/okeyes.gif" width=100 height=100></center>Wow!  You froze that hobo so hard that he shattered into a million pieces!  All that's left is a pair of frozen eyeballs.<p>Richard pops out of a nearby steam tunnel hatch and grabs the eyes.  &quot;Lookin' good, kid!&quot; he says with a wink, as he ducks back down the hatch.</td></tr></table>}{}) {
      $prods->{hobo_eye} = 1;
    }
    if($content =~ s{^<table><tr><td><center><img src="http://images.kingdomofloathing.com/adventureimages/okboots.gif" width=100 height=100></center>Boy, you sure scorched that hobo good!  All that\'s left is a smoking pair of boots!<p>Richard emerges from a nearby steam tunnel entrance, winks at you, grabs the boots, and disappears again.</td></tr></table>}{}) {
      $prods->{hobo_boots} = 1;
    }
    if($content =~ s{^<table><tr><td><center><img src="http://images.kingdomofloathing.com/adventureimages/okskin.gif" width=100 height=100></center>Man, you really hit that hobo hard!  So hard, in fact, that you knocked him completely out of his skin, which is now lying in a wrinkly heap on the ground.<p>Richard pops his head out of a nearby steam tunnel entrance, says &quot;Hey, good, I\'ve been looking for one of these,&quot; grabs the skin, and heads back down into the tunnels.</td></tr></table>}{}) {
      $prods->{hobo_skin} = 1;
    }

    $content = $bot->parse_combat_familiar($content,$prods);

    $content =~ s/^(<p>)+//;

    if($content =~ m|^<center><table><tr><td><img src="http://images.kingdomofloathing.com/itemimages/meat.gif" width=30 height=30></td><td align=center>You gain (\d+) Meat.</td></tr></table></center>(.*)$|s) {
      $prods->{meat} = $1;
      $content = $2;
    }

    $content =~ s/^(<p>)+//;

    $content = $bot->parse_combat_familiar($content,$prods);

    while($content =~ m|^<center><table><tr><td><img src="http://images.kingdomofloathing.com/itemimages/([^\"]+)" alt="[^"]+" title="[^"]+" class=hand onClick='descitem\((\d+)\)'></td><td valign=center class=effect>You acquire an item: <b>([^<]+)</b></td></tr></table></center>(.*)|s) {
      my $item_pic = $1;
      my $item_desc = $2;
      my $item_name = $3;
      $content = $4;

      my $ri =  $bot->items->get_item(desc => $item_desc,
				      name => $item_name,
				      number => 1);
      if(!defined($ri)) {
	print "UNDEFINED ITEM $item_desc, $item_name\n";
	die;
      }
      $prods->{$ri->id} += 1;
      next;
    }

    if($content =~ s/^\((\d+) of (\d+) found\.\)//) {
      # BHH item?!
    }

    $content =~ s/^(<p>)//;

    $content = $bot->parse_combat_familiar($content,$prods);

    $content =~ s/^(<p>)//;

    if($content =~ s/^As the mystical pyramid dissipates, you smile broadly, your brain chock-full \(and chock-a-block\) with new pyramid schemes.//) {
      $prods->{navel_stat} = 1;
    }

    $content =~ s/^(<p>)//;

    if($content =~ /^You gain (\d+) (Fortitude|Strongness|Muscleboundness|Strengthliness|Beefiness)\.<bR>(.*)$/s) {
      $prods->{muscle} += $1;
      $content = $3;
      $content =~ s/^<b>You gain a Muscle point!<\/b><br>//;
    }
    if($content =~ s/^<b>You gain a Level!<\/b>//i) {
      $prods->{level_gain} = 1;
    }
    $content =~ s/^(<p>)+//;
    if($content =~ /^You gain (\d+) (Wizardliness|Magicalness|Mysteriousness|Enchantedness)\.<bR>(.*)$/s) {
      $prods->{myst} += $1;
      $content = $3;
      $content =~ s/^<b>You gain a Mysticality point!<\/b><br>//;
      $content =~ s/^<b>You gain some Mysticality points!<\/b><br>//;
    }
    if($content =~ s/^<b>You gain a Level!<\/b>//i) {
      $prods->{level_gain} = 1;
    }
    $content =~ s/^(<p>)+//;
    if($content =~ s/^(<br>)?You gain (\d+) (Cheek|Chutzpah|Roguishness|Sarcasm|Smarm)\.<bR>//) {
      $prods->{moxie} += $2;
      $content =~ s/^<b>You gain a Moxie point!<\/b><br>//;
    }
    if($content =~ s/^<b>You gain a Level!<\/b>//i) {
      $prods->{level_gain} = 1;
    }
    $content =~ s/^(<p>)+//;

    if($content =~ /^<a href="adventure.php\?snarfblat=(\d+)">Adventure Again \(([^<]+)\)<\/a><p><a href="([^<]+\.php(\?place=[^\"]+)?)">Go back to ([^<]+)<\/a><\/center>(.*)$/s) {
      $prods->{adventure_again} = $1; # $2 is text
      $prods->{return_to} = $3;	# $4 is text
      $content = $6;
    } elsif ($content =~ /^<center><p><a href="(.*?)\.php">Go back to ([^<]+)<\/a><\/center>(.*)$/s) {
      $prods->{return_to} = $1;	# $2 is text
      $content = $3;
    } elsif ($content =~ /^<\/center><center><a href="rats.php">Go back to the Cellar<\/a><\/center>(.*)$/s) {
      # odd, why does it start with </center>?
      $prods->{return_to} = 'rats';	# $2 is text
      $content = $1;
    } elsif ($content =~ s/^<a href="basement.php">Go back to Fernswarthy\'s Basement<\/a><\/center>//) {
      $prods->{return_to} = 'basement';
    } else {
      print "\nNo go back:\n";
      for(keys %$prods) {
	print "$_: ",$prods->{$_},"\n";
      }
      print "content:\n$content\n";
      die "No go back?";
    }

    $content =~ m|^<script>parent.charpane.location.href="charpane.php";</script></td></tr></table></center></td></tr><tr><td height=4></td></tr></table>(\s*</center>)?\s*$|s
      or die "Ending content wrong $content";

    return (0,'',$prods);
  }

  # we didn't win (yet?!)

  # their action

  $content =~ s/^(<p>)+//;
  $content =~ s/^Your opponents? struggles? free of your entangling noodles.//;
  $content =~ s/^(<p>)+//;

  if($content =~ s/^(<p>)*Your Jalape\&ntilde;o Saucesphere deals <font color=(blueviolet|gray|blue|red)><b>(\d+)<\/b><\/font> damage to your opponent\.//) {
    $prods->{damage_sphere} += $3;
  }
  if($content =~ s/^(<p>)*Your Jalape\&ntilde;o Saucesphere deals (\d+) damage to your opponent\.//) {
    $prods->{damage_sphere} += $2;
  }

  if($content =~ s/^(<p>)*Your Jaba\&ntilde;ero Saucesphere deals <font color=(blueviolet|gray|blue|red)><b>(\d+)<\/b><\/font> damage to your opponent\.//) {
    $prods->{damage_sphere} += $3;
  }
  if($content =~ s/^(<p>)*Your Jaba\&ntilde;ero Saucesphere deals (\d+) damage to your opponent\.//) {
    $prods->{damage_sphere} += $3;
  }

  $content =~ s/^(<p>)+//;

  if($content =~ /^<table><tr><td>(.*?)<\/td><\/tr><\/table><b><font color=red>( Eek!| Ouch!| Ugh!| Oof!| Ow!| Ooh!| Argh!){1,}<\/font><\/b><center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose ([\d,]+) hit points?.( \(<font color=(blueviolet|red|blue|green|gray|purple)><b>(cold|hot|sleaze|bad spelling|shadow|spooky|stench) damage<\/b><\/font>\))?<\/td><\/tr><\/table><\/center>(.*)$/s) {
    $prods->{mon_did} = 'hit';
    $prods->{in_damage_desc} = $1;
    $prods->{in_damage} = $3;
    $content = $7;
    defined $4 and $prods->{in_damage_type} = $6;
    $prods->{in_damage} =~ s/,//g;

    # m{(cold|hot|sleaze|spelling|shadow|spooky|stench) damage}i
    # and 
  }elsif($content =~ /^<table><tr><td>(.*?)<\/td><\/tr><\/table><b><font color=red><font color=blue>Br+\.<\/font><\/font><\/b><center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose ([\d,]+) hit points?.( \(<font color=(blueviolet|red|blue|green|gray)><b>(cold|hot|sleaze|spelling|shadow|spooky|stench) damage<\/b><\/font>\))?<\/td><\/tr><\/table><\/center>(.*)$/s) {
    # cold damage Brrs instead of oof owing.
    $prods->{mon_did} = 'hit';
    $prods->{in_damage_desc} = $1;
    $prods->{in_damage} = $2;
    $content = $6;
    defined $3 and $prods->{in_damage_type} = $5;
    $prods->{in_damage} =~ s/,//g;

  } elsif($content =~ m|(The grill sergeant pulls out a bottle of lighter fluid, hoses you down with it, and lights a match.  You can guess what ensues.)(<.*)$|) {
    $prods->{mon_did} = 'fire';
    $content = $2;
  } elsif ($content =~ s/^Your opponents? struggles? against your entangling noodles, but to no avail\.//) {
    $prods->{mon_did} = 'mon_paralyzed';
  } elsif ($content =~ s/^(He beats you about the head and neck with his bindle.  There are a lot of objects in there, and several of them are really sharp!|He clenches a fist made rock-hard through years of migratory labor, and thumps you on the head with it.|He knees you in the primary sexual characteristics.  Ow!|He says "say 'rock,'" and headbutts you.|He beans you on the head with a railroad spike, then eats some beans and goes to the head.|He gives you an open-palm smack on the face.  His calloused hands scratch your sensitive flesh.|He pulls a beer can out of his bindle and throws it at you.  Turns out it was full!|He kicks you with his hobnailed boots.  The multi-colored mirrors lacerate your flesh.|He picks up a rock, looks at it while muttering something inaudible, then throws it at you.|He throws an elbow at you.  His elbow.  It catches you in the [^\.]+.)//) {
    $prods->{mon_did} = 'special';
    $prods->{in_damage_desc} = $1;
    $content =~ s/^<center>(.*?)<\/center>// or die;
    my $match = $1;
    my $skill_results = $bot->parse_results($match);
    for (keys %$skill_results) {
      die "unknown result" unless $_ eq 'hp';
      print "  $_:",$skill_results->{$_},"\n";
      $prods->{$_} = $skill_results->{$_};
    }
    $prods->{in_damage} = $skill_results->{hp};
  }

  if($content =~ /^[^<]*<center><table><tr><td><img class=hand src="http:\/\/images.kingdomofloathing.com\/itemimages\/([^\"]+).gif" onClick='eff\("([\da-f]+)"\);' width=30 height=30><\/td><td valign=center class=effect>You acquire an effect: <b>([^<]+)<\/b><br>\(duration: (\d+) Adventures?\)<\/td><\/tr><\/table><\/center>(.*)$/s) {
    while($content =~ s/^[^<]*<center><table><tr><td><img class=hand src="http:\/\/images.kingdomofloathing.com\/itemimages\/([^\"]+).gif" onClick='eff\("([\da-f]+)"\);' width=30 height=30><\/td><td valign=center class=effect>You acquire an effect: <b>([^<]+)<\/b><br>\(duration: (\d+) Adventures?\)<\/td><\/tr><\/table><\/center>//) {
      my ($image,$id,$name,$turns) = ($1,$2,$3,$4);
      $content =~ s/^(<p>)+//;

      print "AWM DEBUG, acquiring effect: $image $id $name $turns\n";

      $prods->{monster_attack_effects}->{lc($name)} = {
	image => $image,
	id    => $id,
	name  => $name,
	turns => $turns,
      };

      $prods->{effects}->{lc($name)} = 
        {%{$prods->{monster_attack_effects}->{lc($name)}}};
    }
    exists $prods->{mon_did} and $prods->{mon_did} .= ' give_effect';
    $prods->{mon_did} = 'give_effect';
  } 

  $content =~ s/^(<p>)+//;

  if($content =~ s|^Concentrating on your navel, you zoom out of the way, and your opponent is unable to reach you\.||) {
    if(! exists($prods->{mon_did})) {
      $prods->{mon_did} = 'navel_miss';
    }
  }

  $content =~ s/^(<p>)+//;

  if(! exists $prods->{mon_did}
     && ($content =~ m|^<table><tr><td>([^<]*?)</td></tr></table>(.*)$|s
	  ||
	  $content =~ m|^<table><tr><td>([^<]*?<b>[^<]*?<\/b>[^<]*?)</td></tr></table>(.*)$|s)
	  ) {
    $prods->{mon_did} = 'miss';
    my $attackstring = $1;
    $content = $2;
    # otherwise assume it's a miss.  
    $prods->{monster_miss} = 1;
    $prods->{monster_miss_mess} = $attackstring;
    # backward compatibility AWM
    $prods->{unmatched} ||= [];
    push @{$prods->{unmatched}},$attackstring;
  } elsif($content =~ m|^([^<]+?)(<.*)$|s) {
    # These are not really misses, but no clue how they'll be handled
    my $attackstring = $1;
    $content = $2;
    unless (lc($prods->{encounter}) eq lc('a Naughty Sorority Nurse')
	    ||
	    lc($prods->{encounter}) eq lc('A War Hippy Fire Spinner')
	    ||
	    lc($prods->{encounter}) eq lc('a War Hippy Elite Fire Spinner')
	    ||
	    lc($prods->{encounter}) eq lc('a Quiet Healer')
	    ) {
      print "Attackstring is $attackstring, encounter is ",
        $prods->{encounter},"\n";
      die "Weird miss message";
    }
    $prods->{mon_did} = 'special';
    $prods->{monster_miss} = 1;
    $prods->{monster_miss_mess} = $attackstring;
    # backward compatibility AWM
    $prods->{unmatched} ||= [];
    push @{$prods->{unmatched}},$attackstring;
  }

  if(! exists $prods->{mon_did} && !$prods->{runaway}) {
    print "\nNo clue what the monster did, prods:\n";
    for(keys %$prods) {
      print "$_: ",$prods->{$_},"\n";
    }
    print "content:\n$content\n";
    die "no clue what the monster did";
  }

  # DUPLICATED ABOVE
  $content =~ s/^(<p>)+//;
  for my $cdd (@continuing_damage_desc) {
    if($content =~ /^$cdd<center><table><tr><td><img src="http:\/\/images.kingdomofloathing.com\/itemimages\/hp.gif" height=30 width=30><\/td><td valign=center class=effect>You lose ([\d,]+) hit points.( \(<font color=red><b>hot damage<\/b><\/font>\))?<\/td><\/tr><\/table><\/center>(.*)$/s) {
      my $hploss = $1;
      $prods->{hp} -= $hploss;
      $content = $3;
      $hploss =~ s/,//g;
    }
  }

  $content =~ s/^(<p>)+//;

  if($content =~ s/^You lose.  You slink away, dejected and defeated.(<p><p><a href="adventure.php\?snarfblat=(\d+)">Adventure Again \(([^<]+)\)<\/a>)?<p><a href="([^<]+\.php(\?place=[^\"]+)?)">Go back to ([^<]+)<\/a>(<\/center>)?//) {
    if(defined $1) {
      $prods->{adventure_again} = $2; # $3 is text
    }
    $prods->{return_to} = $4;
    $prods->{win} = -1;
    $content =~ m|^<script>parent.charpane.location.href="charpane.php";</script></td></tr></table></center></td></tr><tr><td height=4></td></tr></table>(\s*</center>)?\s*$|s
      or die "Ending content wrong $content";

    return (0,'',$prods);
  }
  if($content =~ s/^You lose.  You slink away, dejected and defeated.<p><p><a href="basement.php">Go back to ([^<]+)<\/a><\/center>//) {
    $prods->{adventure_again} = 'basement';
    $prods->{return_to} = $1;
    $prods->{win} = -1;
    $content =~ m|^<script>parent.charpane.location.href="charpane.php";</script></td></tr></table></center></td></tr><tr><td height=4></td></tr></table>(\s*</center>)?\s*$|s
      or die "Ending content wrong $content";

    return (0,'',$prods);
  }

  $content =~ s/^\s*//;

  # we should have actions to attack here
  # picurl?!
  if($content =~ m|^<center><table><form name=attack action=fight.php method=post><input type=hidden name=action value="attack"><tr><td align=center><input id='tack' picurl=[^>]+ class=button type=submit value="Attack with your ([^\"]+)"></td></tr></form>(.*)$|s) {
    $prods->{in_combat} = 1;

    $prods->{actions}->{attack} = $1;
    $content = $2;

    if($content =~ /^<form name=steal action=fight.php method=post><input type=hidden name=action value="steal"><tr><td align=center><input class=button type=submit value="Pick (His|Its|Her|Their) Pocket"><\/td><\/tr><\/form>(.*)$/s) {
      $prods->{actions}->{steal} = 1;
      $content = $2;
    }

    # might go before steal, dunno
    if($content =~ s/^<form name=chefstaffform action=fight.php method=post><input type=hidden name=action value="chefstaff"><tr><td align=center><input id='chefstaffbutton' picurl=\w+ class=button type=submit value="[^\"]+"><\/td><\/tr><\/form>//) {
      $prods->{actions}->{jiggle} = 1;
    }

    # empty useitem form
    if($content =~ s/^<form name=useitem action=fight.php method=post><input type=hidden name=action value="useitem"><\/form>//) {
    } elsif($content =~ m|^<form name=useitem action=fight.php method=post><input type=hidden name=action value="useitem"><tr><td align=left><select name=whichitem>(.*?)</select> <input class=button type=submit value="Use Item"></td></tr></form>(.*)$|s) {
      $prods->{actions}->{useitem} = {};
      my $options = $1;
      $content = $2;
      
      while($options =~ m|<option\s+value=(\d+)>([^<]+?) \((\d+)\)</option>|g) {
	$prods->{actions}->{useitem}->{$1} = { id => $1,
					       name => $2,
					       number => $3 };
      }
    } elsif ($content =~ m|^<form name=useitem action=fight.php method=post><input type=hidden name=action value="useitem"><tr><td align=left>(.*?)</select><br><select name=whichitem2>(.*?)</select> <input class=button type=submit value="Use Item\(s\)"></td></tr></form>(.*)$|s) {
      $prods->{actions}->{useitem} = {};
      my $options = $1;
      $content = $3;
      while($options =~ m|<option\s+value=(\d+)>([^<]+?) \((\d+)\)</option>|g) {
	$prods->{actions}->{useitem}->{$1} = { id => $1,
					       name => $2,
					       number => $3 };
      }
      $prods->{actions}->{useitem2} = {%{$prods->{actions}->{useitem}}};
    } else {
      die "can't find useitem $content";
    }

    if($content =~ m|^<form name=skill action=fight.php method=post><input type=hidden name=action value="skill"><tr><td align=center><select name=whichskill>(.*?)</select> <input class=button type=submit value="Use Skill"></td></tr></form>(.*)$|s) {
      $prods->{actions}->{skill} = {};
      my $options = $1;
      $content = $2;

      while($options =~ m{<option\s+value=(\d+)>([^<]+?) \((\d+) (Mojo|Mana|Muscularity) Points\)</option>}g) {
	$prods->{actions}->{skill}->{$1} = { id => $1,
					     name => $2,
					     mp => $3 };
      }
    }

    $content =~ m{^<form name=runaway action=fight.php method=post><input type=hidden name=action value="runaway"><tr><td align=center><input class=button type=submit value="Run Away"></td></tr></form>(.*)}s or die "can't find runaway $content";
    $prods->{actions}->{runaway} = 1;
    $content = $1;
  } else {
    print "prods:\n";
    for(keys %$prods) {
      print "$_: ",$prods->{$_},"\n";
    }
    die "Neither won, lost, nor have combat options ",$prods->{mon_did},
      "\n$content";
  }

  # END
  $content =~ m|^</table></center></td></tr></table></center></td></tr><tr><td height=4></td></tr></table>(</center>)?\s*</center>\s*$|s 
    or die "Failed to fully parse combat $content";

  return (0,'',$prods);
}

sub parse_combat_familiar {
  my $bot = shift;
  my $content = shift;
  my $prods = shift;

  # meat enhancing familiars
  while($content =~ m|^<!--familiarmessage--><center><table><tr><td align=center valign=center>$IMG_FAM</td><td valign=center>(.*?)</td></tr></table></center>(.*)$|s) {
    my $fam_content = $2;
    $content = $3;

    $fam_content =~ s/^<b>([^<]+) gains a pound!<\/b>//
      and $prods->{fam_gain_pound}++;

    # meat drop enhancement
    if($fam_content =~ s/^(<p>)?([^<]+) flits around like a hummingbird on crack, helping you pick stuff up\.//) {
      $prods->{fam_item}++;
      $prods->{fam_meat}++;
    }

    $fam_content =~ s/^<p>([^<]+) disappears for a moment, and reappears carrying some extra Meat.//
      and $prods->{fam_meat}++;


    # item drop enhancement
    $fam_content =~ s/^<p>([^<]+) does a little fairy dance. It's rather unlike the dances you've seen other fairies do.<p>//
	and $prods->{fam_item}++;
    $fam_content =~ s/^<p>([^<]+) does a little fairy dance.//
	and $prods->{fam_item}++;
    $fam_content =~ s/^<p>([^<]+) does a couple of karate moves, then swivels his hips and gyrates his pelvis.//
	and $prods->{fam_item}++;

    # familiar equipment
    $fam_content =~ s/^[^<]+ does a spastic, arrhythmic[^<]+house\.//
	and $prods->{fam_id_maypole}++;
    $fam_content =~ s/^([^<]+) holds out the plastic pumpkin bucket, hoping for extra goodies.//
      and $prods->{fam_id_pumkin}++;

    $fam_content =~ s/^<b>([^<]+) gains a pound!<\/b>//
      and $prods->{fam_gain_pound}++;

    # stats enhancement
    $fam_content =~ s/^([^<]+) hace una sonrisa brillante con sus labios de cera.//
      and $prods->{fam_wax_lips}++;

    $fam_content =~ s/^(<p>[^<]{1,20} smiles approvingly|<p>[^<]+ smiles unsettlingly|[^<]+ approximation of a smile).//
      and $prods->{fam_stat}++;

    $fam_content =~ s/^[^<]{1,20} disappears and reappears, leaving only the wax lips floating in the air in a curious \(and curiouser\) smile\.//
      and $prods->{fam_stat}++;

    $fam_content =~ s/^(<p>)?[^<]{1,20} holds out the plastic pumpkin bucket, hoping for extra goodies\.//
      and $prods->{fam_id_pumkin}++;

    die "Still familiar content: " . $fam_content
      unless $fam_content eq '';
  }

  return $content;
}

sub parse_results {
  my $bot = shift;
  my $content = shift;

  my $prods = {};

  $content =~ s/\s+/ /g;
  $content =~ s/<p>/\t/ig;
  $content =~ s/<br>/\t/ig;
  $content =~ s/<tr>/\t/ig;
  $content =~ s/<td>/\t/ig;

  # want to include flavor text
  my @content = split (/\t/,$content);
  while(@content) {
    local $_ = shift @content;
    next if /^\s*$/;

    if(m|<img src="http://images.kingdomofloathing.com/itemimages/hp.gif" height=30 width=30></td><td valign=center class=effect>You gain ([\d,]+) hit points.|) {
      $prods->{hp} += $1;
      $prods->{hp} =~ s/,//g;
    }

    m|It radiates a bright yellow light, and a bolt of lightning arcs towards your opponent. The shockwave throws it to the ground, causing (\d+) damage.|
      and $prods->{damage} = $1;

    while(m|<center><table><tr><td><img src="http://images.kingdomofloathing.com/itemimages/([^\"]+)" [^>]+ onClick='descitem\((\d+)\)'></td><td valign=center class=effect>You acquire an item: <b>([^<]+)</b></td></tr></table></center>|g) {
      my $item_pic = $1;
      my $item_desc = $2;
      my $item_name = $3;

      my $ri =  $bot->items->get_item(desc => $item_desc,
				      name => $item_name,
				      number => 1);
      if(!defined($ri)) {
	print "UNDEFINED ITEM $item_desc, $item_name\n";
	die;
      }
      $prods->{$ri->id} += 1;
      next;
    }

    if(m{<img class=hand src="http://images.kingdomofloathing.com/itemimages/(\w+)\.gif" onClick=\'eff\(\"([\da-f]+)\"\);\'.*?>You acquire an effect: <b>([^<]+)</b>}) {
      my ($image,$id,$name) = ($1,$2,$3);
      if($content[0] =~ m{Duration: (\d+) Adventures?}i) {
	shift @content;
	my $turns = $1;
	$prods->{effects} ||= {};
	$prods->{effects}->{lc($name)} = {
	  image => $image,
	  id    => $id,
	  name  => $name,
	  turns => $turns,
	};
	next;
      }
    }
  }

  return $prods;
}

sub parse_dvorak {
  my $bot = shift;
  my $content = shift;

  if($content =~ m{(You jump to the last letter, and put your pom-poms down with a sign of relief -- thank goodness that\'s over. Worst. Spelling bee. Ever.)}) {
    my $text = '';
    my $prods = { mode => 'adv_results',
		  desc => $1,
		  encounter => 'Dvorak\'s Revenge End',
		};
    return(0,$text,$prods);
  }
    

  unless($content =~ /tiles.php\?action=jump&whichtile=/) {
    print "$content\n";
    die "no choice value";
  }

  my $text = '';
  my $prods = { mode => 'adv_choice',
		encounter => 'Dvorak\'s Revenge',
	      };

  $content =~ /(.*)rightarrow.gif/;
  my $content_pre_arrow = $1;

  # count how many greyed rows before arrows
  my $grey_count = 0;
  while($content_pre_arrow =~ /cell greyed/g) {
    $grey_count ++;
  }

  my $row_on = 6-$grey_count/9;

  $prods->{choice_value} = 'dvorak' . $row_on;
  $prods->{desc} = 'Placeholder for Dvorak\'s Revenge AWM Kludge dvorak' 
    . $row_on;

  while($content =~ m{<a class=nounder href='tiles.php\?action=jump&whichtile=(\d+)'><img src="http://images.kingdomofloathing.com/itemimages/.*?\.gif" width=30 height=30 border=0 alt='Tile labeled "(.*?)"'>}g) {

    exists($prods->{'choice_' . $1}) and die "same option multiple times";
    $prods->{'choice_' . $1} = $2;
  }
  return(0,$text,$prods);
}

sub tiles {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'tiles.php?action=jump&whichtile=' 
    . $params{whichtile};

  my $response = $bot->ua->get($url);

  $bot->check_response($response,'choice');
  return $bot->parse_response($response->content);
}
  

sub use_campground_skill {
  my $bot = shift;
  my $skill = shift;

  my $url = $bot->url . 'campground.php';
  $bot->ua->max_size(0);
  my $args =
      [
	action          => 'skill'.$skill,
	];
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;
}


#
# returns a reference to an array of items with a number field added
#

sub get_inventory {
  my $bot = shift;
  my $page = shift;
  my $equip = shift;
  my $no_get_item = shift;

  $bot->ua->timeout($bot->config->{timeout});

  my $url = $bot->url . 'inventory.php?which=' . $page;
  my $response = $bot->ua->get($url);

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'get_inventory');

  die 'unknown response in get_inventory, ',$response->content
    unless $response->content =~ m|<b>Inventory:</b>|;

  # get rid of head and tail, keep only the juicy filling

  my $content = $response->content;

  $content =~
    m|<center>Though your sack is large, it contains no food or beverages|
    and return ( $equip ? ([],{},{}) : []);
  $content =~
    m|<center>Though your sack is large, it contains no equipment|
    and return ( $equip ? ([],{},{}) : []);

  $content =~
    m|<Table width=100%>(.*)</table></div>|
    or die $content;

  $content = $1;

#  $content =~ s/&quot;/\"/g;

  my $items = [];
  if($content =~ m{http://images.kingdomofloathing.com/itemimages/}) {
    while($content =~ m{onClick='descitem\((\d+)\);'></td><td valign=top><b>(.*?)</b> ( \((\d+)\))?}g) {
      my $desc = $1;
      my $name = $2;
      my $number = $4 || 1;
      if($no_get_item) {
	push @$items, {desc => $desc,
		       name => $name,};
      } else {
	my $i = $bot->items->get_item(desc => $desc,
				      name => $name,
				      number => $number);
	warn "no $desc $name" unless $i;
	push @$items,new KoLBot::Item(desc => $desc, name => $name,
				      id => $i->{id},
				      number => $number);
      }
    }
  } else {
    # works for inventory images off (top two patterns)
    while($content =~ m{<td valign=top>(.*?)</td>}g) {
      my $itemp = $1;
      if($itemp =~ m{<b><a onClick='javascript:descitem\((\d+)\);'>(.*?)</a></b> \((\d+)\)}) {
	my $desc = $1;
	my $name = $2;
	my $number = $3;
	if($no_get_item) {
	  push @$items, {desc => $desc,
			 name => $name,};
	} else {
	  my $i = $bot->items->get_item(desc => $desc,
					name => $name,
					number => $number);
	  warn "no $desc $name" unless $i;
	  push @$items,new KoLBot::Item(desc => $desc, name => $name,
					id => $i->{id},
					number => $number);
	}
      } elsif ($itemp =~ m{<b><a onClick='javascript:descitem\((\d+)\);'>(.*?)</a></b>}) {
	my $desc = $1;
	my $name = $2;
	my $number = 1;
	if($no_get_item) {
	  push @$items, {desc => $desc,
			 name => $name,};
	} else {
	  my $i = $bot->items->get_item(desc => $desc,
					name => $name,
					number => $number);
	  warn "no $desc $name" unless $i;
	  push @$items,new KoLBot::Item(desc => $desc, name => $name,
					id => $i->{id},
					number => $number);
	}
      } else {
	die "Not an item: $itemp";
      }
    }
  }
  my $equipped = {};
  my $outfits = {};
  $content = $response->content;
  if($page == 2) {
    while($content =~ m{<img .*? onClick='descitem\((\d+)\)';></td><td><b>(.*?)</b>.*?<a href="inv_equip.php\?pwd=[01-9a-f]+\&which=2\&action=unequip\&type=(.*?)">}g) {
      if($no_get_item) {
	$equipped->{$3} =  {desc => $1,
			    name => $2,
			    number => 1};
      } else {
	$equipped->{$3} =  $bot->items->get_item(desc => $1,
						 name => $2,
						 number => 1);
      }
    }
    while($content =~ m{<option value=(-?\d+)>(.*?)</option>}g) {
    # outfits
      $outfits->{lc($2)} = {name => $2,
			    id   => $1};
    }
  }

  if($equip) {
    return ($items,$equipped,$outfits);
  } else {
  EQ:
    for (keys %$equipped) {
      for my $i (@$items) {
	if($i->{desc} == $equipped->{$_}->{desc}) {
	  $i->{number} += $equipped->{$_}->{number};
	  next EQ;
	}
      }
      push @$items,$equipped->{$_};
    }
    return $items;
  }
  # uh....
  return $items;
}

# we expect that the item is in our inventory, but not in the bot's
# items.
# BROKEN, AWM
sub find_item {
  my $bot = shift;
  my $args = { @_ };

  my $item_name = $args->{name};
  my ($cinv,$inv) = $bot->get_closet();
  # empty the closet
  for(@$cinv) {
    my $url = $bot->url . 'closet.php?action=take&howmany=all&whichitem='
      . $_->id;
    my $response = $bot->ua->head($url);

    $response->is_success
      or die 'HTTP ' . $response->status_line;
  }
  ($cinv,$inv) = $bot->get_closet();
  if(@$cinv) {
    die "FATAL: that should have emptied the closet.";
  }

  # Ok, our closet is empty, so put in the items we don't know about
  for(@$inv) {
    if(lc($_->name) eq lc($args->{name})) {
      my $url = $bot->url . 'closet.php';

      $bot->ua->max_size(0);
      my $response = $bot->ua->post($url,
	    [ action    => 'put',
	      whichitem => $_->id,
	      pwd       => $bot->hash,
	      howmany   => 'one',]);
      delete($bot->ua->{max_size});
      $response->is_success
	or die 'HTTP ' . $response->status_line;
    }
  }
  ($cinv,$inv) = $bot->get_closet();

  my @nitems = ();
  for(@$cinv) {
    print "MISSING ITEM ADDED, ",$_,"\n";
    $bot->items->add($_);
    push @nitems,$_;

    my $url = $bot->url . 'closet.php?action=take&howmany=all&whichitem='
      . $_->id;
    my $response = $bot->ua->head($url);

    $response->is_success
      or die 'HTTP ' . $response->status_line;
  }

  ($cinv,$inv) = $bot->get_closet();
  if(@$cinv) {
    die "FATAL: that should have emptied the closet.";
  }

  if(@nitems==0) {
    print "No matches\n";
    return undef;
  }

  if(@nitems==1) {
    return $nitems[0];
  }

  my $item = undef;
  for(@nitems) {
    if($_->desc == $args->{desc}) {
      if($item) {
	print "Two or more descr matches\n";
	return undef;
      }
      $item = $_;
    }
  }

  if($item) {
    return $item;
  }
    

  print "No descr matches\n";
  return undef;
}

#
# Returns two references to hashes.  The first is the number of items in 
# the closet and the second is the items in the inventory according 
# to the closet page
#

sub closet {
  my $bot = shift;
  my $params = {@_};

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'closet.php';
  my $response = $bot->ua->get($url);

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'closet');

  unless($response->content =~ m|<b>Your Colossal Closet</b>|) {
    $response->content =~ /You twiddle your thumbs./i
      and die "IN_COMBAT";
    die 'unknown KoLBot::closet, ',$response->content;
  }

  # clean up
  my $content = $response->content;
  $content =~ s/\s+/ /g;

  my $meat = 0;
  $content =~ 
      m|<b>Your closet contains ([\d,]+) meat.</b>|
      and $meat = $1;
  $meat =~ s/,//g;

  my $closet = {};
  my $inv = {};

  # closet items
  if ($content =~ 
      m|<b>Take: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {

    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)' descid='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $name = $2;
      my $number = $3;
      $number =~ s/,//g;

      $closet->{$id} = $number;
    }
  }

  # inventory items
  if ($content =~ 
      m|<b>Put: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {
    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)' descid='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $desc = $2;
      my $name = $3;
      my $number = $4;
      $number =~ s/,//g;

      $inv->{$id} = $number;
    }
  }

  return ($closet,$inv);
}

sub closet_full_inventory {
  my $bot = shift;
  my $params = {@_};

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'closet.php';
  my $response = $bot->ua->get($url);

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'closet');

  unless($response->content =~ m|<b>Your Colossal Closet</b>|) {
    $response->content =~ /You twiddle your thumbs./i
      and die "IN_COMBAT";
    die 'unknown KoLBot::closet, ',$response->content;
  }

  # clean up
  my $content = $response->content;
  $content =~ s/\s+/ /g;

  my $meat = 0;
  $content =~ 
      m|<b>Your closet contains ([\d,]+) meat.</b>|
      and $meat = $1;
  $meat =~ s/,//g;

  my $closet = {};
  my $inv = {};

  # closet items
  if ($content =~ 
      m|<b>Take: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {

    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)' descid='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $desc = $2;
      my $name = $3;
      my $number = $4;
      $number =~ s/,//g;

      $closet->{$id} = new KoLBot::Item(id => $id, desc => $desc,
					name => $name, number => $number);
    }
  }

  # inventory items
  if ($content =~ 
      m|<b>Put: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {
    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)' descid='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $desc = $2;
      my $name = $3;
      my $number = $4;
      $number =~ s/,//g;

      $inv->{$id} = new KoLBot::Item(id => $id, desc => $desc,
				     name => $name, number => $number);
    }
  }

  return ($closet,$inv);
}

sub closet_take {
  my $bot = shift;
  my $params = {@_};

  $bot->ua->timeout($bot->config->{timeout});

  my $url = $bot->url . 'closet.php';
  my $args = [action=>'take',pwd=>$bot->hash];
  my $i = 1;
  for $i (1..@{$params->{item_id}}) {
    push @$args,"whichitem".$i,$params->{item_id}->[$i-1];
    push @$args,"howmany".$i,$params->{number}->[$i-1];
  }

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'closet_take');

  # AWM
}

sub closet_put {
  my $bot = shift;
  my $params = {@_};

  $bot->ua->timeout($bot->config->{timeout});

  my $url = $bot->url . 'closet.php';
  my $args = [action=>'put',pwd=>$bot->hash];
  my $i = 1;
  for $i (1..@{$params->{item_id}}) {
    push @$args,"whichitem".$i,$params->{item_id}->[$i-1];
    push @$args,"howmany".$i,$params->{number}->[$i-1];
  }

  my $response = $bot->ua->post($url,$args);

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'closet_put');

  return $response->content;
}

#
# Collection routines
#

sub managecollection {
  my $bot = shift;
  my $params = {@_};

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'managecollection.php';

  my $args = [];
  if($params->{action}) {
    $args = [action=>$params->{action},pwd=>$bot->hash];
    my $i = 1;
    for $i (1..@{$params->{item_id}}) {
      push @$args,"whichitem".$i,$params->{item_id}->[$i-1];
      push @$args,"howmany".$i,$params->{number}->[$i-1];
    }
  }
  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'managecollection');

  die 'unknown managecollection',$response->content
    unless $response->content =~ m|<b>Your Display Case</b>|;

  # clean up
  my $content = $response->content;
  $content =~ s/\s+/ /g;

  my $collect = {};
  my $inv = {};

  # closet items
  if ($content =~ 
      m|<b>Take: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {

    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $name = $2;
      my $number = $3;
      $number =~ s/,//g;

      $collect->{$id} = $number;
    }
  }

  # inventory items
  if ($content =~ 
      m|<b>Put: <input type=text size=2 name=howmany1> </b> <select name="whichitem1"> <option value=0>-select an item-</option><option (.*?)</option></select>|) {
    my $closet_s = $1;
    my @closet_s = split m{</option><option },$closet_s;

    for (@closet_s) {
      m{value='(\d+)'>(.*) \(([\d,]+)\)}
        or die "Not an item: $_";

      my $id = $1;
      my $name = $2;
      my $number = $3;
      $number =~ s/,//g;

      $inv->{$id} = $number;
    }
  }

  return ($collect,$inv);
}

# parameters
# action: hoboflex,bigbook,tanningbed
# numturns

sub gym {
  my $bot = shift;
  my $params = {$_};


  my $url = $bot->url . 'clan_hall.php';
  $bot->ua->max_size(0);
  my $args =
      [
       action    => $params->{action},
       numturns  => $params->{turns},
	];
  my $response = $bot->ua->post($url,$args);
  delete($bot->ua->{max_size});

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;
}


# AWM: prints successful adventure even when password is wrong.
sub shore {
  my $bot = shift;
  my $parms = { @_ };
  
  warn "Need an trip number",return unless exists($parms->{whichtrip});

  my $url = $bot->url . 'shore.php';
  my $args = [
	      pwd       => $bot->hash,
	      whichtrip => $parms->{whichtrip},
	      ];

  delete($bot->ua->{max_size});
  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'shore');

  return unless exists($parms->{check_response});

  return parse_response($response->content);

#   if($response->content =~ /You don\'t have enough adventures left\./i) {
#     return 1;
#   }

#   if($response->content =~ /Shore without a car\./i) {
#     print "You have no car!\n";
#     return 1;
#   }

#   if($response->content =~ /Vacation Results:/) {
#     return 0;
#   }
}

# Action = boris, jarlsberg, sneakypete
sub shrines {
  my $bot = shift;
  my $parms = {@_};

  my $url = $bot->url . 'shrines.php';
  my $args =
    [
     pwd    => $bot->hash,
     action => $parms->{action},
     howmuch => $parms->{howmuch},
     ];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'shrines');

  # AWM CONSOLIDATE
  # no money
#   if($response->content =~ //i) {
#     return 1;
#   }

  # no key
#   if($response->content =~ /Shore without a car\./i) {
#     print "You have no car!\n";
#     return 1;
#   }

  # already done
#   if($response->content =~ /Vacation Results:/) {
#     return 0;
#   }

  # good response
  if($response->content =~ /You gain (\d+\s+\w+)/i) {
    return (0,$1);
  }

  if($response->content =~ /You may not offer any more/i) {
    return (1,'nomore');
  }

  if($response->content =~ /You don't have that key/i) {
    return (1,'nokey');
  }

  if($response->content =~ /You don't have that much Meat/i) {
    return (1,'nomeat');
  }

  print "AWM, through", $response->content;

}

sub adventure {
  my $bot = shift;
  my %params = ( @_ );
  
  warn "Need an adventure number",return unless exists($params{adv});

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'adventure.php?snarfblat=' . $params{adv};
  if($params{subsnarfblat}) {
    $url .= "&subsnarfblat=" . $params{subsnarfblat};
  }
  delete($bot->ua->{max_size});
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'adventure');

  # Attack page
  if($response->content =~ /value=\"Attack with your .*?\"/) {
    my ($rv,$txt,$results) = $bot->parse_response($response->content);
    $results->{in_combat} = 1;
    # AWM: parse attack options
    my $options = {};
    return ($rv,$txt,$results,$options);
  }
  # AWM: need choice adventures

  return $bot->parse_response($response->content);
}

sub king {
  my $bot = shift;
  
  my $url = $bot->url . 'knob.php';
  my $response = $bot->ua->post($url,
				[ pwd => $bot->{hash},
				  king => 'Yep.', ] );
  $bot->check_response($response,'king');

  # Attack page
  if($response->content =~ /value=\"Attack with your .*?\"/) {
    my ($rv,$txt,$results) = $bot->parse_response($response->content);
    $results->{in_combat} = 1;
    # AWM: parse attack options
    my $options = {};
    return ($rv,$txt,$results,$options);
  }

  return $bot->parse_response($response->content);
}

sub dragon {
  my $bot = shift;
  
  my $url = $bot->url . 'cyrpt.php';
  my $response = $bot->ua->post($url,
				[ pwd => $bot->{hash},
				  action => 'Yep.', ] );
  $bot->check_response($response,'dragon');

  # Attack page
  if($response->content =~ /value=\"Attack with your .*?\"/) {
    my ($rv,$txt,$results) = $bot->parse_response($response->content);
    $results->{in_combat} = 1;
    # AWM: parse attack options
    my $options = {};
    return ($rv,$txt,$results,$options);
  }

  return $bot->parse_response($response->content);
}

sub rats {
  my $bot = shift;
  my $params = { @_ };
  
  my $response;
  if(exists($params->{faucetoff})) {
    my $url = $bot->url . 'rats.php';
    my $args = [
		action => 'faucetoff',];
    $response = $bot->ua->post($url,$args);
  } else {
    die "Need a location",return unless exists($params->{where});
    my $url = $bot->url . 'rats.php?where=' . $params->{where};

    $bot->ua->timeout($bot->config->{timeout});
    delete($bot->ua->{max_size});
    $response = $bot->ua->get($url);
  }

  $bot->check_response($response,'rats');

  # Attack page
  if($response->content =~ /value=\"Attack with your .*?\"/) {
    my ($rv,$txt,$results) = $bot->parse_response($response->content);
    $results->{in_combat} = 1;
    # AWM: parse attack options
    my $options = {};
    return ($rv,$txt,$results,$options);
  }

  return $bot->parse_response($response->content);
}

sub basement {
  my $bot = shift;
  my %params = @_;
  
  my $response;

  my $url = $bot->url . 'basement.php';
  if(exists($params{action})) {
    my $args = [action => $params{action}];
    $response = $bot->ua->post($url,$args);
  } else {
    $response = $bot->ua->get($url);
  }

  $bot->check_response($response,'basement');

  my ($rv,$txt,$resp);
  if($params{action}) {
    ($rv,$txt,$resp) = $bot->parse_response($response->content);
  } else {
    $rv = 0;
    $txt = '';
    $resp = {};
    my $content = $response->content;
    $content =~ /<b>Fernswarthy\'s Basement, Level (\d+)<\/b>/i
      or die "Doesnt' seem to be the basement.";
    $resp->{level} = $1;
    if($content =~ 
       /sense is in the form of ([\d,]+) Bottles of Beer on a Golem/) {
      $resp->{next_enc} = 'beer_golem';
      $resp->{enc_number} = $1;
    } elsif($content =~ /The Beast with ([\d,]+) Eyes!/) {
      $resp->{next_enc} = 'eye_beast';
      $resp->{enc_number} = $1;
    } elsif($content =~ /dreaded Beast with ([\d,]+) Ears/) {
      $resp->{next_enc} = 'ear_beast';
      $resp->{enc_number} = $1;
    } elsif($content =~ /([\d,]+)-dimensional horror/) {
      $resp->{next_enc} = 'd_horror';
      $resp->{enc_number} = $1;
    } elsif($content =~ /guarded by the ghost of Fernswarthy/) {
      $resp->{next_enc} = 'great_grandfather';
      $resp->{enc_number} = 0;
      while($content =~ /great /g) {
	$resp->{enc_number}++;
      }
    } elsif($content =~ /pounding noise.  Coming from an? ([,\d]+) Stone Golem./) {
      $resp->{next_enc} = 'stone_golem';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /ferocious ([\d,]+)-Headed Hydra/) {
      $resp->{next_enc} = 'stone_golem';
      $resp->{enc_number} = $1;

    } elsif ($content =~ /<b>Singled Out<\/b>/) {
      $resp->{next_enc} = 'singled_out';
      $resp->{enc_number} = '';
    } elsif ($content =~ /<b>Unholy Writes<\/b>/) {
      $resp->{next_enc} = 'unholy_writes';
      $resp->{enc_number} = '';
    } elsif ($content =~ /<b>The Unthawed<\/b>/) {
      $resp->{next_enc} = 'unthawed';
      $resp->{enc_number} = '';
    } elsif ($content =~ /<b>Peace, Bra!<\/b>/) {
      $resp->{next_enc} = 'peace_bra';
      $resp->{enc_number} = '';
    } elsif ($content =~ /<b>Still Better than Pistachio<\/b>/) {
      $resp->{next_enc} = 'pistachio';
      $resp->{enc_number} = '';
    } elsif ($content =~ /large statue, sitting exactly where the stairs/) {
      $resp->{next_enc} = 'statue';
      $resp->{enc_number} = 0;
      while($content =~ /very /g) {
	$resp->{enc_number}++;
      }
    } elsif ($content =~ /with this here ([\d,]+)-pound mallet./) {
      $resp->{next_enc} = 'muscle_mallet';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /find ([\d,]+) typewriters/) {
      $resp->{next_enc} = 'typewriters';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /([\d,]+)-pound sleeping baby/) {
      $resp->{next_enc} = 'sleeping_baby';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /mops\.  There appear to be about ([\d,]+) of them/) {
      $resp->{next_enc} = 'mops';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /approximately ([\d,]+) shards of dense dark magic/) {
      $resp->{next_enc} = 'shards';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /looms a wall of Voodoo/) {
      $resp->{next_enc} = 'voodoo';
      $resp->{enc_number} = 0;
      while($content =~ /very /g) {
	$resp->{enc_number}++;
      }
    } elsif ($content =~ /attractive Sorority Orc blocking/) {
      $resp->{next_enc} = 'sorority_orc';
      $resp->{enc_number} = 0;
      while($content =~ /very /g) {
	$resp->{enc_number}++;
      }
    } elsif ($content =~ /off of ([\d,]+) rails, or it doesn\'t count/) {
      $resp->{next_enc} = 'pool_shark';
      $resp->{enc_number} = $1;
    } elsif ($content =~ /<b>A Festering Powder<\/b>/) {
      $resp->{next_enc} = 'festering_powder';
      $resp->{enc_number} = '';
    } elsif ($content =~ /<b>Throwin\' Down the Gauntlet<\/b>/) {
      $resp->{next_enc} = 'gauntlet';
      $resp->{enc_number} = '';
    } elsif ($content =~ /piled high with mint-in-box action figures/) {
      $resp->{next_enc} = 'collectibles';
      $resp->{enc_number} = '';
    } elsif ($content =~ /two jackets hanging on pegs/) {
      $resp->{next_enc} = 'jackets';
      $resp->{enc_number} = '';
    } elsif ($content =~ /table with a glass of water and two pills/) {
      $resp->{next_enc} = 'pills';
      $resp->{enc_number} = '';
    } else {
      print $content;
      die "Can't tell what basement encounter is";
    }

    $resp->{enc_number} =~ s/,//g;
  }

  return ($rv,$txt,$resp);
}

# <centeR><table width=95%  cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=blue><b>Fernswarthy's Basement, Level 1</b></td></tr><tr><td style="padding: 5px; border: 1px solid blue;"><center><table><tr><td><centeR><img src="http://images.kingdomofloathing.com/adventureimages/beergolem.gif" width=100 height=100></center><blockquote>As you start to enter this room, you immediately sense beer.  As you finish entering it, you gasp (<i>gasp!</i>) as you realize that the beer you sense is in the form of 99 Bottles of Beer on a Golem...</blockquote><form action=basement.php method=post><input type=hidden name=action value="1"><center><input class=button type=submit value="Take 99 Down (1)"></form><br><p><a href="fernruin.php">Back to The Ruins of Fernswarthy's Tower</a></centeR></td></tr></table></center></td></tr><tr><td height=4></td></tr></table></center></body></html>

sub fight {
  my $bot = shift;
  my $params = { action => 'attack',
		 @_ };

  my $url = $bot->url . 'fight.php';
  my $args = [action => $params->{action},
	      button => 'submit'];

  if($params->{action} =~ /useitem/i) {
    push @$args,'whichitem',$params->{whichitem}->[0];
    if(@{$params->{whichitem}} > 1) {
      push @$args,'whichitem2',$params->{whichitem}->[1];
    } else {
      push @$args,'whichitem2',0;
    }
  }
  
  if($params->{action} =~ /skill/i) {
    push @$args,'whichskill',$params->{whichskill};
  }

  my $response = $bot->ua->post($url,$args);

  $response->is_success or die "HTTP " . $response->status_line;
  $bot->check_response($response,'fight');

  my ($rv,$txt,$results) = $bot->parse_response($response->content);

  return ($rv,$txt,$results);
}

sub choice {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'choice.php';

  my $args = [pwd => $bot->{hash},
	      whichchoice => $params{choice},
	      option => $params{option},
	      button => 'submit'];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'choice');
  return $bot->parse_response($response->content);
}


sub campground {
  my $bot = shift;
  my $params = { @_ };

  my $url;
  if(exists($params->{action})) {
    $url = $bot->url . 'campground.php?action=' . $params->{action};
  } else {
    $url = $bot->url . 'campground.php';
  }
  delete($bot->ua->{max_size});
  my $response = $bot->ua->get($url);
  $bot->check_response($response,'campground');

  my $stuff = {};

  for(qw(rest4 fengshui pagoda closet
	 beanbag toaster trophycase certificate
	 arches
	 bartender chef terrarrium locker
	 maid scarecrow golem hippystone
	 )) {
    $stuff->{$_} = ($response->content =~ /$_\.gif/i);
  }

  # AWM: response from action	    
  # AWM bouquet barbedwire picket
  return $stuff;
}

sub hiddencity {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'hiddencity.php';

  my $response;
  if(defined $params{which}) {
    $response = $bot->ua->get($url . '?which=' . $params{which});
  } elsif (defined $params{whichitem}) {
    my @args = (action => 'roundthing',
		whichitem => $params{whichitem},
		);
    $response = $bot->ua->post($url,\@args);
  } else {
    $response = $bot->ua->get($url);
  }

  $bot->check_response($response,'hiddencity');

  if(defined $params{which} || defined $params{whichitem}) {
    return $bot->parse_response($response->content);
  }

  my $ret = {map => {} };

  my $content = $response->content;

  while($content =~ m{<a href='hiddencity.php\?which=(\d+)'><img src="http://images.kingdomofloathing.com/otherimages/hiddencity/(.*?).gif" width=100 height=100 border=0></a>}g) {
    my $area = $1;
    my $pic  = $2;

    $pic =~ /^map_(altar|temple|(un)?ruins|spirit)[1-9]?$/ or 
      die "failure parsing hidden city map, area = $area; pic = $pic";

    $pic = $1;

    # map_ruins\d
    # map_unruins\d
    # altar
    # temple

    $ret->{map}->{$area} = $pic;
  }
  return (0,'',{},$ret);
}

  # No clear way to get class.  This should go somewhere we can make use of
  # it for /whois data as well.
   my %title2class = (
		      ## Seal Clubbers
		      'Lemming Trampler' => 'Seal Clubber',
		      'Tern Slapper' => 'Seal Clubber',
		      'Puffin Intimidator' => 'Seal Clubber',
		      'Ermine Thumper' => 'Seal Clubber',
		      'Penguin Frightener' => 'Seal Clubber',
		      'Malamute Basher' => 'Seal Clubber',
		      'Narwhal Pummeler' => 'Seal Clubber',
		      'Otter Crusher' => 'Seal Clubber',
		      'Caribou Smacker' => 'Seal Clubber',
		      'Moose Harasser' => 'Seal Clubber',
		      'Reindeer Threatener' => 'Seal Clubber',
		      'Ox Wrestler' => 'Seal Clubber',
		      'Walrus Bludgeoner' => 'Seal Clubber',
		      'Whale Boxer' => 'Seal Clubber',
		      'Seal Clubber' => 'Seal Clubber',
		      
		      ## Turtle Tamers
		      'Toad Coach' => 'Turtle Tamer',
		      'Skink Trainer' => 'Turtle Tamer',
		      'Frog Director' => 'Turtle Tamer',
		      'Gecko Supervisor' => 'Turtle Tamer',
		      'Newt Herder' => 'Turtle Tamer',
		      'Frog Boss' => 'Turtle Tamer',
		      'Iguana Driver' => 'Turtle Tamer',
		      'Salamander Subduer' => 'Turtle Tamer',
		      'Bullfrog Overseer' => 'Turtle Tamer',
		      'Rattlesnake Chief' => 'Turtle Tamer',
		      'Crocodile Lord' => 'Turtle Tamer',
		      'Cobra Commander' => 'Turtle Tamer',
		      'Aligator Subjugator' => 'Turtle Tamer',
		      'Asp Master' => 'Turtle Tamer',
		      'Turtle Tamer' => 'Turtle Tamer',
		      
		      ## Pastamancers
		      'Dough Acolyte' => 'Pastamancer',
		      'Yeast Scholar' => 'Pastamancer',
		      'Noodle Neophyte' => 'Pastamancer',
		      'Starch Savant' => 'Pastamancer',
		      'Carbohydrate Cognoscenti' => 'Pastamancer',
		      'Spaghetti Sage' => 'Pastamancer',
		      'Macaroni Magician' => 'Pastamancer',
		      'Vermicelli Enchanter' => 'Pastamancer',
		      'Linguini Thaumaturge' => 'Pastamancer',
		      'Ravioli Sorcerer' => 'Pastamancer',
		      'Manicotti Magus' => 'Pastamancer',
		      'Spaghetti Spellbinder' => 'Pastamancer',
		      'Cannelloni Conjurer' => 'Pastamancer',
		      'Angel-Hair Archmage' => 'Pastamancer',
		      'Pastamancer' => 'Pastamancer',
		      
		      ## Saucerors
		      'Allspice Acolyte' => 'Sauceror',
		      'Cilantro Seer' => 'Sauceror',
		      'Parsley Enchanter' => 'Sauceror',
		      'Sage Sage' => 'Sauceror',
		      'Rosemary Diviner' => 'Sauceror',
		      'Thyme Wizard' => 'Sauceror',
		      'Tarragon Thaumaturge' => 'Sauceror',
		      'Oreganoccultist' => 'Sauceror',
		      'Basillusionist' => 'Sauceror',
		      'Coriander Conjurer' => 'Sauceror',
		      'Bay Leaf Brujo' => 'Sauceror',
		      'Sesame Soothsayer' => 'Sauceror',
		      'Marinara Mage' => 'Sauceror',
		      'Alfredo Archmage' => 'Sauceror',
		      'Sauceror' => 'Sauceror',
		      
		      # Accordion Thieves
		      'Polka Criminal' => 'Accordion Thief',
		      'Mariachi Larcenist' => 'Accordion Thief',
		      'Zydeco Rogue' => 'Accordion Thief',
		      'Chord Horker' => 'Accordion Thief',
		      'Chromatic Crook' => 'Accordion Thief',
		      'Squeezebox Scoundrel' => 'Accordion Thief',
		      'Concertina Con Artist' => 'Accordion Thief',
		      'Button Box Burglar' => 'Accordion Thief',
		      'Hurdy-Gurdy Hooligan' => 'Accordion Thief',
		      'Sub-Sub-Apprentice Accordion Thief' => 'Accordion Thief',
		      'Sub-Apprentice Accordion Thief' => 'Accordion Thief',
		      'Pseudo-Apprentice Accordion Thief' => 'Accordion Thief',
		      'Hemi-Apprentice Accordion Thief' => 'Accordion Thief',
		      'Apprentice Accordion Thief' => 'Accordion Thief',
		      'Accordion Thief' => 'Accordion Thief',
		      
		      ## Disco Bandits
		      'Funk Footpad' => 'Disco Bandit',
		      'Rhythm Rogue' => 'Disco Bandit',
		      'Chill Crook' => 'Disco Bandit',
		      'Jiggy Grifter' => 'Disco Bandit',
		      'Beat Snatcher' => 'Disco Bandit',
		      'Sample Swindler' => 'Disco Bandit',
		      'Move Buster' => 'Disco Bandit',
		      'Jam Horker' => 'Disco Bandit',
		      'Groove Filcher' => 'Disco Bandit',
		      'Vibe Robber' => 'Disco Bandit',
		      'Boogie Brigand' => 'Disco Bandit',
		      'Flow Purloiner' => 'Disco Bandit',
		      'Jive Pillager' => 'Disco Bandit',
		      'Rhymer and Stealer' => 'Disco Bandit',
		      'Disco Bandit' => 'Disco Bandit',


		      # Accordion Thieves
		      'polka criminal' => 'Accordion Thief',
		      'mariachi larcenist' => 'Accordion Thief',
		      'zydeco rogue' => 'Accordion Thief',
		      'chord horker' => 'Accordion Thief',
		      'chromatic crook' => 'Accordion Thief',
		      'squeezebox scoundrel' => 'Accordion Thief',
		      'concertina con artist' => 'Accordion Thief',
		      'button box burglar' => 'Accordion Thief',
		      'hurdy-gurdy hooligan' => 'Accordion Thief',
		      'sub-sub-apprentice accordion thief' => 'Accordion Thief',
		      'sub-apprentice accordion thief' => 'Accordion Thief',
		      'pseudo-Apprentice accordion thief' => 'Accordion Thief',
		      'hemi-Apprentice accordion thief' => 'Accordion Thief',
		      'apprentice accordion thief' => 'Accordion Thief',
		      'accordion thief' => 'Accordion Thief',

		      # Disco Bandits
		      'funk footpad' => 'Disco Bandit',
		      'rhythm rogue' => 'Disco Bandit',
		      'chill crook' => 'Disco Bandit',
		      'jiggy grifter' => 'Disco Bandit',
		      'beat snatcher' => 'Disco Bandit',
		      'sample swindler' => 'Disco Bandit',
		      'move buster' => 'Disco Bandit',
		      'jam horker' => 'Disco Bandit',
		      'groove filcher' => 'Disco Bandit',
		      'vibe robber' => 'Disco Bandit',
		      'boogie brigand' => 'Disco Bandit',
		      'flow purloiner' => 'Disco Bandit',
		      'jive pillager' => 'Disco Bandit',
		      'rhymer and stealer' => 'Disco Bandit',
		      'disco bandit' => 'Disco Bandit',
		      );

sub charpane {
  my $bot = shift;
  my $parms = { @_ };

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'charpane.php';
  delete($bot->ua->{max_size});
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'charpane');

  my $resp = {};

  my $content = $response->content;

  $content =~ s/&nbsp;/ /g;
  $content =~ s/ width=\d+//g;
#  $content =~ s/ height=\d+//g;
  $content =~ s/(\d),(\d)/$1$2/g;
#  $content =~ s/<td.*?>/<td>/g;
  $content =~ s/<td v?align=(left|center)/<td/g;
  $content =~ s/<td v?align=(left|center)/<td/g;
  $content =~ s/<font.*?>/<font>/g;

  if($content =~ /var pwdhash = \"(.+?)\";/) {
     $resp->{pwd} = $1;
   }

  if($content =~ m|<a class=nounder target=mainpane href="charsheet.php"><b>(.*?)</b></a>|) {
    $resp->{name} = $1;
  }


#   if($content =~ m{Level (\d+)<br>(.*?)<(t|/td)}) {
#     $resp->{level} = $1;
#     die "Don't know title $2"
#       unless exists($title2class{$2});
#     $resp->{class} = $title2class{$2};
#   } else {
#     die "No level/class info";
#   }

 $content =~ /Outfit: ([^<]*)/
   and $resp->{outfit} = $1;

  $content =~ m|Muscle:</td><td><b>(\d+)</b>|
    and $resp->{base_muscle} = $1,$resp->{muscle} = $1;
  $content =~ m|Muscle:</td><td><b>\d+</b><table title=\'(\d+)|
    and $resp->{muscle_plus_substats} = $1;
  $content =~ m|Mysticality:</td><td><b>(\d+)</b>|
    and $resp->{base_myst} = $1, $resp->{myst} = $1;
  $content =~ m|Mysticality:</td><td><b>\d+</b><table title=\'(\d+)|
    and $resp->{myst_plus_substats} = $1;
  $content =~ m|Moxie:</td><td><b>(\d+)</b>|
    and $resp->{base_moxie} = $1, $resp->{moxie} = $1;
  $content =~ m|Moxie:</td><td><b>\d+</b><table title=\'(\d+)|
    and $resp->{moxie_plus_substats} = $1;
  $content =~ m|Muscle:</td><td><b><font>(\d+)</font> \((\d+)\)</b>|
    and $resp->{muscle} = $1, $resp->{base_muscle} = $2;
  $content =~ m|Muscle:</td><td><b><font>\d+</font> \(\d+\)</b><table title=\'(\d+)|
    and $resp->{muscle_plus_substats} = $1;
  $content =~ m|Mysticality:</td><td><b><font>(\d+)</font> \((\d+)\)</b>|
    and $resp->{myst} = $1, $resp->{base_myst} = $2;
  $content =~ m|Mysticality:</td><td><b><font>\d+</font> \(\d+\)</b><table title=\'(\d+)|
    and $resp->{myst_plus_substats} = $1;
  $content =~ m|Moxie:</td><td><b><font>(\d+)</font> \((\d+)\)</b>|
    and $resp->{moxie} = $1, $resp->{base_moxie} = $2;
  $content =~ m|Moxie:</td><td><b><font>\d+</font> \(\d+\)</b><table title=\'(\d+)|
    and $resp->{moxie_plus_substats} = $1;

  if($content =~ m{<td>(Inebriety|Drunk|Drunkenness|Tipsiness|Temulency):</td><td><b>(\d+)</b>}i) {
    $resp->{drunk} = $2;
  } else {
    $resp->{drunk} = 0;
  }

  $content =~ m{you are falling-down drunk}i
    and $resp->{falling_down} = 1;

  $content =~ m{/hp.gif\".*?><br><span class=(black|red)>(\d+).*?(\d+)</span>}i
    and $resp->{hp} = $2,  $resp->{max_hp} = $3; 

  $content =~ m|HP:</td><td><b><font>(\d+)/(\d+)</font></b>|i
    and $resp->{hp} = $1, $resp->{max_hp} = $2;

  exists($resp->{hp}) or die "Problem getting hp, $content";

  $content =~ m|/mp.gif\".*?><br><span class=black>(\d+).*?(\d+)</span>|
    and $resp->{mp} = $1,  $resp->{max_mp} = $2; 

  $content =~ m|MP:</td><td><b>(\d+)/(\d+)</b>|i
    and $resp->{mp} = $1, $resp->{max_mp} = $2;

  exists($resp->{mp}) or die "Problem getting mp";

  $content =~ m|meat.gif\".*?><br><span class=black>([\d,]+)</span>|
    and ($resp->{meat} = $1) =~ s/,//g;

  $content =~ m|Meat:</td><td><b>([\d,]+)</b>|i
    and ($resp->{meat} = $1) =~ s/,//g;

  exists($resp->{meat}) or die "Problem getting meat";

  $content =~ m|hourglass.gif\".*?><br><span class=black>(\d+)</span>|
    and $resp->{adventures} = $1; 

  $content =~ m|Adv</a>:</td><td><b>(\d+)</b>|
    and $resp->{adventures} = $1; 

  exists($resp->{adventures}) or die "Problem getting adventures";

  $content =~ m|<font><b>Hardcore</b></font>|
    and $resp->{hardcore} = 1;

  if($content =~ m|canadia.php\?place=machine\">Mind Control</a>: <b>(\d+)</b>|) {
    $resp->{mcd} = $1;
  } else {
    $resp->{mcd} = 0;
  }

  $content =~ m|<a class=nounder href="([^\"]*)" target=mainpane>Last Adventure:</a>|
    and $resp->{last_adventure_area} = $1;
  $content =~ m|target=mainpane href="adventure.php\?snarfblat=(\d+)">([^<]*)</a>|
    and $resp->{last_adventure} = $1, $resp->{last_adventure_text} = $2;
  $content =~ m|<a target=mainpane href="rats.php">([^<]*)</a>|
    and $resp->{last_adventure} = 'rats', $resp->{last_adventure_text} = $1;
  $content =~ m|<a target=mainpane href="(lair\d?).php">([^<]*)</a>|
    and $resp->{last_adventure} = $1, $resp->{last_adventure_text} = $2;
  $content =~ m|<a target=mainpane href="(shore).php">([^<]*)</a>|
    and $resp->{last_adventure} = $1, $resp->{last_adventure_text} = $2;

  # do effects

  $resp->{effects} = {};
  while($content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/(\w+)\.gif" class=hand onClick='eff\("([\dabcdef]+)"\);'.*?></td><td.*?><font.*?>([\w\s\-\.\'\&;!,]+) \((\d+)\)</font>}g) {
    $resp->{effects}->{lc($3)} = {
      image => $1,
      id    => $2,
      name  => $3,
      turns => $4,
    };
  }

  # AWM: should do this based on type, as sombrero and chaemeleon fail
  $content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/familiar(\d+).gif".*?border=0></a></td><td><a target=mainpane href="familiar.php"><b><font>(.*?)</a></b>, the\s+(\d+)-pound (.*?)<}
  and $resp->{familiar} = {id => $1,
			   name => $2,
			   weight => $3,
			   type => $4,};

  # sombrero
  $content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/hat2.gif".*?border=0></a></td><td><a target=mainpane href="familiar.php"><b><font>(.*?)</a></b>, the\s+(\d+)-pound (.*?)<}
  and $resp->{familiar} = {id => 18,
			   name => $1,
			   weight => $2,
			   type => $3,};

  # slgfairy
  $content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/slgfairy.gif".*?border=0></a></td><td><a target=mainpane href="familiar.php"><b><font>(.*?)</a></b>, the\s+(\d+)-pound (.*?)<}
  and $resp->{familiar} = {id => 49,
			   name => $1,
			   weight => $2,
			   type => $3,};

  # hare
  $content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/hare.gif".*?border=0></a></td><td><a target=mainpane href="familiar.php"><b><font>(.*?)</a></b>, the\s+(\d+)-pound (.*?)<}
  and $resp->{familiar} = {id => 50,
			   name => $1,
			   weight => $2,
			   type => $3,};

  # hound dog
  $content =~ m{<img src="http://images.kingdomofloathing.com/itemimages/hounddog.gif".*?border=0></a></td><td><a target=mainpane href="familiar.php"><b><font>(.*?)</a></b>, the\s+(\d+)-pound (.*?)<}
  and $resp->{familiar} = {id => 69,
			   name => $1,
			   weight => $2,
			   type => $3,};


  # Hrm. mathces overall bar, except for height=3 bit.
  # comes after <b> in stat lines, not in familiar line
  $content =~ m{[^>]<table title='(\d+) / (\d+)' cellpadding=0 cellspacing=0 style='border: 1px solid #5A5A5A'><tr><td height=3} 
    and $resp->{familiar}->{kills_part} = $1;

  # obsolete
  #  $content =~ m|beatenup|i
  #    and $resp->{beaten_up} = 1;

  return $resp;
}

sub class_from_account {
  my $bot = shift;

  my $url = $bot->url . 'account_avatar.php';
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'charsheet');

  my $content = $response->content;

  my %map = (sauceror => 'Sauceror');

  $content =~ m{the Level \d+ ([^<]+)</td><td class=small><i>\(default\)}
    or die "no class";

  exists $title2class{$1} or die "Can't match class $1\n";

  return $title2class{$1};
}

sub charsheet {
  my $bot = shift;
  my $parms = { @_ };

  my $url = $bot->url . 'charsheet.php';
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'charsheet');

  my $resp = {};

  my $content = $response->content;

  $content =~ /<form name=choiceform\d? action=choice\.php method=post>/
    and confess "Charsheet: in choice";

  $content =~ /You twiddle your thumbs\./
    and confess "Charsheet: in combat";

  $content =~ s/&nbsp;/ /g;
  $content =~ s/ width=\d+//g;
  $content =~ s/ height=\d+//g;
  $content =~ s/(\d),(\d)/$1$2/g;
  $content =~ s/<\/?td.*?>//g;
  $content =~ s/<\/?table.*?>//g;
  $content =~ s/<\/?font.*?>//g;
  $content =~ s/<\/?tr>//g;
  $content =~ s/<\/?b>//g;
  $content =~ s/<font.*?>/<font>/g;

  $content =~ m|Muscle:(\d+)\s*\((\d+)/(\d+)\)|
    and $resp->{mus_sub} = $1*$1+$2;
  $content =~ m|Mysticality:(\d+)\s*\((\d+)/(\d+)\)|
    and $resp->{mys_sub} = $1*$1+$2;
  $content =~ m|Moxie:(\d+)\s*\((\d+)/(\d+)\)|
    and $resp->{mox_sub} = $1*$1+$2;

  $content =~ m|Muscle:(\d+)\s+\(base:\s+(\d+)\)\((\d+)/(\d+)\)|
    and $resp->{mus_sub} = $2*$2+$3;
  $content =~ m|Mysticality:(\d+)\s+\(base:\s+(\d+)\)\((\d+)/(\d+)\)|
    and $resp->{mys_sub} = $2*$2+$3;
  $content =~ m|Moxie:(\d+)\s+\(base:\s+(\d+)\)\((\d+)/(\d+)\)|
    and $resp->{mox_sub} = $2*$2+$3;

  my %rlevel = ( meager => 1,
		 minor  => 2,
		 modest => 3,
		 middling => 4,
		 moderate => 5,
		 major => 6,
		 mighty => 7,
		 monumental => 8,
		 maximal => 9, );
  my $types = join('|',keys %rlevel);
  for my $element(qw(hot cold stench spooky stench sleaze)) {
    $content =~ /${element} Protection:($types)/i
      and $resp->{'res_' . $element} = $rlevel{lc($1)};
  }

  $content =~ /Sign:The (Mongoose|Wallaby|Vole|Platypus|Opossum|Marmot|Wombat|Blender|Packrat)/
    and $resp->{sign} = $1;

  $content =~ /Outfit:([^<]*)/
    and $resp->{outfit} = $1;

  $content =~ /Turns Played \(this run\):(\d+)/
    and $resp->{run_turns} = $1;

  $resp->{skills} = {};
  while($content =~ m{<a onClick='javascript:skill\((\d+)\)'>(.*?)</a>}gi) {
    $resp->{skills}->{$1} = {id => $1,
			     name => $2,
			     extra => ''}; # Later for P, HP
  }

  return $resp;
}

sub showplayer {
  my $bot = shift;
  my $parms = { @_ };

  warn "Need a player number",return unless exists($parms->{who});

  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'showplayer.php?who=' . $parms->{who};
  delete($bot->ua->{max_size});
  my $response = $bot->ua->get($url);

  $bot->check_response($response,'showplayer');

  my $resp = {};
  # Parse response

  if($response->content =~ /Sorry, this player could not be found./) {
    $resp->{sorry} = 1;
  }

  my $pname = '\w[\w\ \d_]{0,28}\w';
  $response->content =~ /<b>($pname)<\/b>/
    and $resp->{name} = $1;

  $response->content =~ /otherimages\/(\w+)\.gif\"/
    and $resp->{image} = $1;
  if($response->content =~ /otherimages\/customavatars\/(\d+).gif\"/) {
    $resp->{image} = 'yourself';
  }

  if($response->content =~ /\(\#(\d+)\)<br>Level ([\d,]+)<br>([\w -]+)(<br><b>\(Hardcore\)<\/b>)?<\/td>/) {
    $resp->{id} = $1;
    $resp->{level} = $2;
    $resp->{level} =~ s/,//g;
    $resp->{title} = $3;
    $resp->{class} = $title2class{$3};
    $resp->{hardcore} = $4 ? 1 : 0;
  }

  if($response->content =~ /\(\#(\d+)\)<br>Level ([\d,]+)<br>([\w -]+)<\/td>/) {
    $resp->{id} = $1;
    $resp->{level} = $2;
    $resp->{level} =~ s/,//g;
    $resp->{title} = $3;
  }

  $response->content =~ /otherimages\/sigils\/(\w+).gif\"/
    and $resp->{sigil} = $1;

  if($response->content =~ /<b>Meat:<\/b><\/td><td>([\d,]+)<\/td>/) {
    $resp->{meat} = $1;
    $resp->{meat} =~ s/,//g;
  }

  if($response->content =~ /<b>Turns Played( \(total\))?:<\/b><\/td><td>([\d,]+)<\/td>/) {
    $resp->{turns} = $2;
    $resp->{turns} =~ s/,//g;
  }

  if($response->content =~ /<b>Turns Played \(this run\):<\/b><\/td><td>([\d,]+)<\/td>/) {
    $resp->{turnsrun} = $1;
    $resp->{turnsrun} =~ s/,//g;
  }
  
  if($response->content =~ /Ascensions<\/a>:<\/b><\/td><td>([\d,]+)<\/td>/) {
    $resp->{ascensions} = $1;
    $resp->{ascensions} =~ s/,//g;
  }

  $response->content =~ /<b>Account Created:<\/b><\/td><td>(\w+\s+\d\d,\s+\d\d\d\d)<\/td>/
    and $resp->{created} = $1;

  $response->content =~ /<b>Last Login:<\/b><\/td><td>(\w+\s+\d\d,\s+\d\d\d\d)<\/td>/
    and $resp->{lastlogin} = $1;

  $response->content =~ /<b>Sign:<\/b><\/td><td>The (Mongoose|Wallaby|Vole|Platypus|Opossum|Marmot|Wombat|Blender|Packrat)<\/td>/
    and $resp->{sign} = $1;

  if($response->content =~ /<center>PvP:<\/center>/i) {
    $resp->{pvp} = 1;
    if($response->content =~ /<b>Ranking:<\/b><\/td><td>([\d,]+)<\/td>/i) {
      $resp->{ranking} = $1;
      $resp->{ranking} =~ s/,//g;
    }
    if($response->content =~ /<b>Fights won:<\/b><\/td><td>([\d,]+)<\/td>/i) {
      $resp->{fightswon} = $1;
      $resp->{fightswon} =~ s/,//g;
    }
    if($response->content =~ /<b>Fights Lost:<\/b><\/td><td>([\d,]+)<\/td>/i) {
      $resp->{fightslost} = $1;
     $resp->{fightslost} =~ s/,//g;
    }
  }
  
  $resp->{gender} = 'None';
  $response->content =~ m|<b>Gender:</b></td><td>(.*?)</td>|
    and $resp->{gender} = $1;

  if($response->content =~ m|Clan: <b><a class=nounder href="showclan.php\?whichclan=(\d+)">(.*?)</a>|) {
    $resp->{clan_id} = $1;
    $resp->{clan} = $2;
  }
  $response->content =~ m|<td colspan=2>Store: <b>|
    and $resp->{store} = 1;
  $response->content =~ m|<b>Display Case</b>|
    and $resp->{case} = 1;

  my $content = $response->content;
  while($content =~ /'descitem\((\d+)\)'><\/td><td valign=center><b>([^\)].*?)<\/b>/g) {
    $resp->{items} ||= [];
    push @{$resp->{items}},{name => $2,desc=>$1};
  }

  return $resp;
}

#
# Helper functions
#

#
# What day is it?
# This is fucking confusing.  Midnight by message stamps is not
# the same as when the moons roll over.
#
# To use the same serial form (UNIX epoch), I'll use separate
# routines for converting
#

# 30-11-2006 = 1389

# This gives "KoL Time" from unix epoch
sub koltime {
  my $secs = shift;

  my $day_per_month = 8;
  my $mnth   = $day_per_month*24*60*60;
  my $yr     = $mnth * 12;
  my $yoff   = 6;
  my $moff   = -782-788;
  my $doff   = -4;
  my $hoff   = -4;		# Used to be -5
  my $off = $yoff * $yr + $moff * $mnth + $doff * 24*60*60 + $hoff*60*60;

  $secs += $off;

  my $year   = floor($secs/$yr);
  my $month  = (floor($secs           /$mnth))%12 + 1;
  my $day    = floor($secs%$mnth     /(24*60*60)) + 1;
  my $hour   = floor($secs%(24*60*60)/(60*60));
  my $minute = floor($secs%(60*60)   /(60));
  my $second = floor($secs%(60));

  # is it significantly after rollover, but before the next "day"?
  # if so, advance day/month in a really inefficient way
  if($hour == 23 && $minute > 40) {
    $day++;
    if($day == 9) {
      $day = 1;
      $month++;
      if($month == 13) {
	$month = 1;
	$year++;
      }
    }
  }
  
  return ($second, $minute, $hour,$day,$month,$year);
}

# returns epoch time of next rollover
sub next_rollover {
  my $secs = shift;

#  my ($sec,$min,$hour,$mday,$mon,$year) = gmtime($secs);
#  my ($second, $minute, $hour,$day,$month) = koltime($secs);

  return ($rollover_hour * 3600 + $rollover_minute * 60 - $secs)%(24*3600);
#   my $h = ($rollover_hour-$hour - ($rollover_minute < $min))%24;
#   my $m = ($rollover_minute - $min)%60;

#   print "AD $h $m\n";

#   $secs = 60*$m + 3600 * $h - $sec;

#   return $secs;
}
  

# This gives "Message Time", should be offloaded to KMail
sub kmailtime {
  my $secs = shift;

  return gmtime($secs + $kmail_offset);
}

my @substitutions = (['\bis\b','be'],
                  ['\b(([Ii]t)|([Tt]here))\'s\b','$1 be'],
                  ['\. ','. ','. ','. Avast, '],
                  ['^','','',
                   'Shiver me timbers. ','Yar. ',
                   'Avast, me hearties. '],
                  ['$','','',
                   ' Yo ho ho and a bottle of rum.', ' Aarr!'],
                  ['\bYes\b','Aye'], ['\byes\b','aye'],
                  ['\bbeer\b','grog'],
                  ['\bHello\b','Ahoy'],['\bhello\b','ahoy'],
                  ['\bfriend'],['shipmate'],
                  ['\bmy\b','me']);

sub filter_text {
  my ($text_ref) = shift;
  foreach my $s (@substitutions) {
    $$text_ref =~ s/$s->[0]/'"' . $s->[1+int(rand ($#{$s}))] . '"'/eeg;
  }
}

sub storage {
  my $bot = shift;
  my $params = { action => '',	# take,takemeat,
		 amt => 0,
		 @_ };

  my $response;

  my $url = $bot->url . 'storage.php';
  my $args =[
	     pwd       => $bot->{hash},
	     ];

  if($params->{action} eq 'takemeat') {
    push @$args, action    => $params->{action},
                 amt       => $params->{amt};
    $response = $bot->ua->post($url,$args);
  } elsif ($params->{action} eq 'take') {
    push @$args, action    => $params->{action};
    for(0..$#{$params->{number}}) {
      push @$args, 'howmany'.(($_)+1) ,$params->{number}->[$_];
      push @$args, 'whichitem'.(($_)+1) ,$params->{item_id}->[$_];
    }
    $response = $bot->ua->post($url,$args);
  } elsif ($params->{action} eq '') {
    $response = $bot->ua->get($url);
  } else {
    die "bad action passed to storage.";
  }

  $bot->check_response($response,'storage');

  # Parse the response
  $response->content =~ 
  m{<option value=0>-select an item-</option><option (.*?)</option></select>}i;
  my $content1 = $1;
  my @i1 = split m{</option><option },$content1;
  my $items = [];
  
  for (@i1) {
    m{value='(\d+)'>(.*?) \((\d+)\)}
      or die "Not an item: $_";

    my $id = $1;
    my $name = $2;
    my $number = $3;

    push @$items,new KoLBot::Item(desc => 0, name => $name, id => $id,
				  number => $number);
  }

  $response->content =~ 
#    m{<select name="whichitemtoview">.*?<option (.*)</option>.*?</select>}i
    m{<select name="whichitemtoview">[\s\n]*?<option (.*)</option>[\s\n]*?</select>}mi
    or die;
  $content1 = $1;
  @i1 = split m{</option><option },$content1;
  my $i = 0;

  for (@i1) {
    m{value='(\d+)'>(.*?) \((\d+)\)}
      or die "Not an item: $_";

    my $desc = $1;
    my $name = $2;
    my $number = $3;

    if ($items->[$i]->name ne $name) {
      print "name mismatch, ",$items->[$i]->id," ",$items->[$i]->{name}," ",
      $name,"\n";
      die;
    }
    if ($items->[$i]->{number} ne $number) {
      print "number mismatch, ",$items->[$i]->id," ",$items->[$i]->{number},
      " ",$number,"\n";
      die;
    }
    $items->[$i]->{desc} = $desc;
    $i++;
  }

  return $items;
}

#
# Store stuff
#

sub managestore {
  my $bot = shift;
  my %params = ( @_ );

  my $response;

  # AWM add additems
  if($params{action} eq 'additem') {
    my $url = $bot->url . 'managestore.php';
    my $args =
      [
       action    => $params{action},   # additem
       pwd       => $bot->{hash},
       ];
    my $i = 1;
    for(@{$params{items}}) {
      push @$args,('qty'.$i),$_->{number};
      push @$args,('item'.$i),$_->{id};
      push @$args,('price'.$i),$_->{price};
      push @$args,('limit'.$i),$_->{limit};
    }
    $response = $bot->ua->post($url,$args);
  } elsif ($params{action} eq 'take'
	   ||
	   $params{action} eq 'takeall'
	   ) {
    my $url = $bot->url . 'managestore.php?action=' . $params{action}
    . '&whichitem=' . $params{whichitem};
    $response = $bot->ua->get($url);
  } else {
    die "bad action passed to managestore.";
  }

  $response->is_success
    or die 'HTTP ' . $response->status_line;

  $bot->check_response($response,'managestore');

  return $response->content;
  # AWM check returns
}

sub store_log {
  my $bot = shift;

  # makes an array of hashes

  my $url = $bot->url . 'storelog.php';
  my $response = $bot->ua->get($url);
  my $ret = [];

  $bot->check_response($response,'store_log');

  my $content = $response->content;
  while ($content =~ m|(\d\d)/(\d\d)/(\d\d) (\d\d):(\d\d):(\d\d) <a class=nounder href="showplayer\.php\?who=(\d+)"><b>(.*?)</b></a> bought (\d+) \((.*?)\) for (\d+) Meat.|g) {
    # make a time, convert to UTC
    # As of Feb 11, 2006, the offset was 7 hours, meaning a good sync on the
    # DB server (and little variation between servers)
    # times don't respect your time zone setting
    my $time = timegm($6,$5,$4,$1,$2-1,100+$3) + 7*3600;

    my $ri =  $bot->items->get_item(name => $10,
				    number => 1);

    push @$ret,{time    => $time,
		user_id => $7,
		user    => $8,
		number  => $9,
		item    => $ri,
		price   => $11};
  }

  return $ret;
}

#
# Mallish stuff
#

sub search_mall {
  my $bot = shift;
  my $params = {@_};

  my $url = $bot->url . 'searchmall.php';
  my $args =
      [
       whichitem => $params->{whichitem},
       cheaponly => $params->{cheaponly},
       shownum   => $params->{shownum},
	];

# other type
#       whichstore   => $params->{whichstore},

  my $response = $bot->ua->post($url,$args);

  # Again, it might be useful to check for other failures

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'search_mall');

  die "SEARCH_DOWN" 
    if $response->content =~ /Mall searching is temporarily disabled\./i;

  die 'unknown search_mall, ',$response->content
    unless $response->content =~ m|<b>Search the Mall:</b>|;

  my $content = $response->content;
  $content =~ s/\n//g;
  $content =~ s/<td.*?>/<td>/g;

  # get rid of head and tail, keep only the juicy filling
  ( $content =~ 
      m|Price:</b></td></tr><tr><td><b>(.*?)&nbsp;Meat</td></tr>(</tbody>)?</table>|i )
#    or die 'unknown search_mall, ',$response->content;
    or return [];

  my $results = $1;
  $results =~ s/&nbsp;/ /g;

  my @lines = split m{ Meat</td></tR><tr><td><b>}i,$results;
  $results = [];

#  seal-clubbing club</b> (5) (1&nbsp;/&nbsp;day)</td><td><a  href="mallstore.php?whichstore=135934">You know you want it!</a></td><td>10 
  for (@lines) {
#    m|^(.*?)</b> \(([\d,]+)\)( \(([\d,]+) / day\))?</td><td><a\s+href="mallstore.php\?whichstore=(\d+).*?">(.*?)</a></td><td>([\d,]+)\s*$|
    m|^(.*?)</b> \(([\d,]+)\)( \(([\d,]+) / day\))?</td><td><a\s+(style='color: gray;'\s+)?href="mallstore.php\?whichstore=(\d+)&searchitem=(\d+).*?">(.*?)</a></td><td>([\d,]+)\s*$|
      or die "Not a line: $_";

    my $r = { name => $1,
	      number => $2,
	      limit => $4 || 0,
	      grayed => defined($5),
	      store => $6,
	      item_id => $7,
	      price => $9};

      $r->{price} =~ s/,//g;
      $r->{number} =~ s/,//g;
      $r->{limit} =~ s/,//g;

    push @$results, $r;
    }

  return $results;
}

sub mallstore {
  my $bot = shift;
  my $params = {quantity => 1,
		@_};

  my $url = $bot->url . 'mallstore.php';
  my $args =
      [
       whichstore => $params->{whichstore},
       buying    => 'Yep.',
       whichitem => $params->{whichitem},
       quantity  => $params->{quantity},
       pwd       => $bot->{hash},
	];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'mallstore');

  return $response->content;
}

#
# Misc
#

sub trickortreat {
  my $bot = shift;
  my $params = {quantity => 1,
		@_};

  my $url = $bot->url . 'trickortreat.php';
  my $args =
      [
       action => 'Yep.',
	];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'trickortreat');

  return $bot->parse_response($response->content);
}

#
# Utility
#

sub id_from_name {
  my $bot = shift;
  my $name = shift;

  my $STALE_TIME = 59*24*3600;

  my ($sql,$dbi,$rv);
  my $USER_TABLE = $bot->config->{db_conf}->{table_pre} . "users";

  $name =~ s/^\s+//;
  $name =~ s/\s+$//;

  # already a number.  Yes there is one exception, but....
  if($name =~ /^\d+$/) {
    my $id = $name;
    my $entry = 0;
    $sql = qq{
      SELECT name,last_whois,able
	FROM $USER_TABLE
	WHERE user_id = ?
      };
    $dbi = $bot->dbh->prepare($sql);
    $rv = $dbi->execute(
			$name,
			);
    if($rv > 0) {
      $entry = 1;
      my $row = $dbi->fetchrow_hashref();
      my $nameret = $row->{name};
      return ($id,$nameret) 
	if $row->{able} && time() - $row->{last_whois} < $STALE_TIME;
    }

    # No entries, expired, or !exists.  Try a /whois
    my $ret = $bot->send_message_raw('/whois ' . $name,1);
    if($ret =~ /green\;\">(.*?) \(\#(\d+)\)/) {
      my $nameret = $1;
      my $id = $2;

      if(!$entry) {
	$sql = qq{
	  INSERT INTO $USER_TABLE (user_id,name,bans,last_whois,able)
	    VALUES (?,?,0,?,1)
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute($id,$nameret,time());
	return ($id,$nameret);
      } else {
	$sql = qq{
	  UPDATE $USER_TABLE SET name = ?,last_whois = ?,able = 1
	    WHERE user_id = ?
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute($nameret,time(),$id);
	return ($id,$nameret);
      }
    }
    print "AWM: User ID not found $id\n";
    if($entry) {
      $sql = qq{
	UPDATE $USER_TABLE SET last_whois = ?,able = 0
	  WHERE user_id = ?
	};
      $dbi = $bot->dbh->prepare($sql);
      $rv = $dbi->execute(time(),$id);
    }
    return;
  }

  # not a number, alphanumeric, presume it is a user name
  my $name1 = $name;
  $name1 =~ s/(\w{20}) /$1/;

  $sql = qq{
    SELECT name,user_id,last_whois,able FROM $USER_TABLE
      WHERE LOWER(name) = LOWER(?) OR LOWER(name) = LOWER(?)
      ORDER BY user_id DESC
    };
  $dbi = $bot->dbh->prepare($sql);
  $rv = $dbi->execute(
		      $name,
		      $name1,
		      );

  my %rows;
  if($rv) {
    while(my $row = $dbi->fetchrow_hashref()) {
      $rows{$row->{user_id}} = $row;
    }
  }

  my $rname;
  my $rid;

  # update any entries necessary
  for(sort {$b <=> $a} keys %rows) {
    # ", user_id = ", $rows{$_}->{user_id}, " name = ",$rows{$_}->{name},"\n";
    if(time() - $rows{$_}->{last_whois} > $STALE_TIME
       &&
       $rows{$_}->{able}) {
      my $ret = $bot->send_message_raw('/whois ' . $rows{$_}->{user_id},1);
      if($ret =~ /green\;\">(.*?) \(\#(\d+)\)/) {
	# still exists
	$sql = qq{
	  UPDATE $USER_TABLE SET last_whois = ?
	    WHERE user_id = ?
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute(time(),$rows{$_}->{user_id});

	$rows{$_}->{last_whois} = time();
      } else {
	# gone
	$sql = qq{
	  UPDATE $USER_TABLE SET last_whois = ?,able = 0
	    WHERE user_id = ?
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute(time(),$rows{$_}->{user_id});

	$rows{$_}->{able} = 1;
      }
    }
    if(!defined($rid) && $rows{$_}->{able}) {
      $rid = $rows{$_}->{user_id};
      $rname = $rows{$_}->{name};
    }
  }
  return ($rid,$rname) if defined($rid);

  # no name found, or thought not to exist

  my $ret = $bot->send_message_raw('/whois ' . $name,1);
  if($ret =~ /green\;\">(.*?) \(\#(\d+)\)/) {
    $rname = $1;
    $rid = $2;

    if(exists $rows{$rid}) {
      $sql = qq{
	UPDATE $USER_TABLE SET last_whois = ?,able = 1
	  WHERE user_id = ?
	};
      $dbi = $bot->dbh->prepare($sql);
      $rv = $dbi->execute(time(),$rid);
    } else {
      $sql = qq{
	INSERT INTO $USER_TABLE (user_id,name,bans,last_whois,able)
	  VALUES (?,?,0,?,1)
	};
      $dbi = $bot->dbh->prepare($sql);
      $rv = $dbi->execute($rid,$rname,time());
    }
    return ($rid,$rname);
  }

  if(!($name eq $name1)) {
    my $ret = $bot->send_message_raw('/whois ' . $name1,1);
    if($ret =~ /green\;\">(.*?) \(\#(\d+)\)/) {
      $rname = $1;
      $rid = $2;
      
      if(exists $rows{$rid}) {
	$sql = qq{
	  UPDATE $USER_TABLE SET last_whois = ?,able = 1
	    WHERE user_id = ?
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute(time(),$rid);
      } else {
	$sql = qq{
	  INSERT INTO $USER_TABLE (user_id,name,bans,last_whois,able)
	    VALUES (?,?,0,?,1)
	  };
	$dbi = $bot->dbh->prepare($sql);
	$rv = $dbi->execute($rid,$rname,time());
      }
      return ($rid,$rname);
    }
  }

  print "AWM: name not found, $name\n";
  return;
}

##############################################################################
# Stuff.  I should clean this up.
##############################################################################

sub questlog {
  my $bot = shift;

  my $url = $bot->url . 'questlog.php?which=1';
  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'questlog 1');

  my $ret = {};

  my $content = $response->content;
  while($content =~ m|<b>(.*?)</b>|gi) {
    local $_ = $1;
    /your quest log/i and next;
    /council quests/i and next;
    /guild quests/i and next;
    /miscellaneous quests/i and next;
  
    /looking for a larva/i and $ret->{mosquito} = 1,next;
    /smell a rat/i and $ret->{rat} = 1,next;
    /smell a bat/i and $ret->{bat} = 1,next;
    /wouldn\'t be king/i and $ret->{king} = 1,next;
    /trial by friar/i and $ret->{friars} = 1,next;
    /Cyrptic Emanations/i and $ret->{cyrpt} = 1,next;
    /trapper\'s keeper/i and $ret->{trapz0r} = 1,next;
    /quest, lol/i and $ret->{baron} = 1,next;
    /is mainly garbage/i and $ret->{garbage} = 1,next;

    if(/Holy MacGuffin/) {
      $content =~ /find the Black Market/
	and $ret->{macguffin} = 'black_market',next;
      $content =~ /hit the Travel Agency/
	and $ret->{macguffin} = 'travel_agency',next;
      $content =~ /complicated\. Oh dear\./
	and $ret->{macguffin} = 'complicated',next;
      die "Unknown MacGuffin state";
    }
    if(/never odd or even/i) {
      $content =~ /first step is to get into the Palindome/i
	and $ret->{macguffin_palindome} = 'pirates',next;
      $content =~ /you\'ve discovered the fabulous Palindome/i
	and $ret->{macguffin_palindome} = 'palindome',next;
      $content =~ /track down this Mr\. Alarm guy/i
	and $ret->{macguffin_palindome} = 'mr_alarm',next;
      $content =~ /wants some wet stew in return/i
	and $ret->{macguffin_palindome} = 'wet_stew',next;
      $content =~ /pain to Dr. Awkward/i
	and $ret->{macguffin_palindome} = 'dr_awkward',next;
      die "unknown quest state";
    }
    if(/gotta worship them all/i) {
      $content =~ /the hidden temple/
	and $ret->{macguffin_temple} = 'hidden_temple',next;
      $content =~ /one of the Hidden Temple/
	and $ret->{macguffin_temple} = 'hidden_temple',next;
      $content =~ /no hollaback/
	and $ret->{macguffin_temple} = 'hidden_temple',next;
      $content =~ /Awesome, you.ve evaded all of the temple/
	and $ret->{macguffin_temple} = 'hidden_city',next;
      die "unknown  quest state";
    }
    if(/In a Manor of Spooking/i) {
      $content =~ /you figure spookyraven manor is/i
	and $ret->{macguffin_spookyraven} = 'begin',next;
      # You've unlocked the wine cellar
      $content =~ /unlocked the wine cellar/i
	and $ret->{macguffin_spookyraven} = 'wine_cellar',next;
      die "unknown  quest state";
    }
    if(/pyramid scheme/i) {
      $content =~ /hope you\'ve got your walking shoes on/
	and $ret->{macguffin_pyramid} = 'start',next;
      $content =~ /managed to stumble upon a hidden oasis/i
	and $ret->{macguffin_pyramid} = 'oasis',next;
      $content =~ /finding a stone rose/i
	and $ret->{macguffin_pyramid} = 'stone_rose',next;
      $content =~ /satisfied with the tasks you performed/i
	and $ret->{macguffin_pyramid} = 'come_later',next;
      $content =~ /vibration to summon sandworms/i
	and $ret->{macguffin_pyramid} = 'thumper',next;
      $content =~ /fifteen missing pages/i
	and $ret->{macguffin_pyramid} = 'missing_pages_15',next;
      $content =~ /fourteen to go/i
	and $ret->{macguffin_pyramid} = 'missing_pages_14',next;
      $content =~ /thirteen to go/i
	and $ret->{macguffin_pyramid} = 'missing_pages_13',next;
      $content =~ /found all of gnasir/i
	and $ret->{macguffin_pyramid} = 'missing_pages_done',next;
      $content =~ /earned your hooks/i
	and $ret->{macguffin_pyramid} = 'earned_hooks',next;
      $content =~ /found the little pyramid/
	and $ret->{macguffin_pyramid} = 'found_pyramid',next;
      $content =~ /found the hidden buried/
	and $ret->{macguffin_pyramid} = 'found_big_pyramid',next;

      die "unknown macguffing_pyramid quest state";
    }
    if(/make war, not/i) {
      $content =~ /if you can\'t stir up some trouble/
	and $ret->{war} = 'start_war',next;

      die "unknown quest state";
    }

    /The Final Ultimate Epic Final Conflict/i 
      and $ret->{sorceress} = 1,next;

    # guild quests
    /my other car is made of meat/i and $ret->{meatcar} = 1,next;
    /the wizard of ego/i and $ret->{fernswarthy} = 1,next;
    if(/me and my nemesis/i) {
      $content =~ /defeat Beelzebozo/
	and $ret->{nemesis} = 'beelzebozo',next;
      $content =~ /have to wait for your Guild/
	and $ret->{nemesis} = 'wait',next;
      $content =~ /Epic Weapon/
	and $ret->{nemesis} = 'weapon',next;
      die "unknown quest state: me and my nemesis";
    }
# You've been charged by your Guild (sort of) with the task of bringing back a delicious meal from the legendary White Citadel. You've been told it's somewhere near Whitey's Grove, in the Distant Woods.
    /go to white citadel/i and $ret->{citadel} = 1,next;

    # misc quests
    /this is Azazel in Hell/i and $ret->{azazel} = 1,next;
    /hammer time/i and $ret->{hammer_time} = 1,next;
    /baker, baker/i and $ret->{baker_baker} = 1,next;
    /when rocks attack/i and $ret->{rocks_attack} = 1,next;
    /what\'s up, doc/i and $ret->{galaktik} = 1,next;
    /driven crazy/i and $ret->{untinker} = 1,next;
    /^Toot!$/i and $ret->{toot} = 1,next;

    /save uncle crimbo/i and $ret->{uncle_crimbo} = 1,next;
    /when rocks attack/i and $ret->{rocks_attack} = 1,next;

    # not really a quest, part of make war description
    /^big$/ and next;

    die "Can't tell quest $1";
  }

  if($ret->{trapz0r}) {
    my $url = $bot->url . 'trapper.php';
    my $response = $bot->ua->get($url);

    $bot->check_response($response,'questlog trapper');
    $content = $response->content;
    $content =~ /I reckon 3 chunks of (chrome|linoleum|asbestos) ore/i
      and $ret->{trapz0r_ore} = $1;
    $content =~ /me back 6 chunks of goat cheese/i
      and $ret->{trapz0r_goat} = $1;
  }

  return $ret;
}

sub pyramid {
  my $bot = shift;

  my $url = $bot->url . 'pyramid.php';

  my $response = $bot->ua->get($url);
  $bot->check_response($response,'pyramid');
  my $content = $response->content;

  my $state = {};

  if($content =~ /pyramid4_(\d+)\.gif/) {
    return $1;
  } else {
    print "$content\n";
    die "unrecognized pyramid state";
  }
}

sub manor3goblet {
  my $bot = shift;

  my $url = $bot->url . 'manor3.php?place=goblet';
  my $response = $bot->ua->get($url);
  $bot->check_response($response,'manorgoblet');
  my $content = $response->content;
  return if $content =~ /There\'s nothing interesting to look at here./;
  my @glyphs;
  while($content =~ /alt="Arcane Glyph \#(\d+)"/g) {
    push @glyphs,$1;
  }
  return @glyphs;
}

sub hermit {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'hermit.php';

  my $args =
      [
       pwd => $bot->{hash},
       whichitem => $params{item},
       quantity => $params{quantity},
       action => 'trade',
       ];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'hermit');
  return $bot->parse_response($response->content);
}

sub sewer {
  my $bot = shift;

  my $url = $bot->url . 'sewer.php';
  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'sewer');
  return $bot->parse_response($response->content);
}

## AWM: untinkerall (checkbox);
sub untinker {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'town_right.php';
  
  my $response;
  if(exists($params{item})) {
    my $args =
      [
       pwd => $bot->{hash},
       whichitem => $params{item},
       action => 'untinker',
       ];

    $response = $bot->ua->post($url,$args);
  } else {
    $response = $bot->ua->get($url . '?place=untinker');
  }
  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'untinker');
  return $bot->parse_response($response->content);
}

sub bounty_hunter {
  my $bot = shift;
  my %params = @_;

  my $url = $bot->url . 'bhh.php';
  my $response;
  if(defined $params{action}) {
    if($params{action} eq 'takebounty') {
      my $args = [
		  pwd => $bot->{hash},
		  action => $params{action},
		  whichitem => $params{whichitem},
		  ];
      delete($bot->ua->{max_size});
      $response = $bot->ua->post($url,$args);
    } else {
      die "KoLBot::bounty_hunter, unknown action " . $params{action};
    }
  } else {
    $response = $bot->ua->get($url);
  }
  my %ret;

  $bot->check_response($response,'bounty_hunter');

  my $content = $response->content;

  my $rv = 0;
  my $txt = 0;
  my $resp = {};

  # we're looking for something
  if($content =~ /<input type=submit class=button value="I Give Up!">/) {
    $content =~ /I\'m still waiting for you to bring me (\d+) (.*?), Bounty Hunter!  You can get them from (.*?)<p>/
      or die "KoLBot::bounty_hunter, looking for something, what?";
    $ret{looking} = 1;
    $ret{number} = $1;
    $ret{item} = $2;
    $ret{where} = $3;
  } elsif ($content =~ /Excellent work, Bounty Hunter!  In exchange for/) {
    ($rv,$txt,$resp) = $bot->parse_response($content);
    $resp->{2098} = 1;		# doesn't get parsed
    $ret{done} = 1;
  } elsif ($content =~ /Since you\'ve already turned in a Bounty today,/) {
    $ret{done} = 1;
  } else {
    # find choices
    $ret{choices} = [];
    while ($content =~ m|<td valign=center><b>(\d+) (.*?)</b> from (.*?)&nbsp;&nbsp;</td><form action=bhh.php method=post><input type=hidden name=pwd value=[\dabcdef]+><input type=hidden name=action value="takebounty"><input type=hidden name=whichitem value=(\d+)><td>|g) {
      my $i =  $bot->items->get_item(name => $2,id => $4,number => $1);
      $i ||= {name => $2,id => $4,number => $1};
      $i->{from} = $3;
      push @{$ret{choices}},$i;
    }
    if(@{$ret{choices}} < 1) {
      print $content,"\n";
      die "I thought there should be choices.";
    }
  }
  return ($rv,$txt,$resp,\%ret);
}

sub knoll {
  my $bot = shift;
  my %params = (action => 'smith',
		@_);

  my $url = $bot->url . 'knoll.php';
  my $args =
      [
       pwd => $bot->{hash},
       action => $params{action},
       item1 => $params{item1},
       item2 => $params{item2},
       quantity => $params{quantity},
       ];

  my $response = $bot->ua->post($url,$args);

  $bot->check_response($response,'knoll');

  return $bot->parse_response($response->content);
}

sub galaktik {
  my $bot = shift;
  my %params = (@_);

  my $url = $bot->url . 'galaktik.php';

  my $response;
  if(exists($params{whichitem})) {
    my $args =
      [
       pwd => $bot->{hash},
       whichitem => $params{whichitem},
       howmany => $params{howmany},
       action => 'buyitem',
       ];

    $response = $bot->ua->post($url,$args);
  } elsif(exists($params{curemp})) {
    my $args =
      [
       pwd => $bot->{hash},
       quantity => $params{quantity},
       action => 'curemp',
       ];
    $response = $bot->ua->post($url,$args);
  } else {
    $response = $bot->ua->get($url);
  }

  $bot->check_response($response,'galaktik');

  my ($rv,$ret,$resp) = $bot->parse_response($response->content);

  my $items = {};

  my $content = $response->content;

  # of course this is slightly different from the store() code.
  while ($content =~ m{<input type=radio name=whichitem value=(\d+)><td><img src="[^\"]*" class=hand onClick='descitem\((\d+)\)'></td><td valign=center><a onClick='javascript:item\(\d+\)'><b>(.*?)</b></a></td><td>(\d+) Meat}g) {
    $items->{$1} = {id => $1,desc => $2,name => $3,cost => $4};
  }

  return ($rv,$ret,$resp,$items);
}

sub store {
  my $bot = shift;
  my %params = (@_);

  my $url = $bot->url . 'store.php';
  my $response;
  unless (exists($params{whichitem})) {
    $response = $bot->ua->get($url);
  } else {
    my $args =
      [
       phash => $bot->{hash},
       whichstore => $params{whichstore}, # 'm' is demon market
       buying => 'Yep.',
       whichitem => $params{whichitem},
       howmany => $params{howmany},
       ];
    $response = $bot->ua->post($url,$args);
  }

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'store');

  my ($rv,$txt,$resp) = $bot->parse_response($response->content);

  # Get items at store
  my $items = {};

  my $content = $response->content;
  while ($content =~ m{<input type=radio name=whichitem value=(\d+)></td><td><img src="[^\"]*" class=hand onClick='javascript:descitem\((\d+)\)'></td><td valign=center><a onClick='javascript:descitem\(\d+\)'><b>(.*?)</b></a></td><td>(\d+) Meat}g) {
    $items->{$1} = {id => $1,desc => $2,name => $3,cost => $4};
  }

  return ($rv,$txt,$resp,$items);
}

sub bigisland {
  my $bot = shift;
  my %params = (@_);

  my $url = $bot->url . 'bigisland.php';
  my $response;
  if(exists $params{action}) {
    $url .= "?action=" . $params{action} . "&pwd=" . $bot->{hash};
  } elsif (exists $params{place}) {
    $url .= "?place=" . $params{place};
  }

  $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'bigisland');

  my ($rv,$txt,$resp) = $bot->parse_response($response->content);

  # parse other results

  my $content = $response->content;
  if($params{action} eq 'junkman') {
    if($content =~ /The last time I saw my (hammer|wrench|pliers|screwdriver), (it was|they were)\s+(next to that barrel with something burning in it|over where the old tires are|out by that rusted-out car|near an abandoned refrigerator)\.\s+Maybe you should/i) {
      $resp->{junkman_item} = $1;

      if($3 =~ 'old tires')       { $resp->{junkman_subsnarfblat} = 3; }
      elsif($3 =~ 'rusted-out')   { $resp->{junkman_subsnarfblat} = 4; }
      elsif($3 =~ 'refrigerator') { $resp->{junkman_subsnarfblat} = 2; }
      elsif($3 =~ 'barrel')       { $resp->{junkman_subsnarfblat} = 1; }
      else { die "Don't know junkyard area";}
    }
    else { 
      $content =~ /<b>Yossarian, the Junkman<\/b>(.*?)<b>The Junkyard<\/b>/i;
      print "AWM DEBUG Yossarian debug, $1\n";
      die;
    }
  }
  return ($rv,$txt,$resp);
}

sub ascensionhistory {
  my $bot = shift;
  my $params = {@_};

  my $url = $bot->url . 'ascensionhistory.php?back=other&who='
    . $params->{id};

  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'ascensionhistory');

  my $content1 = $response->content;

  $url = $bot->url . 'ascensionhistory.php?back=other&prens13=1&who='
    . $params->{id};

  $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'ascensionhistory');

  my $content2 = $response->content;

  my $ret = [];
  for my $s ((
	      {era => 11,content => $content2},
	    {era => 13,content => $content1},
	      )) {
    my $content = $s->{content};
    $content =~ s/\&nbsp;/ /g;
    $content =~ s/\s+/ /g;
    $content =~ s/ ?<td.*?>/<td>/g;
    $content =~ s/ ?<\/td>/<\/td>/g;
    $content =~ s/(\d),(\d)/$1$2/g;
    $content =~ s/<img[^<]*?alt="([^\"<]*?)"[^<]*?>/ $1 /g;
    $content =~ s/<img[^<]*?spacer.gif[^<]*?>/ Normal /g;
    $content =~ s/\s+/ /g;
    $content =~ s/<span[^>]*?>//g;
    $content =~ s/<\/span>//g;

    while($content =~ m{<td>(\d+)</td><td>(\d+)/(\d+)/(\d+)</td><td>(\d+)</td><td> (Sauceror|Turtle Tamer|Pastamancer|Disco Bandit|Accordion Thief|Seal Clubber) </td><td>(.*?)</td><td>(\d+)</td><td>(\d+)</td><td>(.*?)</td><td> (.*?) </td></tr>}g) {
# </tr><td>68</td><td>07/28/07</td><td>30</td><td> Sauceror </td><td>Opossum</td><td><span title='Total Turns: 11882'>11882</span></td><td><span title='Total Days: 119'>119</span></td><td> Hovering Sombrero (51.2%) - Total Run: Hovering Sombrero (51.2%) </td><td> Hardcore Teetotaler </td></tr>


      my %class_sub = ('Seal Clubber' => 'SC',
		       'Turtle Tamer' => 'TT',
		       'Pastamancer'  => 'P',
		       'Sauceror'     => 'S',
		       'Disco Bandit' => 'DB',
		       'Accordion Thief' => 'AT');

      my $e = {number => $1,
	       date   => "$2/$3/$4",
	       level  => $5,
	       class  => $class_sub{$6},
	       sign   => $7,
	       turns  => $8,
	       days   => $9,
	       familiar => $10,
	       details => $11,
	       era    => $s->{era},
	     };
      
      $e->{details} =~ /Boozetafarian|Boozetefarian|Oxygenarian|Teetotaler/
	or $e->{details} .= ' No Path';

      push @$ret,$e;
    }
  }

  return $ret;
}

sub raffle {
  my $bot = shift;

  my $url = $bot->url . 'raffle.php';

  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'raffle');

  my $results = {};

  $response->content =~  m{First Prize:</td><td><img class=hand onclick='descitem\((\d+)\);' alt='\w+?' style='vertical-align: middle' src='http://images.kingdomofloathing.com/itemimages/\w+?\.gif'> <b>(.*?)</b>}i
  and $results->{first} = $2;
  $response->content =~  m{Second Prize:</td><td><img class=hand onclick='descitem\((\d+)\);' alt='\w+?' style='vertical-align: middle' src='http://images.kingdomofloathing.com/itemimages/\w+?\.gif'> <b>(.*?)</b>}i
  and $results->{second} = $2;

  return (0,'',$results);
}

sub arena {
  my $bot = shift;

  my $url = $bot->url . 'arena.php';

  my $response = $bot->ua->get($url);

  $response->is_success or die "HTTP " . $response->status_line;

  $bot->check_response($response,'arena');

  my $results = {familiars => []};

  my $content = $response->content;
  while($content =~ m{<input.*?value=(\d+)></td><td valign=center><img src="http://images.kingdomofloathing.com/itemimages/familiar(\d+).gif" width=30 height=30></td><td class=small><b>(.*?)</b> the (.*?)<br>(\d+) lbs., [\d,]+ wins, [\d,]+ losses.</td>}g) {
    push @{$results->{familiars}},{ id => $1,
				    name => $3,
				    type => $4,
				    weight => $5};
  }
  return (0,'',$results);
}    
    
  
# eP2
# Return the collection for a given player by player ID
sub displaycollection {
  my $bot    = shift;
  my $params = {@_};
  my $args   = [
		who => $params->{who},
		];
  
  # Make the call
  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'displaycollection.php';
  my $response = $bot->ua->post($url,$args);
  
  # Get the response
  $response->is_success or die 'HTTP ' . $response->status_line;
  
  # See if we're cool
  $bot->check_response($response,'displaycollection');
  
  # Create a data structure to return
  my $displaycase = {
    user        => '',
    user_id     => '',
    description => '',
    shelves     => [],
  };
  
  # Save the content
  my $content = $response->content;
  my @content = split (/<tr>/i, $content);
  
  # Placeholder for new shelf data
  my $shelf = '';
  
  # Parse the content
  while ($content = shift (@content)) {
  # Is this display case empty?
    if ($content =~ m!<p><center><b>This display case is currently empty.</b></center><p><center>!m) {
      last;

    } elsif ($content =~ m!^<td style="color: white;" align=center bgcolor=blue><b>Display Case \(<a style='color: white; text-decoration: none;' href="showplayer.php\?who=(\d+)">(.*)</a>\)</b></td></tr>!m) {
    # Can we snag the player name out of here, to prevent another DB hit?
      $displaycase->{user_id} = $1;
      $displaycase->{user}    = $2;
    } elsif ($content =~ m!^<td valign=center><img src="http://images.kingdomofloathing.com/otherimages/museum/displaycase.gif" width=100 height=100></td><td valign=center>(.*)</td></tr></table>!m) {

      # Is this the case description?
      $displaycase->{description} = $1;
    } elsif ($content =~ m!^<td style="color: white;" align=center bgcolor=blue><b>(.*)</b></td></tr>!m) {
      # Is this a new shelf?
      # Save the previous shelf if we were working on one already
      if ($shelf ne '') {
	push (@{$displaycase->{shelves}}, $shelf);
      }

      # Create a new shelf
      $shelf = { title => $1, item => {} };
    } elsif ($content =~ m!^<td width=30 height=30><img src="http://images\.kingdomofloathing\.com/itemimages/(.*)\.gif" class=hand onClick='descitem\((\d+)\)'></td><td valign=center><b>(.*?)</b>( \(([\d,]+)\))?!) {
      # Is this an item on a shelf?
      # Remember the matches
      my $item_desc = $2;
      my $quantity  = $5 ? $5 : 1;
  
      # Remove commas from quantity
      $quantity =~ s/,//g;
  
      # Store the record
      $shelf->{item}->{ $item_desc } = $quantity;
    }
  }

  # If we have a shelf left open, push it
  push (@{$displaycase->{shelves}}, $shelf) if $shelf ne '';

  # Return the display case structure
  return $displaycase;
}

# eP2
# Return the collection for a given player by player ID
# Returns a pointer to an hash of hashes with DescID, name, and quantity of
# item.  Ignores shelf structure
sub displaycollection_items {
  my $bot    = shift;
  my $params = {@_};
  my $args   = [
		who => $params->{who},
		];
  
  # Make the call
  $bot->ua->timeout($bot->config->{timeout});
  my $url = $bot->url . 'displaycollection.php';
  my $response = $bot->ua->post($url,$args);
  
  # Get the response
  $response->is_success or die 'HTTP ' . $response->status_line;
  
  # See if we're cool
  $bot->check_response($response,'displaycollection_items');
  
  # Create a data structure to return
  my $displaycase = {};
  
  # Save the content
  my $content = $response->content;
  my @content = split (/<tr>/i, $content);
  
  # Parse the content
  while ($content = shift (@content)) {
  # Is this display case empty?
    if ($content =~ m!<p><center><b>This display case is currently empty.</b></center><p><center>!m) {
      last;

    } elsif ($content =~ m!^<td style="color: white;" align=center bgcolor=blue><b>Display Case \(<a style='color: white; text-decoration: none;' href="showplayer.php\?who=(\d+)">(.*)</a>\)</b></td></tr>!m) {
    } elsif ($content =~ m!^<td valign=center><img src="http://images.kingdomofloathing.com/otherimages/museum/displaycase.gif" width=100 height=100></td><td valign=center>(.*)</td></tr></table>!m) {
    } elsif ($content =~ m!^<td style="color: white;" align=center bgcolor=blue><b>(.*)</b></td></tr>!m) {
    } elsif ($content =~ m!^<td width=30 height=30><img src="http://images\.kingdomofloathing\.com/itemimages/(.*)\.gif" class=hand onClick='descitem\((\d+)\)'></td><td valign=center><b>(.*?)</b>( \(([\d,]+)\))?!) {
      # Is this an item on a shelf?
      # Remember the matches


      my $item_desc = $2;
      my $item_name = $3;
      my $quantity  = $5 ? $5 : 1;
      $quantity =~ s/,//g;
      $displaycase->{$2} = { desc => $item_desc,
			     name => $item_name,
			     number => $quantity };
  
    }
  }

  # Return the display case structure
  return $displaycase;
}

# eP2
# Searches for a player, returns a hash which maps player name into player ID
#
# Input params it can take, which are identical to the Search Player dialog box
#    searchstring  => substring of name
#    searchlevel   => level player
#    searchranking => maximum ranking
#    pvponly       => If set, limit to PvP players only
#    hardcoreonly  => If set, limit to hardcore players only

sub search_player {
  my $bot    = shift;
  my $args = [
	      searching => 'Yep',
	      @_
	      ];
  
  # Get the response to the query
  my $url = $bot->url . 'searchplayer.php';
  my $response = $bot->ua->post ($url, $args);
  
  # Search through the results, looking for players
  my $player = '';
  my $player_list = {};
  
  foreach $player (split (/showplayer\.php/, $response->content)) {
    # Look for player strings
    if ($player =~ /^\?who=(\d+)(\'|\")>(.*?)<\/a>/) {
      # Add the player to the response list
      $player_list->{$3} = $1;
    }
  }
  
  # Return the result
  return ($player_list);
}

# eP2
sub item_image_url {
  my $bot = shift;
  my $args = [
	      @_
	      ];
  
  my $url = $bot->url . 'desc_item.php';
  my $response = $bot->ua->post ($url, $args);
  my $content = $response->content;
  
  if($content =~ m!(http://images.kingdomofloathing.com/itemimages/.*\.gif)!) {
    return $1;
  } else {
    return '';
  }
}

# eP2
sub get_item_description {
  my $bot = shift;
  my $args = [
	      @_
	      ];
  
  # Get the description
  my $url = $bot->url . 'desc_item.php';
  my $response = $bot->ua->post ($url, $args);
  $bot->check_response($response,'get_item_description');
  my $content = $response->content;

  # Placeholder for the return value
  my $item = '';
  
  # First, see if we have a valid item at all
  return $item if $content =~ /No description data exists for this item\./;

  # Remove newlines
  $content =~ s/\s+/ /g;
  
  # Create a new hash ref for the item fields
  $item = {};
  
  # Save the entire content of this thing
  $item->{description} = $1 if $content =~ /<blockquote>(.*)<\/blockquote>/;
  
  # Get the image filename
  $item->{image_filename} = $1 if $content =~ /images\.kingdomofloathing\.com\/itemimages\/(.*?\.gif)/;
  
  # Get the name
  $item->{name} = $1 if $content =~ /<b>(.*?)<\/b><p><\/center><blockquote>/;
  
  # Is there a type for this thing?
  if ($content =~ /<br>Type: <b>(.*?)<\/b></) {
    # Hold onto this type
    my $type = $1;
    
    # If it's a weapon, pick it apart further
    if ($type =~ /^(ranged )?weapon \((\d+)-handed (.*)\)$/) {
      $item->{type}          = 'weapon';
      $item->{ranged_weapon} = $1 ? 1 : 0;
      $item->{weapon_hands}  = $2;
      $item->{weapon_type}   = $3;

    } elsif ($type =~ /^off-hand item (\((.*)\))?/) {
      # If it's an off-hand item, pick it apart further
      $item->{type} = "off-hand item";
      $item->{off_hand_type} = $2;
    } else {
      # Otherwise, just keep it the same type as defined
      $item->{type} = $type;
    }

    # Do other things with type
    $item->{usable} = ($type eq 'usable'      || $type eq 'combat / usable item') ? 1 : 0;
    $item->{combat} = ($type eq 'combat item' || $type eq 'combat / usable item') ? 1 : 0;
  }

  # Is there built-in (non-enchantment) damage reduction?
  if($content =~ /<br>Damage Reduction: <b>(\d+) per level<\/b></) {
    $item->{damage_reduction_per_level} = $1;
  } elsif ($content =~ /<br>Damage Reduction: <b>(\d+)<\/b></) {
    $item->{damage_reduction} = $1;
  }

  # Is there some power associated with this item?
  $item->{power} = $1 if $content =~ /<br>Power: <b>(\d+)<\/b></;

  # Is this item part of an outfit?
  $item->{outfit} = $1 if $content =~ /<br>Outfit: <b>(.*?)<\/b>/;

# Are there equipping requirements?
  $item->{muscle_required}      = $1 if $content =~ /<br>Muscle Required: <b>(\d+)<\/b>/;
  $item->{mysticality_required} = $1 if $content =~ /<br>Mysticality Required: <b>(\d+)<\/b>/;
  $item->{moxie_required}       = $1 if $content =~ /<br>Moxie Required: <b>(\d+)<\/b>/;
  $item->{level_required}       = $1 if $content =~ /<br>Level required: <b>(\d+)<\/b>/;

# Can we combine this in some special way?
  $item->{meatsmithing_component}      = $content =~ /<br>\(Meatsmithing component\)</      ? 1 : 0;
  $item->{cooking_ingredient}          = $content =~ /<br>\(Cooking ingredient\)</          ? 1 : 0;
  $item->{cocktailcrafting_ingredient} = $content =~ /<br>\(Cocktailcrafting ingredient\)</ ? 1 : 0;
  $item->{jewelrymaking_component}     = $content =~ /<br>\(Jewelrymaking component\)</     ? 1 : 0;
  $item->{gift_item}                   = $content =~ /<br><b>Gift Item<\/b>/                ? 1 : 0;

# Can we trade it or junk it?
  $item->{tradeable}   = $content =~ /<br>Cannot be traded\.?</  ? 0 : 1;
  $item->{discardable} = $content =~ /<br>Cannot be discarded</ ? 0 : 1;
  $item->{quest_item}  = $content =~ />Quest Item</             ? 1 : 0;

  $item->{enchantment} = $1 if
    $content =~ /<center>Enchantment:<br><b><font color=blue>(.*)<br>/;

  $item->{enchantment_plus} = $1 if
    $content =~ /<center>Enchantment:<br><b><font color=blue>(.*)/;

# Can we sell it?
  $item->{selling_price} = $1 if $content =~ /Selling Price: <b>(\d+) Meat\.</;

  return $item;
}
  
##############################################################################
# Derived functions
##############################################################################


sub make_scrumptious_reagent {
  my $bot = shift;
  my $number = shift || 1;

  return $bot->use_skill(skill => 4006,
			 castings=>$number,
			 check_response => 1);
}

sub make_dry_noodles {
  my $bot = shift;
  my $number = shift || 1;

  return $bot->use_skill(skill => 3006,
			 castings=>$number);
}


END {} ;

1;

